!     THIS FUNCTION RETURNS THE SQUARED MATRIX ELEMENT FOR A GIVEN
!     DIFFERENCE BETWEEN INITIAL AND FINAL K-VECTORS, Q, AND A GIVEN
!     SCATTERING MECHANISM, M. THE SQUARED MATRIX ELEMENT IS MULTIPLIED
!     BY THE FACTOR 2*PI/HBAR FROM FERMI'S GOLDEN RULE 

      FUNCTION MATEL(Q,M)
      USE Rateinit

      REAL(KIND=DBL) :: MATEL,Q
      INTEGER :: M
      LOGICAL :: FIRST
!     CHANGED FROM ORIGINAL PROGRAM, HALVORSEN ONLY SAVED 'FIRST'
      DATA FIRST/.TRUE./

      REAL(KIND=DBL) :: N,E
!     FUNCTION DETERMINING PHONON OCCUPATION NUMBER
!     THE FUNCTION IS WRITTEN IN THIS MORE COMPLICATED FORM IN
!     IN ORDER TO MAKE AN UNDERFLOW TO 0.0 WHEN EXPONENT IS BIG 
      N(E)=EXP(-E/(KB*T))/(1-EXP(-E/(KB*T)))
! 
!     MAIN PART 
!     IF IT IS THE FIRST CALL TO THE FUNCTION SOME EXPRESSIONS ARE
!     EVALUATED FIRST
      IF (FIRST) THEN
          FIRST=.FALSE.
          CPOLOP=PI*((EC**2)/HBAR)*OPTEN*EV*((1/EPSINF)-(1/EPSZER))/EPS0
          CAC=PI*D**2/(RHO*S)*EV
          BETSQR=((NIMP*EC/(EPS0*EPSZER))*EC)/(KB*T)
          CION=2*PI*(((NIMP*EC)*EC/(EPS0*EPSZER)**2)*EC)*EC/HBAR
          CNONOP=PI*DKSQR*EV/(RHO*OPTEN)
          CALLOY=2*PI*DV*DV*EV*EV*COMP*(1-COMP)/(NCAT*HBAR)
      ENDIF
      MATEL=0
      GOTO (111,112,113,114,115,116,117,118) M 
      RETURN
!     POLAR OPTICAL, ABSORPTION
111   MATEL=(N(OPTEN*EV)*CPOLOP)/(Q**2)
      RETURN
 
!     POLAR OPTICAL, EMISSION
112   MATEL=(N(OPTEN*EV)+1)*CPOLOP/Q**2
      RETURN
 
!     NONPOLAR OPTICAL, ABSORPTION
113   MATEL=CNONOP*(HBAR*N(OPTEN*EV))
      RETURN
 
!     NONPOLAR OPTICAL, EMISSION
114   MATEL=CNONOP*(HBAR*(N(OPTEN*EV)+1))
      RETURN
 
!     ACOUSTIC DEFORMATION POTENTIAL, ABSORPTION
115   IF (Q.EQ.0) THEN
          MATEL=(CAC*KB*T/(HBAR*S))*EV
      ELSE
          MATEL=(CAC*Q*N(HBAR*Q*S))*EV
      ENDIF
      RETURN

!     ACOUSTIC DEFORMATION POTENTIAL, EMISSION
116   IF (Q.EQ.0) THEN
          MATEL=(CAC*KB*T/(HBAR*S))*EV
      ELSE
          MATEL=(CAC*Q*(N(HBAR*Q*S)+1))*EV
      ENDIF
      RETURN
 
!     IONIZED IMPURITY SCATTERING
117   MATEL=(CION/(Q**2+BETSQR))/(Q**2+BETSQR)
      RETURN

!     ALLOY SCATTERING
118   MATEL=CALLOY
      RETURN
      END
 
 
      FUNCTION AREA(L1,L2,L3,W,B)
      USE Rateinit, ONLY: DBL
      REAL(KIND=DBL) :: AREA,L1,L2,L3,W,B,W1,W2,W3,W4

!     THIS FUNCTION TAKES THE DIRECTION COSINES OF THE UNIT
!     PLANE NORMAL, HALF THE SIDE LENGTH OF THE CUBE ,B, AND THE
!     DISTANCE OF THE PLANE FROM THE CUBE CENTRE, W, TO CALCULATE
!     THE CROSS-SECTION AREA BETWEEN THE PLANE AND THE CUBE.
!     THE PARAMETERS MUST OBEY L1 >= L2 >= L3 >=0 AND W >= 0 .
!     NONE OF THE PARAMETERS ARE CHANGED.


!     HERE THE DISTANCES BETWEEN THE FOUR CORNERS OF THE CUBE AND 
!     THE PLANE CROSSING THROUGH THE CENTRE ARE COMPUTED
      W1 = B*ABS(L1-L2-L3)
      W2 = B*(L1-L2+L3)
      W3 = B*(L1+L2-L3)
      W4 = B*(L1+L2+L3)

!     HERE THE AREA IS COMPUTED
      IF (W .LE. W1) THEN
         IF (L1 .GE. L2+L3) THEN
            AREA=4*B**2/L1
         ELSE
            AREA=(2*B**2*(L1*L2+L2*L3+L1*L3)-W**2-B**2)/(L1*L2*L3)
         ENDIF
      ELSEIF (W .LE. W2) THEN
         AREA=(B**2*(3*L2*L3+L1*L2+L1*L3)-B*W*(L2+L3-L1)-(W**2+B**2)/2) &
           /(L1*L2*L3)
      ELSEIF (W .LE. W3) THEN
         AREA=2*(B**2*(L1+L2)-B*W)/(L1*L2)
      ELSEIF (W .LE. W4) THEN
         AREA=(B*(L1+L2+L3)-W)**2/(2*L1*L2*L3)
      ELSE
         AREA=0
      ENDIF
      END
 
 
!     THIS SUBROUTINE CALCULATES THE VECTOR FROM THE CENTRE OF THE
!     CUBE TO THE CENTROID OF THE VERTICES OF THE CROSS-SECTION AREA AT THE 
!     DISTANT W FROM THE CENTRE OF THE CUBE. C CONTAINS THE PLANE
!     NORMAL, AND B IS HALF THE SIDE LENGTH OF THE CUBE. THE SORTED
!     MAGNITUDES OF THE DIRECTION COSINES IN C IS RETURNED IN L.
!     THE VECTOR IS RETURNED IN DK
 
      SUBROUTINE CORREC(C,W,B,L1,L2,L3,DK)
      USE Rateinit, ONLY: DBL
      REAL(KIND=DBL) :: C(3),L(3),W,B,DK(3),L1,L2,L3,M(3)
      INTEGER        :: TI,I,J,INV(3)
      REAL(KIND=DBL) :: TR,W1,W2,W3,W4,S(3)
 
!     FIRST THE TRANSFORMATION INTO VALID VALUES OF THE DIRECTION 
!     COSINES ARE DONE (THESE ARE HELD IN L).
      DO I=1,3
          IF (C(I).GE.0) THEN 
              S(I)=1
          ELSE    
              S(I)=-1
          ENDIF
          L(I)=ABS(C(I))
          INV(I)=I
      END DO

      DO J=2,1,-1
         DO I=1,J
            IF (L(I+1).GT.L(I)) THEN
                TR=L(I)
                TI=INV(I)
                L(I)=L(I+1)
                INV(I)=INV(I+1)
                L(I+1)=TR
                INV(I+1)=TI
            ENDIF
         END DO
      END DO

      L1=L(1)
      L2=L(2)
      L3=L(3)

!     HERE THE DISTANCES BETWEEN THE FOUR CORNERS OF THE CUBE AND 
!     THE PLANE CROSSING THROUGH THE CENTRE ARE COMPUTED
      W1 = B*ABS(L(1)-L(2)-L(3))
      W2 = B*(L(1)-L(2)+L(3))
      W3 = B*(L(1)+L(2)-L(3))
      W4 = B*(L(1)+L(2)+L(3))
 
!     HERE THE VECTOR IS COMPUTED
      IF (W .LE. W1) THEN
          IF (L1 .GE. L2+L3) THEN
              M(1)=W/L(1)
              M(2)=0
              M(3)=0
          ELSE
              M(1)=W/(3*L(1))
              M(2)=W/(3*L(2))
              M(3)=W/(3*L(3))
          ENDIF
      ELSEIF (W .LE. W2) THEN
          M(1)=(3*W+B*(2*L(1)-L(2)-L(3)))/(5*L(1))
          M(2)=(W-B*(L(1)-L(3)))/(5*L(2))
          M(3)=(W-B*(L(1)-L(2)))/(5*L(3))
      ELSEIF (W .LE. W3) THEN
          M(1)=(W+B*(L(1)-L(2)))/(L(1)*2)
          M(2)=(W+B*(L(2)-L(1)))/(L(2)*2)
          M(3)=0
      ELSEIF (W .LE. W4) THEN
          M(1)=(W+B*(2*L(1)-L(2)-L(3)))/(3*L(1))
          M(2)=(W+B*(2*L(2)-L(1)-L(3)))/(3*L(2))
          M(3)=(W+B*(2*L(3)-L(1)-L(2)))/(3*L(3))
      ELSE
         WRITE (*,*) 'PLANE OUTSIDE CUBE!'
         STOP
      ENDIF
 
!     THEN THE VECTOR IS TRANSFORMED BACK INTO THE ORIGINAL SYSTEM OF
!     COORDINATES
      DO I=1,3
          DK(INV(I))=M(I)*S(INV(I))
      END DO

      END
 

!     THIS FUNCTION CALCULATES THE ENERGY DIFFERENCE BETWEEN
!     FINAL AND INITIAL STATES 
      FUNCTION EDELTA(M,Q)
      USE Rateinit
      REAL(KIND=DBL) :: EDELTA,Q
      INTEGER :: M
 
      GOTO (111,112,111,112,115,116,117) M
      RETURN
111   EDELTA=OPTEN
      RETURN
112   EDELTA=-OPTEN
      RETURN
115   EDELTA=(HBAR*S*Q)/EV
      RETURN
116   EDELTA=-(HBAR*S*Q)/EV
      RETURN
117   EDELTA=0
      END
 
!     THIS PROGRAM CALCULATES NUMERICALLY THE SCATTTERING RATES FOR THE
!     DIFFERENT SCATTERING MECHANISMS. IT IS BASED ON THE EXTRPOLATION
!     METHOD AFTER GILAT AND RAUBENHEIMER (ORIGINALLY ADOPTED TO 
!     SCATTERING RATE CALCULATIONS BY FISCHETTI AND LAUX)
 
      PROGRAM SCRATE
      USE Rateinit 

      LOGICAL :: OK,TEMP
 
!     MISCELLANEOUS "TECHNICAL" VARIABLES 
!     MNUM HOLDS THE MAXIMUM VALUE OF MESHNU, THAT IS THE NUMBER OF 
!     MESHPOINTS ALONG AN PRINCIPAL AXIS. THE AMOUNT OF MEMORY REQUIRED
!     BY THE PROGRAM IS TO A LARGE DEGREE DEPENDENT ON THE VALUE OF THIS
!     PARAMETER
 
      INTEGER            :: X,Y,Z,OFFSET,I,J,K
      INTEGER, PARAMETER :: MNUM=100,N=MS
      INTEGER            :: RES
      REAL(KIND=DBL)     :: EPS,AREA,TRATE,Q,MATEL,LGRAD,R,RATE,EDELTA
      COMPLEX(KIND=DBL)  :: G1,G2,G3,G4 
      INTEGER            :: ENLEN,GRLEN,VELEN

!     MISCELLANEOUS "TECHNICAL" VARIABLES CONCERNING INITIAL STATES
      INTEGER           :: IMESHN,XI,YI,ZI,IBASE
      INTEGER           :: ENI,VEI,GRI
      REAL(KIND=DBL)    :: IWAVMX,IMSIZE,EI,KXI,KYI,KZI
      COMPLEX(KIND=DBL) :: GI(N,2) 
      LOGICAL           :: IFINE

!     MISCELLANEOUS "TECHNICAL" VARIABLES CONCERNING FINAL STATES 
      INTEGER :: FMESHN(2),FBASE,M,FANTM
      INTEGER, PARAMETER :: MAXFIN=12*MNUM**2
      INTEGER :: FINALS(MAXFIN,3,2),ANTFIN(2),ENF,GRF,VEF
      REAL(KIND=DBL) :: FWAVMX(2),FMSIZE(2),EMIN(0:MNUM-1,0:MNUM-1,0:MNUM-1,2),&
           EMAX(0:MNUM-1,0:MNUM-1,0:MNUM-1,2),&
           EF,E(0:MNUM-1,0:MNUM-1,0:MNUM-1,2),&
           GRADF(3,0:MNUM-1,0:MNUM-1,0:MNUM-1,2),&
           KXF,KYF,KZF,DK(3),W,L1,L2,L3,C(3),CEFLIM,FEFLIM,DE
      COMPLEX(KIND=DBL) :: GF(N,2,0:MNUM-1,0:MNUM-1,0:MNUM-1,2) 
      LOGICAL :: FFINE

      DATA EPS/1D-10/
 
!     WRITE HEADING
      WRITE (*,*) '******************* SCRATES *******************'
      WRITE (*,*) 'Program for numerical calculation of scattering'
      WRITE (*,*) 'rates with discretized band structure.'
      write (*,*) ' '

!     INITIALIZING PROGRAM
 
!     SET APPROPRIATE RECORD LENGTH BY ASSIGNING COMPUTER TYPE TO ANT
      ANT=PC
      IF (MOD(8,ANT).NE.0) THEN
          ENLEN=8/ANT+1
      ELSE
          ENLEN=8/ANT
      ENDIF

      IF (MOD(24,ANT).NE.0) THEN
          GRLEN=24/ANT+1
      ELSE
          GRLEN=24/ANT
      ENDIF
      IF (MOD(32*MS,ANT).NE.0) THEN
          VELEN=32*MS/ANT+1
      ELSE
          VELEN=32*MS/ANT
      ENDIF
 
!     READ INITIALIZATION FILE
      IF (MOD(BYTES,ANT).NE.0) THEN
          LEN=BYTES/ANT+1
      ELSE
          LEN=BYTES/ANT
      ENDIF
 
      OPEN(10,FILE='SCRATES.INI',ACCESS='DIRECT',FORM='UNFORMATTED', &
           IOSTAT=ERROR,STATUS='OLD',RECL=LEN)
      CLOSE(10)
 
      IF (ERROR.NE.0) THEN
          WRITE (*,9910) 'ERROR READING INITIALIZATION FILE '// &
                         '"SCRATES.INI"' 
          STOP
      ENDIF
 
      OPEN (10,FILE='SCRATES.INI',ACCESS='DIRECT',FORM='UNFORMATTED', &
            STATUS='OLD',RECL=LEN)
      READ (10,REC=1) BANNUM,BNAM,ENFIL,GRFIL,VEFIL,INIT,FINAL, &
                      OUTFIL,T,MECH,OVMETH
      CLOSE (10)

!     CHECK INITIALIZATION DATA
      OK=.TRUE.
      INITOK=INIT.GE.1.AND.INIT.LE.BANNUM
      FINOK=FINAL.GE.1.AND.FINAL.LE.BANNUM
!     CHECK THAT SCATTERING MECHANISM IS OK
      IF(.NOT.(MECH.GE.1.AND.MECH.LE.SCRNUM)) THEN
          WRITE (*,9910) 'SCATTERING MECHANISM IS UNDEFINED.'
          OK=.FALSE.
      ENDIF
!     CHECK THAT INITIAL AND FINAL BANDS ARE OK
      INITOK = INIT.GE.1.AND.INIT.LE.BANNUM
      FINOK=FINAL.GE.1.AND.FINAL.LE.BANNUM
      IF (.NOT.INITOK) WRITE(*,9910) 'INITIAL BAND IS UNDEFINED'
      IF (.NOT.FINOK) WRITE(*,9910) 'FINAL BAND IS UNDEFINED'
!     CHECK THAT FILENAMES ARE OK
      IF (INIT.NE.FINAL) THEN
          TEMP=ENFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.TEMP) & 
              WRITE (*,9910) 'ENERGY FILE FOR INITIAL BAND IS UNDEFINED'
          TEMP=GRFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.TEMP) &
              WRITE (*,9910) 'GRADIENT FILE FOR INITIAL BAND IS '// &
                             'UNDEFINED'
          TEMP=VEFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.TEMP) &
              WRITE (*,9910) 'EIGENVECTOR FILE FOR INITIAL BAND IS ' &
                           //'UNDEFINED' 
          TEMP=ENFIL(FINAL).EQ.' ' 
          OK=OK.AND.(.NOT.TEMP)
          IF (FINOK.AND.TEMP) & 
              WRITE (*,9910) 'ENERGY FILE FOR FINAL BAND IS UNDEFINED'
          TEMP=GRFIL(FINAL).EQ.' '     
          OK=OK.AND.(.NOT.TEMP)
          IF (FINOK.AND.TEMP) &
              WRITE (*,9910) 'GRADIENT FILE FOR FINAL BAND IS UNDEFINED'
          TEMP= VEFIL(FINAL).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (FINOK.AND.TEMP) &
              WRITE (*,9910) 'EIGENVECTOR FILE FOR FINAL BAND IS ' &
                           //'UNDEFINED' 
      ELSE 
          TEMP=ENFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.ENFIL(INIT).EQ.' ') &
              WRITE (*,9910) 'ENERGY FILE FOR INITIAL AND FINAL ' &
                           //'BAND IS UNDEFINED' 
          TEMP=GRFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.GRFIL(INIT).EQ.' ') &
              WRITE (*,9910) 'GRADIENT FILE FOR INITIAL AND FINAL ' &
                           //'BAND IS UNDEFINED' 
          TEMP=VEFIL(INIT).EQ.' '
          OK=OK.AND.(.NOT.TEMP)
          IF (INITOK.AND.VEFIL(INIT).EQ.' ') &
              WRITE (*,9910) 'EIGENVECTOR FILE FOR INITIAL AND FINAL ' &
                           //'BAND IS UNDEFINED' 
          ENDIF
      TEMP=OUTFIL.EQ.' '
      OK=OK.AND.(.NOT.TEMP)
      IF (TEMP) WRITE (*,9910) 'OUTPUT FILE IS UNDEFINED'
      TEMP=OVMETH.GE.1.AND.OVMETH.LE.3
      IF (.NOT.TEMP) THEN
          WRITE (*,9910) 'CALCULATION OF OVERLAP INTEGRAL IS UNDEFINED'
      ENDIF
      OK=OK.AND.TEMP
      IF (.NOT.OK) STOP

!     GIVE INFORMATION ABOUT WHAT IS DONE
      WRITE (*,*) 'SCATTERING MECHANISM IS : ',SCRNAM(MECH)
      WRITE (*,*) 'TRANSITION FROM : ',BNAM(INIT)
      WRITE (*,*) '             TO : ',BNAM(FINAL)
      WRITE (*,*) 'AT TEMPERATURE : ',T,' K'
      IF (OVMETH.EQ.1) THEN
          WRITE (*,*) 'OVERLAP INTEGRAL IS CALCULATED FROM EIGENVECTORS'
      ELSEIF (OVMETH.EQ.2) THEN
          WRITE (*,*) 'OVERLAP INTEGRAL IS CALCULATED FROM FORMULAS'
      ELSE
          WRITE (*,*) 'OVERLAP INTEGRAL IS SET TO UNITY'
      ENDIF

      WRITE (*,*) 'INPUT FILES FOR INITIAL STATES ARE: '
      WRITE (*,*) ENFIL(INIT)
      WRITE (*,*) GRFIL(INIT)
      WRITE (*,*) VEFIL(INIT)
      WRITE (*,*) 'INPUT FILES FOR FINAL STATES ARE: '
      WRITE (*,*) ENFIL(FINAL)
      WRITE (*,*) GRFIL(FINAL)
      WRITE (*,*) VEFIL(FINAL)
      WRITE (*,*) 'OUTPUT FILE IS : ',OUTFIL

!     OPEN DATAFILES AND CHECK CONSISTENCY.
!     AT THE MOMENT DEFAULT FILENAMES ARE USED  
 
      ENI=10
      GRI=15
      VEI=20
      ENF=25
      GRF=30
      VEF=35

      OPEN (ENF,FILE=ENFIL(FINAL),ACCESS='DIRECT',FORM='UNFORMATTED', &
            STATUS='OLD',RECL=ENLEN)
      READ(ENF,REC=1) FMESHN(1)
      READ(ENF,REC=2) FWAVMX(1)
      READ(ENF,REC=4) FFINE
      READ(ENF,REC=6) CEFLIM
      IF (FMESHN(1).GT.MNUM) THEN
          WRITE(*,9910) 'FILE TOO BIG "'//ENFIL(FINAL)//'"'
          STOP
      ENDIF
      IF (FFINE) THEN
          FBASE=10+FMESHN(1)**3
          READ(ENF,REC=FBASE+1) FMESHN(2)
          READ(ENF,REC=FBASE+2) FWAVMX(2)
          READ(ENF,REC=FBASE+6) FEFLIM
          FANTM=2
          IF (FMESHN(2).GT.MNUM) THEN
              WRITE(*,9910) 'FILE TOO BIG "'//ENFIL(FINAL)//'"'
              STOP
          ENDIF
      ELSE
          FANTM=1
      ENDIF

      OPEN (GRF,FILE=GRFIL(FINAL),ACCESS='DIRECT',FORM='UNFORMATTED', &
            STATUS='OLD',RECL=GRLEN)
      READ(GRF,REC=1) I
      READ(GRF,REC=2) R
      IF ((I.NE.FMESHN(1)).OR.ABS((FWAVMX(1)-R)/FWAVMX(1)).GT.EPS) THEN
          WRITE (*,9910) 'INCONSISTENT INPUT FILE "'//GRFIL(FINAL)//'"' 
          STOP
      ENDIF
     
      IF (OVMETH.EQ.1) THEN
          OPEN (VEF,FILE=VEFIL(FINAL),ACCESS='DIRECT', &
                FORM='UNFORMATTED',STATUS='OLD',RECL=VELEN)
          READ(VEF,REC=1) I
          READ(VEF,REC=2) R
          IF ((I.NE.FMESHN(1)).OR.ABS((FWAVMX(1)-R)/FWAVMX(1)).GT.EPS) &
              THEN 
                  WRITE (*,9910) 'INCONSISTENT INPUT FILE "' &
                             //VEFIL(FINAL)//'"' 
              STOP
          ENDIF
      ENDIF

 
      WRITE(*,*) 'READING DATA FOR FINAL STATES. '
!     READ FINAL ENERGIES,GRADIENTS AND EIGENVECTORS 
      DO M=1,FANTM
          IF (M.EQ.1) WRITE (*,*) 'COARSE MESH :'
          IF (M.EQ.2) WRITE (*,*) 'FINE MESH :'
          FBASE=11+(M-1)*(FMESHN(1)**3+10)
          WRITE (*,*) 'ENERGIES...'
          OFFSET=0
          DO Z=0,FMESHN(M)-1
             DO Y=0,FMESHN(M)-1
                DO X=0,FMESHN(M)-1
                   READ(ENF,REC=OFFSET+FBASE) E(X,Y,Z,M)
                   OFFSET=OFFSET+1
                END DO
             END DO
          END DO
          WRITE (*,*) 'GRADIENTS...' 
          OFFSET=0
          DO Z=0,FMESHN(M)-1
             DO Y=0,FMESHN(M)-1
                DO X=0,FMESHN(M)-1
                   READ(GRF,REC=OFFSET+FBASE) (GRADF(I,X,Y,Z,M),I=1,3)
                   OFFSET=OFFSET+1
                END DO
             END DO
          END DO
          IF (OVMETH.EQ.1) THEN
              WRITE (*,*) 'EIGENVECTORS...' 
              OFFSET=0
              DO Z=0,FMESHN(M)-1
                 DO Y=0,FMESHN(M)-1
                    DO X=0,FMESHN(M)-1
                       READ(VEF,REC=OFFSET+FBASE) &
                       ((GF(I,J,X,Y,Z,M),I=1,N),J=1,2)
                       OFFSET=OFFSET+1
                    END DO
                 END DO
              END DO
          ENDIF
      END DO ! M-LOOP
   

!     OPEN FILES FOR INITIAL STATES 
      IBASE=11
      IF (INIT.EQ.FINAL) THEN
          ENI=ENF 
          GRI=GRF
          VEI=VEF
          IMESHN=FMESHN(1)
          IWAVMX=FWAVMX(1)
          IFINE=FFINE
      ELSE
          CLOSE(ENF)
          CLOSE(GRF)
          CLOSE(VEF)
          OPEN (ENI,FILE=ENFIL(INIT),ACCESS='DIRECT', &
                FORM='UNFORMATTED',STATUS='OLD',RECL=ENLEN)
          READ(ENI,REC=1) IMESHN
          READ(ENI,REC=2) IWAVMX
          READ(ENI,REC=4) IFINE 
          IF (IMESHN.GT.MNUM) THEN
              WRITE(*,9910) 'FILE TOO BIG "'//ENFIL(INIT)//'"'
              STOP
          ENDIF


 
          IF (OVMETH.EQ.1) THEN
              OPEN (VEI,FILE=VEFIL(INIT),ACCESS='DIRECT', &
                    FORM='UNFORMATTED',STATUS='OLD',RECL=VELEN)
              READ(VEI,REC=1) I
              READ(VEI,REC=2) R
              IF ((I.NE.IMESHN).OR.ABS((IWAVMX-R)/IWAVMX).GT.EPS) THEN
                  WRITE (*,9910) 'INCONSISTENT INPUT FILE "'// &
                                  VEFIL(INIT)//'"' 
                  STOP
              ENDIF
          ENDIF
      ENDIF
 
!     OPEN RESULTFILE
      RES=50
      OPEN(RES,FILE=OUTFIL,ACCESS='DIRECT',FORM='UNFORMATTED', &
           STATUS='NEW',RECL=ENLEN)
      WRITE(RES,REC=1) IMESHN
      WRITE(RES,REC=2) IWAVMX
      K=10

!     CREATE MIN-MAX TABLES OF THE FINAL ENERGIES
      WRITE (*,*) 'CREATING MIN/MAX TABLES FOR FINAL STATES...' 
      DO M=1,FANTM
          FMSIZE(M)=2*FWAVMX(M)/(FMESHN(M)-1)
          IF (M.EQ.1) WRITE(*,*) 'COARSE MESH...'
          IF (M.EQ.2) WRITE(*,*) 'FINE MESH...'
          DO Z=0,FMESHN(M)-1
             DO Y=0,FMESHN(M)-1
                DO X=0,FMESHN(M)-1
                   DE=(ABS(GRADF(1,X,Y,Z,M))+ABS(GRADF(2,X,Y,Z,M))+ &
                       ABS(GRADF(3,X,Y,Z,M)))*FMSIZE(M)/2
                   EMIN(X,Y,Z,M)=E(X,Y,Z,M)-DE
                   EMAX(X,Y,Z,M)=E(X,Y,Z,M)+DE
                END DO
             END DO
          END DO
      END DO ! M-LOOP


!     LOOP OVER ALL INITIAL STATES TO CREATE SCATTERING RATE
      WRITE (*,*) 'COMPUTING RATES:'
      WRITE (*,*) 'COARSE MESH...'
      WRITE (*,*) ' ' 

!     A JUMP IS PERFORMED BACK TO THIS LINE FOR GENERATION OF A FINE MESH 
1000  IMSIZE=2*IWAVMX/(IMESHN-1)
 
!     COMPUTE THE MESH, THIS ROUTINE IS CHANGED SO THAT IT CALCULATE THE WHOLE
!     MESH.
      DO ZI=0,IMESHN-1
       DO YI=0,IMESHN-1
        DO XI=0,IMESHN-1



          K=K+1

!         DETERMINE INITIAL ENERGY, K-VECTOR AND EIGENVECTORS
          OFFSET=ZI*IMESHN**2+YI*IMESHN+XI
          READ(ENI,REC=IBASE+OFFSET) EI
          IF (OVMETH.EQ.1) READ(VEI,REC=IBASE+OFFSET) GI
          KXI=XI*IMSIZE-IWAVMX
          KYI=YI*IMSIZE-IWAVMX
          KZI=ZI*IMSIZE-IWAVMX

!         ESTIMATE FINAL ENERGY          
          EF=EI+EDELTA(MECH,0.0)
 
!         FIND FINAL CUBES
          ANTFIN(2)=0
          IF (FFINE.AND.EF.LT.FEFLIM) THEN
              M=2
              DO Z=0,FMESHN(2)-1
                 DO Y=0,FMESHN(2)-1
                    DO X=0,FMESHN(2)-1
                       KXF=FMSIZE(M)*X-FWAVMX(M)
                       KYF=FMSIZE(M)*Y-FWAVMX(M)
                       KZF=FMSIZE(M)*Z-FWAVMX(M)
                       Q=SQRT((KXF-KXI)**2+(KYF-KYI)**2+(KZF-KZI)**2)
                       EF=EI+EDELTA(MECH,Q)
                       IF (EF.GT.FEFLIM) GOTO 999
                       IF (EMIN(X,Y,Z,2).LT.EF.AND.EMAX(X,Y,Z,2).GT.EF) THEN
                           ANTFIN(2)=ANTFIN(2)+1
                           FINALS(ANTFIN(2),1,2)=X
                           FINALS(ANTFIN(2),2,2)=Y
                           FINALS(ANTFIN(2),3,2)=Z
                       ENDIF
                    END DO
                 END DO
              END DO
          ENDIF
 
999       ANTFIN(1)=0
          IF ((.NOT.FFINE).OR.EF.GE.FEFLIM) THEN
              M=1
              DO Z=0,FMESHN(1)-1
                 DO Y=0,FMESHN(1)-1
                    DO X=0,FMESHN(1)-1
                       KXF=FMSIZE(M)*X-FWAVMX(M)
                       KYF=FMSIZE(M)*Y-FWAVMX(M)
                       KZF=FMSIZE(M)*Z-FWAVMX(M)
                       Q=SQRT((KXF-KXI)**2+(KYF-KYI)**2+(KZF-KZI)**2)
                       EF=EI+EDELTA(MECH,Q)
                       IF (EMIN(X,Y,Z,1).LT.EF.AND.EMAX(X,Y,Z,1).GT.EF) THEN
                           ANTFIN(1)=ANTFIN(1)+1
                           FINALS(ANTFIN(1),1,1)=X
                           FINALS(ANTFIN(1),2,1)=Y
                           FINALS(ANTFIN(1),3,1)=Z
                       ENDIF
                    END DO
                 END DO
              END DO
          ENDIF

!         CREATE SCATTERING RATE          
          RATE=0
          DO I=1,ANTFIN(M)
              X=FINALS(I,1,M)
              Y=FINALS(I,2,M)
              Z=FINALS(I,3,M)
 
              KXF=FMSIZE(M)*X-FWAVMX(M)
              KYF=FMSIZE(M)*Y-FWAVMX(M)
              KZF=FMSIZE(M)*Z-FWAVMX(M)
              Q=SQRT((KXF-KXI)**2+(KYF-KYI)**2+(KZF-KZI)**2)
              EF=EI+EDELTA(MECH,Q)
 

!             CALCULATE SQUARED MATRIX ELEMENT INCLUDING 2*PI/HBAR
!             FIRST FIND CORRECTION TO FINAL K-VECTOR. THE DIRECTION
!             COSINES OF THE PLANE NORMAL ARE ALSO RETURNED SORTED 
!             AND POSITIVE FOR USE IN THE CALCULATION OF THE DOS.
              LGRAD=SQRT(GRADF(1,X,Y,Z,M)**2+GRADF(2,X,Y,Z,M)**2+ &
                         GRADF(3,X,Y,Z,M)**2)
              W=(EF-E(X,Y,Z,M))/LGRAD
              IF (W.LT.0) THEN
                  C(1)=-GRADF(1,X,Y,Z,M)/LGRAD
                  C(2)=-GRADF(2,X,Y,Z,M)/LGRAD
                  C(3)=-GRADF(3,X,Y,Z,M)/LGRAD
              ELSE
                  C(1)=GRADF(1,X,Y,Z,M)/LGRAD
                  C(2)=GRADF(2,X,Y,Z,M)/LGRAD
                  C(3)=GRADF(3,X,Y,Z,M)/LGRAD
              ENDIF
              W=ABS(W)
              CALL CORREC(C,W,FMSIZE(M)/2,L1,L2,L3,DK)
              KXF=KXF+DK(1)
              KYF=KYF+DK(2)
              KZF=KZF+DK(3)
              Q=SQRT((KXF-KXI)**2+(KYF-KYI)**2+(KZF-KZI)**2)
              TRATE = MATEL(Q,MECH)
             
!             CALCULATE AND MULTIPLY DOS
              TRATE=TRATE*AREA(L1,L2,L3,W,FMSIZE(M)/2) &
                  /((2*PI)**3*LGRAD*EV)

!             CALCULATE AND MULTIPLY OVERLAP FACTOR
!             USES THE OVERLAP FACTOR IN THE CENTRE OF THE CUBE
              IF (OVMETH.EQ.1) THEN
                  G1=(0,0)
                  G2=(0,0)
                  G3=(0,0)
                  G4=(0,0)
                  DO J=1,N
                      G1=G1+GI(J,1)*CONJG(GF(J,1,X,Y,Z,M))
                      G2=G2+GI(J,1)*CONJG(GF(J,2,X,Y,Z,M))
                      G3=G3+GI(J,2)*CONJG(GF(J,1,X,Y,Z,M))
                      G4=G4+GI(J,2)*CONJG(GF(J,2,X,Y,Z,M))
                  END DO
                  TRATE=TRATE*(ABS(G1)**2+ABS(G2)**2+ABS(G3)**2+ &
                               ABS(G4)**2)/2
              ELSEIF (OVMETH.EQ.2) THEN
                  R=(KXF*KXI+KYF*KYI+KZF*KZI) &
                    /(SQRT(KXF**2+KYF**2+KZF**2) &
                    *SQRT(KXI**2+KYI**2+KZI**2))
                  IF (INIT.EQ.FINAL) THEN
                      TRATE=TRATE*(1+3*R**2)/4    
                  ELSE
                      TRATE=TRATE*3.0*ABS(1-R**2)/4
                  ENDIF
              ENDIF
 
!             ADD CONTRIBUTION OF THE CUBE TO THE TOTAL RATE
              RATE=RATE+TRATE
          END DO ! I-LOOP
          WRITE(RES,REC=K) RATE
!     END MESH LOOPS
        END DO ! XI
       END DO ! YI
      END DO ! ZI
 
!     IF A FINE MESH EXISTS FOR INITIAL STATES, A FINE MESH IS ALSO
!     GENERATED FOR THE RATE FILE
      IF (IFINE) THEN
          IBASE=10+IMESHN**3
          READ (ENI,REC=IBASE+1) IMESHN
          READ (ENI,REC=IBASE+2) IWAVMX
          WRITE (RES,REC=K+1) IMESHN
          WRITE (RES,REC=K+2) IWAVMX
          WRITE (RES,REC=K+4) .FALSE.
          WRITE (RES,REC=4) .TRUE.
          IFINE=.FALSE.
          IBASE=IBASE+11
          K=K+10
          WRITE (*,*) 'FINE MESH...'
          WRITE (*,*) ' '
          GOTO 1000
      ENDIF

9900  FORMAT(' ',T7,I1,'. ',A)
9901  FORMAT(' ',A)
9902  FORMAT(A)
9910  FORMAT(' SCRATES ERROR : ',A)

      END
!.... PROGRAM SCRATE ............................




