      MODULE Rateinit
      INTEGER, PARAMETER :: DBL = SELECTED_REAL_KIND(p=13)
     
!     TEMPERATURE 
      REAL(KIND=DBL) :: T
!     SET VALUES OF MISCELLANEOUS PHYSICAL CONSTANTS AND MATERIAL PARAMETERS 
!     THE PARAMETERS ARE: OPTICAL PHONON ENERGY(eV),OPTICAL PERMITTIVITY,
!     STATIC PERMITTIVITY, ACOUSTIC DEFORMATION POTENTIAL(eV), AVERAGE
!     VELOCITY OF SOUND(m/s), MASS DENSITY(kg/m**3), ELECTRONVOLT(J), 
!     BOLTZMAN'S CONSTANT(J/K), PI, ELEMENTARY CHARGE(C), REDUCED
!     PLANCK'S CONSTANT(Js). 

!     SCATTERING MECHANISM PARAMETERS:
!----------------------------------------------------------------------------------

!     POLAR OPTICAL PHONON SCATTERING (equal for holes and electrons)
      REAL(KIND=DBL), PARAMETER :: OPTEN=0.0178,EPSINF=11.47,EPSZER=16.58          ! [eV,1,1]

!     ACOUSTIC DEFORMATION POTENTIAL SCATTERING (equal for holes and electrons)
!      REAL(KIND=DBL), PARAMETER :: D=5.60,S=4570.0,RHO=7406.0,EV=1.602176487D-19  ! [eV,m/s,kg/m³,J/eV](holes)
      REAL(KIND=DBL), PARAMETER :: D=7.00,S=4570.0,RHO=7406.0,EV=1.602176487D-19  ! [eV,m/s,kg/m³,J/eV](electrons)

!     CONSTANTS
      REAL(KIND=DBL), PARAMETER :: KB=1.3806504D-23,PI=3.14159265358979           ! [J/K,1]
      REAL(KIND=DBL), PARAMETER :: EC=1.602176487D-19,HBAR=1.054571628D-34        ! [C/e,Js]
      REAL(KIND=DBL), PARAMETER :: EPS0=8.854187817D-12                           ! [C²/(Nm²)]

!     NON-POLAR OPTICAL PHONON SCATTERING AND IONIZED IMPURITY SCATTERING
!     1ST MECHANISM ONLY FOR HOLES, 2ND MECHANISM IS UNCERTAIN
      REAL(KIND=DBL), PARAMETER :: DKSQR=3.16D23,NIMP=1.0D23                      ! [(eV²)/(m²),1/m³]

!     ALLOY SCATTERING (1st line for holes, 2nd line for electrons)

      REAL(KIND=DBL), PARAMETER :: DV=0.3,COMP=0.28,NCAT=1.47D28                  ! [eV,(#Cd)/(#Te),1/m³](holes)
!      REAL(KIND=DBL), PARAMETER :: DV=1.5,COMP=0.28,NCAT=1.47D28                  ! [eV,(#Cd)/(#Te),1/m³](electrons)

!----------------------------------------------------------------------------------
      
      REAL(KIND=DBL) :: CPOLOP,CNONOP,CAC,CION,BETSQR,CALLOY

!     MODEL SIZE PARAMETERS
      INTEGER, PARAMETER :: MS=14, MSH=7, MSD=28

!     PARAMETERS HANDLING INITIALIZATION DATA
!     THE NUMBER OF SCATTERING MECHANISMS IS HELD IN SCRNUM,
!     MAXIMUM NUMBER OF BANDS IN MAXBAN. IF THE INITIALIZATION
!     DATA IS CHANGED HERE, THE CORRESPONDING VARIABLES MUST ALSO
!     BE CHANGED IN SCRINI

      INTEGER            :: MECH,BANNUM,INIT,FINAL,ANT,LEN
      INTEGER, PARAMETER :: SCRNUM=8,MAXBAN=5,VAX=4,PC=1,BYTES=340
      CHARACTER(LEN=20)  :: BNAM(MAXBAN)
      CHARACTER(LEN=12)  :: ENFIL(MAXBAN),GRFIL(MAXBAN),VEFIL(MAXBAN),OUTFIL
      LOGICAL            :: INITOK, FINOK
      INTEGER            :: OVMETH,ERROR 

      CHARACTER(LEN=50), PARAMETER  :: SCRNAM(SCRNUM) = &
                (/'POLAR OPTICAL, ABSORPTION',&
                  'POLAR OPTICAL, EMISSION',&
                  'NONPOLAR OPTICAL, ABSORPTION',&
                  'NONPOLAR OPTICAL, EMISSION',&
                  'ACOUSTIC DEFORMATION POTENTIAL, ABSORPTION',&
                  'ACOUSTIC DEFORMATION POTENTIAL, EMISSION',&
                  'IONIZED IMPURITY SCATTERING',&
                  'ALLOY SCATTERING'/)


      END MODULE Rateinit

