!     PROGRAM USED TO READ BAND STRUCTURE FROM THE FILES PRODUCED BY KPBAND2mkl.f

      PROGRAM READBAND
      INTEGER, PARAMETER :: DBL = SELECTED_REAL_KIND(p=13)
      CHARACTER(LEN=12) :: INPUT, OUTPUT
      INTEGER :: FN, I, J, L, M
!     NMAX IS JUST FOR INITIALIZATION. MUST BE CHANGED IF YOU WANT LARGER MESH SIZES.
      INTEGER, PARAMETER :: NMAX = 64
      INTEGER :: CRECORDS(NMAX), FRECORDS(NMAX)
      REAL(KIND=DBL) :: ENERGY, K, WAVMAX, FWAVMAX, MSIZE, FMSIZE
      LOGICAL :: SKIP

      I=1
      J=1
      L=1
      M=1
      SKIP=.FALSE.      

      WRITE(*,*) 'Enter input filename:'
      READ(*,*) INPUT
      
      OPEN(10,FILE=INPUT,ACCESS='DIRECT',RECL=8)
      READ(10,REC=1) N
      READ(10,REC=2) WAVMAX
      READ(10,REC=10+N**3+1) FN
      READ(10,REC=10+N**3+2) FWAVMAX
      CLOSE(10)

!      SCALE THE WAVE NUMBERS FOR PLOTTING.
      WAVMAX = WAVMAX*1.0E-10
      FWAVMAX = FWAVMAX*1.0E-10
    
      WRITE(*,*) 'Enter output filename:'
      READ(*,*) OUTPUT
      
      CALL PICKER( N,J,CRECORDS)
      CALL PICKER(FN,M,FRECORDS)

!     DISTANCE BETWEEN MESH POINTS.
      MSIZE = 2*WAVMAX/(J-1)
      FMSIZE = 2*FWAVMAX/(M-1)

      WRITE(*,*) 'NUMBER OF LINEAR MESHPOINTS (111), J, FOUND:'
      WRITE(*,*) J
      WRITE(*,*) 'NUMBER OF FINE MESHPOINTS:'
      WRITE(*,*) M
      WRITE(*,*) 'MAXIMUM WAVE VECTOR [10^10 1/m]:'
      WRITE(*,*) WAVMAX
      WRITE(*,*) 'MAXIMUM FINE MESH WAVE VECTOR [10^10 1/m]:'
      WRITE(*,*) FWAVMAX
      
!      PRINT *, 'RECORD NUMBERS', CRECORDS
!      PRINT *, 'FINE RECORD NUMBER', FRECORDS
      OPEN(10,FILE=INPUT,ACCESS='DIRECT',RECL=8)
      OPEN(35,FILE=OUTPUT)

!     THIS SECTION MERGES THE COARSE AND FINE MESHES.
300   IF (-WAVMAX+MSIZE*(I-1).LE.WAVMAX) THEN
!     SKIP OVER THE COARSE MESH POINTS NEAR k = 0.
          IF (SKIP) THEN
              I=I+CEILING((J-1)*FWAVMAX/WAVMAX)
              SKIP=.FALSE.
          ENDIF
!     THIS IF CONDITION CHECKS IF YOU ARE IN THE COARSE OR FINE MESH,
!     AND WRITES THE CORRESPONDING ENERGY/K-VALUE TO FILE.
          IF (-WAVMAX+MSIZE*(I-1).LT.(-FWAVMAX).OR. &
              -FWAVMAX+FMSIZE*(L-1).GT.FWAVMAX) THEN
              READ(10,REC=CRECORDS(I)) ENERGY
              K=-WAVMAX+MSIZE*(I-1)
              WRITE(35,220) K, ENERGY
              I=I+1
          ELSE
              READ(10,REC=(10+J**3+FRECORDS(L))) ENERGY
              K=-FWAVMAX+FMSIZE*(L-1)
              WRITE(35,220) K, ENERGY
              L=L+1
!     IF YOU ARE IN THE FINE MESH, AND THE NEXT MESH POINT IS
!     OUTSIDE OF THE FINE MESH, THIS SAYS THAT IT IS TIME TO SKIP
!     THE COARSE MESH POINTS LOCATED INSIDE THE FINE MESH.
              IF (-FWAVMAX+FMSIZE*(L-1).GT.FWAVMAX) THEN
                  SKIP=.TRUE.
              ENDIF
          ENDIF
          GOTO 300
      ENDIF
      CLOSE(10)

220   FORMAT(E9.3,' ', E9.3)
      CLOSE(35)
      STOP
      END

!     SUBROUTINE FOR PICKING OUT MESHPOINTS IN THE (111) PLANE.      
      SUBROUTINE PICKER(MESHNU,J,RECORDS)
      
      INTEGER :: X,Y,Z,I,J,MESHNU
      INTEGER, PARAMETER :: NMAX = 64
      INTEGER :: RECORDS(NMAX)
      
      I=10
      J=0

      DO Z=0,MESHNU-1
         DO Y=0,MESHNU-1
            DO X=0,MESHNU-1
               I=I+1
!              ALONG THE (111) DIRECTION.
               IF ((X.EQ.Y).AND.(Y.EQ.Z)) THEN
                  J=J+1  
                  RECORDS(J)=I
               ENDIF
            END DO
         END DO
      END DO

      RETURN       
      END
