  MODULE readmodule
  USE Halvorinit
  IMPLICIT NONE

  REAL(KIND=DBL),PARAMETER :: eC=1.602176487D-19     ! Coulomb charge
  REAL(KIND=DBL),PARAMETER :: hbar=1.054571628D-34   ! Reduced Planck const Ø: Js
  REAL(KIND=DBL),PARAMETER :: xfrac=0.28       ! alloy fraction
  REAL(KIND=DBL),PARAMETER :: hw0=(xfrac*2.92+(1-xfrac)*2.62+xfrac*2.81*2+ &
                                  (1-xfrac)*2.28*2)*1.D13*hbar/3.   ! hbar*omega0, optic. phon. freq, CMT

  INTEGER, PARAMETER :: maxLength = 32
  INTEGER, PARAMETER :: NRates    = 28
  INTEGER, PARAMETER :: NBands    = 7

  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,3,2) :: kMesh ! 3 is for kx, ky, kz
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NRates,2)   :: halvorScRates
!  Derivatives. 1: x, 2: y, 3: z.
  REAL(KIND=DBL), DIMENSION(3,maxLength,maxLength,maxLength,NRates,2) :: halvorScRatesDer
!  Second derivatives. 1: xx, 2: yy, 3: zz, 4: xy, 5: xz, 6: yz
  REAL(KIND=DBL), DIMENSION(6,maxLength,maxLength,maxLength,NRates,2) :: halvorScRates2Der
  ! Make a list of which rates are polar/non-polar emission rates (needed because of the interpolation).
  INTEGER, PARAMETER                      :: nEmList = 8
  INTEGER, PARAMETER, DIMENSION(nEmList)  :: emList  = (/5,6,7,8,13,14,15,16/)
  INTEGER :: corner(8,3)
  INTEGER :: meshType
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NBands,2)   :: energyMesh
  REAL(KIND=DBL), DIMENSION(3,maxLength,maxLength,maxLength,NBands,2) :: halvorBandGrad
  REAL(KIND=DBL), DIMENSION(6,maxLength,maxLength,maxLength,NBands,2) :: halvorBand2Der
  COMPLEX(KIND=DBL), DIMENSION(MS,2,maxLength,maxLength,maxLength,NBands,2) :: halvorBandEVec! 2 spin states per band
  REAL(KIND=DBL), DIMENSION(NBands)                                   :: fineMeshMaxEnergy
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NBands,2)   :: energyMeshMax !maximum energies within cells
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NBands,2)   :: energyMeshMin !minimum energies within cells
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NRates,2)   :: meshRateMax !maximum rate within cells
  REAL(KIND=DBL), DIMENSION(maxLength,maxLength,maxLength,NRates,2)   :: meshRateMin !minimum rate within cells
  INTEGER, DIMENSION(2)                                               :: meshNumber
  REAL(KIND=DBL), DIMENSION(2)                                        :: maxWavenumber
  REAL(KIND=DBL), DIMENSION(2)                                        :: deltaK
  LOGICAL, DIMENSION(NBands)                                          :: hasFine
  REAL(KIND=DBL), DIMENSION(NBands,2)                                 :: bandMin, valleyE


  CONTAINS

  SUBROUTINE loadBandData(bandName)
  IMPLICIT NONE
  INTEGER, PARAMETER :: ENE=101, EIG=102, GRA=103, DER=104
  INTEGER            :: ind, recordLength, record, recordBase(4), dir(3),l
  INTEGER            :: X,Y,Z,j,i
  CHARACTER(LEN=2)   :: bandName
  CHARACTER(LEN=12)  :: openFile(4)
  REAL(KIND=DBL)     :: swap
  INTEGER,PARAMETER  :: coarse=1,fine=2

    if (bandName == 'SO') then
      ind = 1
    else if (bandName == 'LH') then
      ind = 2
    else if (bandName == 'HH') then
      ind = 3
    else if (bandName == 'CB') then
      ind = 4
    else if (bandName == 'CS') then
      ind = 5
    else if (bandName == 'CL') then
      ind = 6
    else if (bandName == 'CH') then
      ind = 7
    else
      write(*,*) 'Error in bandName.'
    end if

!    bandName    = TRIM(bandName)
    openFile(1) = bandName(1:LEN_TRIM(bandName))//'ENE.TAB'
    openFile(2) = bandName(1:LEN_TRIM(bandName))//'EIG.TAB'
    openFile(3) = bandName(1:LEN_TRIM(bandName))//'GRA.TAB'
    openFile(4) = bandName(1:LEN_TRIM(bandName))//'DER.TAB'
    recordLength=8
    open(ENE,file=openFile(1), ACCESS='DIRECT', FORM='UNFORMATTED',STATUS='OLD',RECL=recordLength)
    open(EIG,file=openFile(2), ACCESS='DIRECT', FORM='UNFORMATTED',STATUS='OLD',RECL=recordLength*MS*2*2)
    open(GRA,file=openFile(3), ACCESS='DIRECT', FORM='UNFORMATTED',STATUS='OLD',RECL=recordLength*3)
    open(DER,file=openFile(4), ACCESS='DIRECT', FORM='UNFORMATTED',STATUS='OLD',RECL=recordLength*6)

    read(ENE,REC=1) meshNumber(coarse)
    read(ENE,REC=2) maxWavenumber(coarse)
    read(ENE,REC=3) valleyE(ind,coarse)
    read(ENE,REC=4) hasFine(ind)
!    read(ENE,REC=5) BFRAC(ind,coarse)
    read(ENE,REC=6) bandMin(ind,coarse)
    deltaK(coarse)=2*maxWavenumber(coarse)/(meshNumber(coarse)-1)
    record=0

    do Z=1,meshNumber(coarse)
      dir(3)=Z
    do Y=1,meshNumber(coarse)
      dir(2)=Y
    do X=1,meshNumber(coarse)
      dir(1)=X
      read(ENE,REC=11+record) energyMesh(X,Y,Z,ind,coarse)
      read(EIG,REC=11+record) ((halvorBandEVec(j,i,X,Y,Z,ind, coarse), j=1,MS),i=1,2)
      read(GRA,REC=11+record) (halvorBandGrad(j,X,Y,Z,ind,coarse),j=1,3)
      read(DER,REC=11+record) (halvorBand2Der(j,X,Y,Z,ind,coarse),j=1,6)

      ! This was originally without /2._DBL. Think that was an error.
      energyMeshMax(X,Y,Z,ind,coarse)=energyMesh(X,Y,Z,ind,coarse)+sqrt(sum(halvorBandGrad(:,X,Y,Z,ind,coarse)**2))*deltaK(coarse)/2._DBL
      energyMeshMin(X,Y,Z,ind,coarse)=energyMesh(X,Y,Z,ind,coarse)-sqrt(sum(halvorBandGrad(:,X,Y,Z,ind,coarse)**2))*deltaK(coarse)/2._DBL
      record=record+1
      ! Swap xz and yz second derivatives for convenience
      swap = halvorBand2Der(5,X,Y,Z,ind,coarse)
      halvorBand2Der(5,X,Y,Z,ind,coarse) = halvorBand2Der(6,X,Y,Z,ind,coarse)
      halvorBand2Der(6,X,Y,Z,ind,coarse) = swap
      do l=1,3
        ! Storing the mesh like this stores a lot of redundant information, but it makes accessing
        ! the mesh easier, so it's worth it. Besides, a huge meshsize of 64 gives a memory usage of
        ! 64^3 * 3 * 2 * 8 byte = 12 MB. Worth it. Meshsize 32 uses 1.5 MB, which is negligible.
        kMesh(X,Y,Z,l,coarse) = -maxWavenumber(coarse)+deltaK(coarse)*(dir(l)-1)
      end do
    enddo
    enddo
    enddo

!    print *, trim(bandName), ": loaded coarse mesh."
!    print *, "meshNumber=",meshNumber(coarse)
!    print *, "deltaK=",deltaK(coarse)
!    print *, "maxWavenumber=",maxWavenumber(coarse)
    if(hasFine(ind)) then !if there is a fine mesh, then load it.
      recordBase=(/21+record,21+record,21+record,21+record/)
      read(ENE,REC=10+record+1) meshNumber(fine)
      read(ENE,REC=10+record+2) maxWavenumber(fine)
!      read(ENE,REC=10+record+5) BFRAC(ind,fine)
      read(ENE,REC=10+record+6) bandMin(ind,fine)
      deltaK(fine)=2*maxWavenumber(fine)/(meshNumber(fine)-1)
      record=0
      fineMeshMaxEnergy(ind)=0._DBL
      do Z=1,meshNumber(fine)
        dir(3)=Z
      do Y=1,meshNumber(fine)
        dir(2)=Y
      do X=1,meshNumber(fine)
        dir(1)=X

        read(ENE,REC=recordBase(1)+record) energyMesh(X,Y,Z,ind,fine)
        read(EIG,REC=recordBase(2)+record) ((halvorBandEVec(j,i,X,Y,Z,ind, fine), j=1,MS),i=1,2)
        read(GRA,REC=recordBase(3)+record) (halvorBandGrad(j,X,Y,Z,ind,fine),j=1,3)
        read(DER,REC=recordBase(4)+record) (halvorBand2Der(j,X,Y,Z,ind,fine),j=1,6)

        energyMeshMax(X,Y,Z,ind,fine)=energyMesh(X,Y,Z,ind,fine)+sqrt(sum(halvorBandGrad(:,X,Y,Z,ind,fine)**2))*deltaK(fine)/2._DBL
        energyMeshMin(X,Y,Z,ind,fine)=energyMesh(X,Y,Z,ind,fine)-sqrt(sum(halvorBandGrad(:,X,Y,Z,ind,fine)**2))*deltaK(fine)/2._DBL
        ! Find the highest energy in the fine mesh.
        if (energyMeshMax(X,Y,Z,ind,fine) >= fineMeshMaxEnergy(ind)) then
          fineMeshMaxEnergy(ind) = energyMeshMax(X,Y,Z,ind,fine)
        endif
        record=record+1
        ! Swap xz and yz second derivatives for convenience
        swap = halvorBand2Der(5,X,Y,Z,ind,fine)
        halvorBand2Der(5,X,Y,Z,ind,fine) = halvorBand2Der(6,X,Y,Z,ind,fine)
        halvorBand2Der(6,X,Y,Z,ind,fine) = swap
        do l=1,3
          kMesh(X,Y,Z,l,fine) = -maxWavenumber(fine)+deltaK(fine)*(dir(l)-1)
        end do
      enddo
      enddo
      enddo

      ! Bug testing.
!      WRITE(*,*) 'maxWavenumber: ' , bandName, maxWavenumber
!      WRITE(*,*) 'halvorBandGrad: ', halvorBandGrad(:,10,10,10,ind,2)
!      WRITE(*,*) 'halvorBand2Der: ', halvorBand2Der(:,10,10,10,ind,2)
!      WRITE(*,*) 'halvorBandEVec: ', halvorBandEVec(:,:,10,10,10,ind,2)

!      print *, trim(bandName), ": loaded fine mesh."
!      print *, "meshNumber=",meshNumber(fine)
!      print *, "deltaK=",deltaK(fine)
!      print *, "maxWavenumber=",maxWavenumber(fine)
    endif
    close(ENE)
    close(EIG)
    close(GRA)
    close(DER)
  END SUBROUTINE loadBandData

  SUBROUTINE loadRates(rateMech,ind,bandchar)
  ! bandchar is the initial band in the chosen scattering mechanism.
  IMPLICIT NONE
  INTEGER            :: ind,recordLength,record,recordBase(4),band
  INTEGER            :: X,Y,Z,j,i
  CHARACTER(LEN=*)   :: rateMech, bandchar
  INTEGER, PARAMETER :: coarse=1,fine=2

  IF (bandchar == 'SO') THEN
    band = 1
  ELSE IF (bandchar == 'LH') THEN
    band = 2
  ELSE IF (bandchar == 'HH') THEN
    band = 3
  ELSE IF (bandchar == 'CB') THEN
    band = 4
  ELSE IF (bandchar == 'CS') THEN
    band = 5
  ELSE IF (bandchar == 'CL') THEN
    band = 6
  ELSE IF (bandchar == 'CH') THEN
    band = 7
  ELSE
    WRITE(*,*) 'Error in loadRates. Invalid band choice.'
    STOP
  END IF

!    bandName    = TRIM(bandName)
    recordLength=8
    open(101,file=rateMech, ACCESS='DIRECT', FORM='UNFORMATTED', STATUS='OLD',RECL=recordLength)
    deltaK(coarse)=2*maxWavenumber(coarse)/(meshNumber(coarse)-1)
    record=0
    do Z=1,meshNumber(coarse)
    do Y=1,meshNumber(coarse)
    do X=1,meshNumber(coarse)
      read(101,REC=11+record) halvorScRates(X,Y,Z,ind,coarse)
      record=record+1
    enddo
    enddo
    enddo
!    print *, trim(rateMech), ": loaded coarse mesh."
!    print *, "meshNumber=",meshNumber(coarse)
!    print *, "deltaK=",deltaK(coarse)
!    print *, "maxWavenumber=",maxWavenumber(coarse)
    if(hasFine(band)) then !if there is a fine mesh, then load it.
      recordBase=(/21+record,21+record,21+record,21+record/)
      deltaK(fine)=2*maxWavenumber(fine)/(meshNumber(fine)-1)
      record=0
      do Z=1,meshNumber(fine)
      do Y=1,meshNumber(fine)
      do X=1,meshNumber(fine)
        read(101,REC=recordBase(1)+record) halvorScRates(X,Y,Z,ind,fine)
        record=record+1
      enddo
      enddo
      enddo
!      print *, trim(rateMech), ": loaded fine mesh."
!      print *, "meshNumber=",meshNumber(fine)
!      print *, "deltaK=",deltaK(fine)
!      print *, "maxWavenumber=",maxWavenumber(fine)
    endif
    close(101)
  END SUBROUTINE loadRates


  FUNCTION ktoEinterpolate(kx,ky,kz,bandchar)
  IMPLICIT NONE
  REAL(KIND=DBL),  INTENT(IN) :: kx,ky,kz
  CHARACTER(LEN=2),INTENT(IN) :: bandchar
  REAL(KIND=DBL)              :: ktoEinterpolate,k(3),energy(8),weight(8) ! 8 corners
  INTEGER                     :: lowx,highx,lowy,highy,lowz,highz,test,band
  INTEGER                     :: i,j,l,x,y,z
  INTEGER, PARAMETER          :: coarse=1, fine=2
  REAL(KIND=DBL)              :: wx,wy,wz,kmax,kmin

  IF (bandchar == 'SO') THEN
    band = 1
  ELSE IF (bandchar == 'LH') THEN
    band = 2
  ELSE IF (bandchar == 'HH') THEN
    band = 3
  ELSE IF (bandchar == 'CB') THEN
    band = 4
  ELSE IF (bandchar == 'CS') THEN
    band = 5
  ELSE IF (bandchar == 'CL') THEN
    band = 6
  ELSE IF (bandchar == 'CH') THEN
    band = 7
  ELSE
    WRITE(*,*) 'Error in ktoEinterpolate. Invalid band choice.'
  END IF

  kmax=maxWavenumber(coarse)
  IF (kx > kmax .OR. ky > kmax .OR. kz > kmax) WRITE(*,*) kx,ky,kz, 'k-value too large. Weird results may follow.'
  kmin=-maxWavenumber(coarse)
  IF (kx < kmin .OR. ky < kmin .OR. kz < kmin) WRITE(*,*) kx,ky,kz, 'k-value too small. Weird results may follow.'

  k(1)=kx
  k(2)=ky
  k(3)=kz

  IF ( hasFine(band) .AND. (maxval(abs(k)) < maxWavenumber(fine)) ) THEN
    meshType = fine
  ELSE
    meshType = coarse
  END IF

  lowx = 1
  highx=meshNumber(meshType)
! x-search
  DO
    test = highx - (highx-lowx)/2
    IF ( kMesh(test,1,1,1,meshType) >= kx ) THEN
      highx = test
    ELSE
      lowx = test
    END IF
    IF (highx-lowx == 1) EXIT
  END DO
! y-search
  lowy = 1
  highy=meshNumber(meshType)
  DO
    test = highy - (highy-lowy)/2
    IF (kMesh(1,test,1,2,meshType) >= ky) THEN
      highy = test
    ELSE
      lowy = test
    END IF
    IF (highy-lowy == 1) EXIT
  END DO

! z-search
  lowz = 1
  highz=meshNumber(meshType)
  DO
    test = highz - (highz-lowz)/2
    IF (kMesh(1,1,test,3,meshType) >= kz) THEN
      highz = test
    ELSE
      lowz = test
    END IF
    IF (highz-lowz == 1) EXIT
  END DO

  corner(1,:) = (/lowx,lowy,lowz/)
  corner(2,:) = (/highx,lowy,lowz/)
  corner(3,:) = (/highx,highy,lowz/)
  corner(4,:) = (/lowx,highy,lowz/)
  corner(5,:) = (/lowx,lowy,highz/)
  corner(6,:) = (/highx,lowy,highz/)
  corner(7,:) = (/highx,highy,highz/)
  corner(8,:) = (/lowx,highy,highz/)

  DO l=1,8 ! 8 corners
    x=corner(l,1)
    y=corner(l,2)
    z=corner(l,3)
    ! Fischetti & Laux method.
    energy(l) = energyMesh(x,y,z,band,meshType)
!    WRITE(*,*) 'Corner loop energy:', energy(l)
!    DO i=1,3 ! Tricky tricks.
      ! The factor 0.5 in front of halvorBand2Der is gone because we only use each of the derivatives
      ! xy, yz, zx once, instead of calculating xy, yx, yz, zy, xz and zx, as the order of 
      ! differentiation is irrelevant.
      ! Note: This indexing requires the 2Der to be stored in the order xx,yy,zz,xy,yz,zx.

      ! Double note: interpolation using gradient/second derivatives is currently disabled, due to
      ! bad results for these values causing inaccuracies in the interpolation.

!      energy(l) = energy(l)! +halvorBandGrad(i,x,y,z,band,meshType)*(k(i)-kMesh(x,y,z,i,meshType))  !  & 
!                        +0.5*halvorBand2Der(i,x,y,z,band,meshType)*(k(i)-kMesh(x,y,z,i,meshType))**2 &
!                            +halvorBand2Der(i+3,x,y,z,band,meshType) &
!                            *(k(i)-kMesh(x,y,z,i,meshType)) &
!                            *(k(MOD(i,3)+1)-kMesh(x,y,z,MOD(i,3)+1,meshType))
      ! The index trickery with MOD(i,3)+1 is so we can manage with three iterations of the i-loop. 
!    END DO
  END DO

  DO l=1,8
    ! F&L don't use absolute values here, but I think that's a typo on their part. This works great.
    wx=( 1-abs(kx-kMesh(corner(l,1),corner(l,2),corner(l,3),1,meshType))/deltaK(meshType) )
    wy=( 1-abs(ky-kMesh(corner(l,1),corner(l,2),corner(l,3),2,meshType))/deltaK(meshType) )
    wz=( 1-abs(kz-kMesh(corner(l,1),corner(l,2),corner(l,3),3,meshType))/deltaK(meshType) )
    weight(l) = wx*wy*wz
  END DO

  ! And finally sum up the contributions from each corner with the proper weights.
  ktoEinterpolate=0._DBL
  DO l=1,8
    ktoEinterpolate = ktoEinterpolate+weight(l)*energy(l)
  END DO
  END FUNCTION ktoEinterpolate

  FUNCTION ratesInterpolate(kx,ky,kz,rate,bandchar)
  IMPLICIT NONE
  REAL(KIND=DBL),   INTENT(IN) :: kx,ky,kz
  INTEGER,          INTENT(IN) :: rate
  CHARACTER(LEN=2), INTENT(IN) :: bandchar
  REAL(KIND=DBL)               :: ratesInterpolate, rates(8), weight(8) ! 8 corners
  INTEGER                      :: lowx,highx,lowy,highy,lowz,highz,test
  INTEGER                      :: i,j,l,x,y,z,band
  INTEGER, PARAMETER           :: coarse=1, fine=2
  REAL(KIND=DBL)               :: wx,wy,wz,kmax,kmin,k(3)

  IF (bandchar == 'SO') THEN
    band = 1
  ELSE IF (bandchar == 'LH') THEN
    band = 2
  ELSE IF (bandchar == 'HH') THEN
    band = 3
  ELSE IF (bandchar == 'CB') THEN
    band = 4
  ELSE IF (bandchar == 'CS') THEN
    band = 5
  ELSE IF (bandchar == 'CL') THEN
    band = 6
  ELSE IF (bandchar == 'CH') THEN
    band = 7
  ELSE
    WRITE(*,*) 'Error in ktoEinterpolate. Invalid band choice.'
  END IF

  kmax=maxWavenumber(coarse)
  IF (kx >= kmax .OR. ky >= kmax .OR. kz >= kmax) WRITE(*,*)'k-value too large. Weird results may follow.'
  kmin = -maxWavenumber(coarse)
  IF (kx <= kmin .OR. ky <= kmin .OR. kz <= kmin) WRITE(*,*)'k-value too small. Weird results may follow.'

  k(1)=kx
  k(2)=ky
  k(3)=kz

  IF ( hasFine(band) .AND. (maxval(abs(k)) < maxWavenumber(fine)) ) THEN
    meshType = fine
  ELSE
    meshType = coarse
  END IF

  lowx = 1
  highx=meshNumber(meshType)
! x-search
  DO
    test = highx - (highx-lowx)/2
    IF (kMesh(test,1,1,1,meshType) >= kx) THEN
      highx = test
    ELSE
      lowx = test
    END IF
    IF (highx-lowx == 1) EXIT
  END DO
! y-search
  lowy = 1
  highy=meshNumber(meshType)
  DO
    test = highy - (highy-lowy)/2
    IF (kMesh(1,test,1,2,meshType) >= ky) THEN
      highy = test
    ELSE
      lowy = test
    END IF
    IF (highy-lowy == 1) EXIT
  END DO

! z-search
  lowz = 1
  highz=meshNumber(meshType)
  DO
    test = highz - (highz-lowz)/2
    IF (kMesh(1,1,test,3,meshType) >= kz) THEN
      highz = test
    ELSE
      lowz = test
    END IF
    IF (highz-lowz == 1) EXIT
  END DO

!  Figure showing the numbering of corners.
!
!       8 ________7       z
!        /       /|        ^   y
!       /       / |        |  /
!    5 /__ ____/6 |        | /
!      |  |    |  |        |/_____> x
!      | 4|____|__|3
!      |  /    |  /
!      | /     | /
!      |/______|/
!     1         2

  corner(1,:) = (/lowx,lowy,lowz/)
  corner(2,:) = (/highx,lowy,lowz/)
  corner(3,:) = (/highx,highy,lowz/)
  corner(4,:) = (/lowx,highy,lowz/)
  corner(5,:) = (/lowx,lowy,highz/)
  corner(6,:) = (/highx,lowy,highz/)
  corner(7,:) = (/highx,highy,highz/)
  corner(8,:) = (/lowx,highy,highz/)
  DO l=1,8 ! 8 corners
    x=corner(l,1)
    y=corner(l,2)
    z=corner(l,3)
    ! Fischetti & Laux method.
    rates(l) = halvorScRates(x,y,z,rate,meshType)
!    All of this is commented out since the numerical rate derivatives were, quite frankly, shit.
!    DO i=1,3 ! Tricky tricks.
!      ! The factor 0.5 in front of the second halvorScRates2Der is gone because we only use each of the
!      ! derivatives xy, yz, zx once, instead of calculating xy, yx, yz, zy, xz and zx, as the order of 
!      ! differentiation is irrelevant.
!      ! Note: This indexing requires the 2Der to be stored in the order xx,yy,zz,xy,yz,zx.
!      rates(l) = rates(l) + halvorScRatesDer(i,x,y,z,rate,meshType)*(k(i)-kMesh(x,y,z,i,meshType)) &
!                   +0.5_DBL*halvorScRates2Der(i,x,y,z,rate,meshType)*(k(i)-kMesh(x,y,z,i,meshType))**2 &
!                           +halvorScRates2Der(i+3,x,y,z,rate,meshType) &
!                           *(k(i)-kMesh(x,y,z,i,meshType)) &
!                           *(k(MOD(i,3)+1)-kMesh(x,y,z,MOD(i,3)+1,meshType))
!      ! The index trickery with MOD(i,3)+1 is so we can manage with three iterations of the i-loop. 
!    END DO
  END DO

  DO l=1,8
    ! F&L don't use absolute values here, but I think that's a typo on their part. This works great.
    ! And makes more sense.
    wx=( 1-abs(kx-kMesh(corner(l,1),corner(l,2),corner(l,3),1,meshType))/deltaK(meshType) )
    wy=( 1-abs(ky-kMesh(corner(l,1),corner(l,2),corner(l,3),2,meshType))/deltaK(meshType) )
    wz=( 1-abs(kz-kMesh(corner(l,1),corner(l,2),corner(l,3),3,meshType))/deltaK(meshType) )

    weight(l) = wx*wy*wz
  END DO

  ! And finally sum up the contributions from each corner with the proper weights.
  ratesInterpolate=0._DBL
  DO l=1,8
    ratesInterpolate = ratesInterpolate+weight(l)*rates(l)
  END DO

  END FUNCTION ratesInterpolate

  FUNCTION ratesInterpolateAvg(kx,ky,kz,rate,bandchar)
  IMPLICIT NONE
  REAL(KIND=DBL),   INTENT(IN) :: kx,ky,kz
  INTEGER,          INTENT(IN) :: rate
  CHARACTER(LEN=2), INTENT(IN) :: bandchar
  REAL(KIND=DBL)               :: ratesInterpolateAvg, cubeSize, sizeChange
  INTEGER                      :: l
  LOGICAL                      :: isEmission

  ! Kind of arbitrarily chosen delta. Could probably be optimized somehow. This works fine.
  ! These values could be given by an input argument, if it is found that different rates require
  ! different values.
!  cubeSize   = 0.5e7_DBL
  sizeChange = 5e8_DBL

  IF (SQRT(kx**2+ky**2+kz**2) <= sizeChange) THEN
    cubeSize = 5e6_DBL
  ELSE
    cubeSize = 5e7_DBL
    ! These two rates go completely crazy with the wrong cubeSize.
    IF (rate==1 .OR. rate==2) cubeSize = 4e7
  END IF


  ! Important that ratesInterpolate(kx,ky,kz,rate,band) is called last, as the global variables
  ! meshType and corner are important for the rest of the routine. They must have the values
  ! they obtain from the central function call. This was easier, and more efficient, than
  ! repeating the search from ratesInterpolate in this routine.
  ratesInterpolateAvg =(ratesInterpolate(kx-cubeSize,ky-cubeSize,kz-cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx+cubeSize,ky-cubeSize,kz-cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx+cubeSize,ky+cubeSize,kz-cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx-cubeSize,ky+cubeSize,kz-cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx-cubeSize,ky-cubeSize,kz+cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx+cubeSize,ky-cubeSize,kz+cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx+cubeSize,ky+cubeSize,kz+cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx-cubeSize,ky+cubeSize,kz+cubeSize,rate,bandchar) &
                      + ratesInterpolate(kx,ky,kz,rate,bandchar)) &
                      / 9._DBL

  ! These checks are added because of some of the emission rates, which are 0 until some threshold value.

  ! This loop checks if the given rate is one of these rates.
  ! emList declared in parameters.f90
  DO l=1,nEmList
    IF (rate == emList(l)) THEN
      isEmission = .TRUE.
      EXIT
    END IF
  END DO
  IF (isEmission) THEN
    IF (ktoEinterpolate(kx,ky,kz,bandchar) < hw0/eC) ratesInterpolateAvg = 0._DBL
  END IF
  END FUNCTION

  END MODULE readmodule

  PROGRAM reader
  USE readmodule
  IMPLICIT NONE
  INTEGER            :: x,y,z,points,printRate
  REAL(KIND=DBL)     :: kx,ky,kz,Etest,rates,dk
  CHARACTER(LEN=2)   :: band
  CHARACTER(LEN=3)   :: direction
  CHARACTER(LEN=12)  :: bandout, rateout
  INTEGER, PARAMETER :: coarse=1

  printRate = 13
  band      = 'CH'
  direction = '111'
  bandout   = band(1:LEN_TRIM(band))//'ENE.dat'
  points    = 1000

  CALL loadBandData('SO')
  CALL loadBandData('LH')
  CALL loadBandData('HH')
  CALL loadBandData('CB')
  CALL loadBandData('CS')
  CALL loadBandData('CL')
  CALL loadBandData('CH')

  WRITE(*,*) 'Band data loaded successfully.'

!  CALL loadRates('poabHH', 1, 3)
!  CALL loadRates('poabLL', 2, 2)
!  CALL loadRates('poabHL', 3, 3)
!  CALL loadRates('poabLH', 4, 2)
!  CALL loadRates('poemHH', 5, 3)
!  CALL loadRates('poemLL', 6, 2)
!  CALL loadRates('poemHL', 7, 3)
!  CALL loadRates('poemLH', 8, 2)
!  CALL loadRates('npabHH', 9, 3)
!  CALL loadRates('npabLL',10, 2)
!  CALL loadRates('npabHL',11, 3)
!  CALL loadRates('npabLH',12, 2)
!  CALL loadRates('npemHH',13, 3)
!  CALL loadRates('npemLL',14, 2)
!  CALL loadRates('npemHL',15, 3)
!  CALL loadRates('npemLH',16, 2)
!  CALL loadRates('acabHH',17, 3)
!  CALL loadRates('acabLL',18, 2)
!  CALL loadRates('acabHL',19, 3)
!  CALL loadRates('acabLH',20, 2)
!  CALL loadRates('acemHH',21, 3)
!  CALL loadRates('acemLL',22, 2)
!  CALL loadRates('acemHL',23, 3)
!  CALL loadRates('acemLH',24, 2)

  dk=2*maxWavenumber(coarse)/(points-1)
  OPEN(200, FILE=bandout, STATUS='REPLACE')
  WRITE(*,*) bandout, ' opened'

!  OPEN(400, FILE='rate.dat', STATUS='REPLACE')
!  WRITE(*,*) 'FLrate.dat opened'
  kz=-maxWavenumber(coarse)
  ky=-maxWavenumber(coarse)
  kx=-maxWavenumber(coarse)
  DO z=1,points
    kz = -maxWavenumber(coarse) + dk*(z-1)
    DO y=1,points
      ky = -maxWavenumber(coarse) + dk*(y-1)
      DO x=1,points
        kx = -maxWavenumber(coarse) + dk*(x-1)
        IF ( max(abs(kx),abs(ky),abs(kz)) < maxWavenumber(2) ) THEN
          meshType = 2
        ELSE
          meshType = coarse
        END IF
        IF (direction == '111') THEN
          IF (x==y .AND. y==z) THEN
            Etest = ktoEinterpolate(kx,ky,kz,band)
            ! Choose here if you want the averaged rate results or not.
!            rates = ratesInterpolateAvg(kx,ky,kz,printRate,band)
!            rates = ratesInterpolate(kx,ky,kz,printRate,band)
            !WRITE(200,'(7E23.15)') kx, Etest
            IF (kx <= 0 .OR. ky <= 0 .OR. kz <= 0) THEN
              WRITE(200,'(7E23.15)') -sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)') -sqrt(kx**2+ky**2+kz**2), rates
!              WRITE(400,'(7E23.15)') -Etest, rates
            ELSE
              WRITE(200,'(7E23.15)')  sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)')  sqrt(kx**2+ky**2+kz**2), rates
!              WRITE(400,'(7E23.15)')  Etest, rates
            END IF
          END IF
        ELSE IF (direction == '110') THEN
          IF (x==y .AND. z==(points-1)/2) THEN
            Etest = ktoEinterpolate(kx,ky,kz,band)
            ! Choose here if you want the averaged rate results or not.
!            rates = ratesInterpolateAvg(kx,ky,kz,printRate,band)
!            rates = ratesInterpolate(kx,ky,kz,printRate,band)
            !WRITE(200,'(7E23.15)') kx, Etest
            IF (kx <= 0 .OR. ky <= 0) THEN
              WRITE(200,'(7E23.15)') -sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)') -sqrt(kx**2+ky**2), rates
!              WRITE(400,'(7E23.15)') -Etest, rates
            ELSE
              WRITE(200,'(7E23.15)')  sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)')  sqrt(kx**2+ky**2), rates
!              WRITE(400,'(7E23.15)')  Etest, rates
            END IF
          END IF
        ELSE IF (direction == '100') THEN
          IF (y==(points-1)/2 .AND. z==(points-1)/2) THEN
            Etest = ktoEinterpolate(kx,ky,kz,band)
            ! Choose here if you want the averaged rate results or not.
!            rates = ratesInterpolateAvg(kx,ky,kz,printRate,band)
!            rates = ratesInterpolate(kx,ky,kz,printRate,band)
            IF (kx <= 0) THEN
              WRITE(200,'(7E23.15)') -sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)') kx, rates
!              WRITE(400,'(7E23.15)') Etest, rates
            ELSE
              WRITE(200,'(7E23.15)')  sqrt(kx**2+ky**2+kz**2), Etest
!              WRITE(400,'(7E23.15)') kx, rates
!              WRITE(400,'(7E23.15)') Etest, rates
            END IF
          END IF
        ELSE
          WRITE(*,*) 'Invalid direction.'
          STOP
        END IF
      END DO
    END DO
  END DO

  END PROGRAM reader
