!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!     THE GENERAL BAND LABELS USED BY THIS PROGRAM CORRESPOND TO REAL BANDS
!     AS FOLLOWS:
!     
!     BAND 1: SPLIT-OFF
!     BAND 2: LIGHT HOLE
!     BAND 3: HEAVY HOLE
!     BAND 4: 1ST CONDUCTION BANN
!     BAND 5: UPPER SPLIT-OFF LIKE CONDUCTION BAND
!     BAND 6: UPPER LIGHT HOLE LIKE CONDUCTION BAND
!     BAND 7: UPPER HEAVY HOLE LIKE CONDUCTION BAND
!RECL

!     SEE APPENDIX D IN KARLSENS'S PROJECT THESIS FOR MORE INFORMATION.
!
!     THE FILE I/O IN THIS PROGRAM CONTAINS INFORMATIONS FOR CONVERTING
!     THE OUTPUT FILE FROM ABINIT THORUGH ABINIT.f90, AND FOR COMBINING
!     KPBAND AND ABINIT CALCULATIONS THROUGH ABINITANDKPBAND.f90.
!     THE OUTPUT FILE FROM THIS PROGRAM IS CALLED ABINIT.INI, AND HAS ONE 
!     SINGLE RECORD OF LENGTH 348
!     
!     IN ORDER TO COMPILE THIS PROGRAM WITH IFORT, WRITE THE FOLLOWING:
!
!     ifort halvorinit.f90 ABINITINI.f90 -i8 -o ABINITINI.out
!
!     - Bjoernar Karlsen, April 2013
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 
! 
!                             ABINITINI
! 
!     This is a program for creating or changing the initialization 
!     file ABINIT.INI, which holds the user definable parameters of
!     for ABINIT.
! 
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


!     THIS IS A FUNCTION USED TO FIND THE END OF TEXT STRING 
      FUNCTION LAST(STR)
      INTEGER :: LAST
      CHARACTER(LEN=*) :: STR
 
      LAST=LEN(STR)
10    IF (STR(LAST:LAST).EQ.' '.AND.LAST.GT.1) THEN
          LAST=LAST-1
          GOTO 10
      ENDIF

      END FUNCTION


!     THIS SUBROUTINE CREATES THE INITIALIZATION FILE 
!     WITH SOME DEFAULT VALUES
 
      SUBROUTINE INIT
      USE halvorinit

      INTEGER           :: I
      CHARACTER(LEN=12) :: ABINAMEC,ABINAMEF,ENNAMEA(7)

      OPEN(60,FILE='ABINIT.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='NEW',RECL=360)

      ABINAMEC=' '
      ABINAMEF=' '

      DO I=1,MSH
          ENNAMEA(I)=' '
          ENNAME(I)=' '
          GRNAME(I)=' '
          SDNAME(I)=' '
      END DO

      WRITE (60,REC=1) ABINAMEC,ABINAMEF,ENNAMEA,ENNAME,GRNAME,SDNAME
 
      CLOSE(60)
 
      WRITE (*,*) ' '
      WRITE (*,*) 'File ABINIT.INI is created.'
      WRITE (*,*)

      END SUBROUTINE


      PROGRAM ABINITINI
      USE halvorinit


!     "TECHNICAL" PARAMETERS 
      INTEGER            :: X,Y,Z
      INTEGER, PARAMETER :: N = MS
      INTEGER            :: J,ERROR,CH

!     MENU RELATED VARIABLES
      INTEGER :: L,LAST,I 
      INTEGER, PARAMETER :: ANTMEN=4
      CHARACTER(LEN=12) :: NAM,ABINAMEC,ABINAMEF,ENNAMEA(7)
      CHARACTER :: SELECTOR
      CHARACTER(LEN=70) :: MENU(ANTMEN)
      DATA MENU/'RENAME ABINIT OUTPUT FILE.', &
                'RENAME OUTPUT FILES.', &
                'SHOW SETTINGS.', &
                'SAVE AND QUIT.'/

!     ABORT PROGRAM IF MODEL SIZE MARAMETERS ARE INCONSITSTENT
      IF ((MS.EQ.6).OR.(MS.EQ.8).OR.(MS.EQ.14)) THEN
          IF ((MS/MSH).EQ.2) THEN
              IF ((MSD/MS).EQ.2) THEN
              ELSE
                  WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
                  STOP
              ENDIF
          ELSE
              WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
              STOP
          ENDIF
      ELSE
          WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
          STOP
      ENDIF

      OPEN(50,FILE='ABINIT.INI',ACCESS='DIRECT',FORM='UNFORMATTED',IOSTAT=ERROR,STATUS='OLD',RECL=360)
      CLOSE(50)
      IF (ERROR.NE.0) THEN
          CALL INIT
      ENDIF

      OPEN(50,FILE='ABINIT.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='OLD',RECL=360)
      READ (50,REC=1) ABINAMEC,ABINAMEF,ENNAMEA,ENNAME,GRNAME,SDNAME
      CLOSE(50)

100   WRITE(*,*) ' '
      WRITE(*,*) '*****ABINITINI MAIN MENU*****'
      WRITE(*,*) ' '
      WRITE (*,9900) (I,MENU(I),I=1,ANTMEN)
      WRITE (*,'(/,'' '',A)') 'ENTER SELECTION : '
110   READ (*,*) CH
      IF (CH.LT.1.OR.CH.GT.ANTMEN) THEN
          WRITE (*,*) 'ILLEGAL SELECTION! TRY ONCE MORE. '
          GOTO 110
      ENDIF

      WRITE (*,*) ' '
      GOTO (211,212,213,214) CH


213   WRITE (*,*) '**** SHOW ALL DATA ****'
      WRITE (*,9940) 'Filenames','EnergiesA   ','Energies    ', &
                                 'Gradients   ','Sec. deriv. '
      WRITE (*,9950) '1. Band 1:',ENNAMEA(1),ENNAME(1),GRNAME(1), &
                     SDNAME(1)
      WRITE (*,9950) '2. Band 2:',ENNAMEA(2),ENNAME(2),GRNAME(2), &
                     SDNAME(2)
      WRITE (*,9950) '3. Band 3:',ENNAMEA(3),ENNAME(3),GRNAME(3), &
                     SDNAME(3)
      IF(MSH.GE.4) THEN
          WRITE (*,9950) '4. Band 4:',ENNAMEA(4),ENNAME(4),GRNAME(4), &
                         SDNAME(4)
      ENDIF
      IF(MSH.GE.7) THEN
          WRITE (*,9950) '5. Band 5:',ENNAMEA(5),ENNAME(5),GRNAME(5), &
                         SDNAME(5)
          WRITE (*,9950) '6. Band 6:',ENNAMEA(6),ENNAME(6),GRNAME(6), &
                         SDNAME(6)
          WRITE (*,9950) '7. Band 7:',ENNAMEA(7),ENNAMe(7),GRNAME(7), &
                         SDNAME(7)
      ENDIF
      WRITE (*,*) 'COARSE ABINIT OUTPUT FILE: ',ABINAMEC
      WRITE (*,*) 'FINE ABINIT OUTPUT FILE:   ',ABINAMEF
      WRITE (*,*) ' '

      WRITE (*,*) 'PRESS ENTER TO RETURN...'
      READ  (*,9998) SELECTOR 
      GOTO 100


212   WRITE (*,*) 'OF WHICH BAND SHOULD FILENAMES BE CHANGED?(NONE=0): '
350   READ (*,*) I 
      IF (I.EQ.0) GOTO 100
      IF (I.GT.MSH) THEN
          WRITE (*,*) 'ILLEGAL SELECTION! TRY ONCE MORE. '
          GOTO 350
      ENDIF

      WRITE (*,9995) 'New filenames for band',I
      L=LAST(ENNAME(I))
      WRITE (*,*) 'Enter name of converted ABINIT energy file ['//ENNAMEA(I)(1:L)//'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') ENNAMEA(I)=NAM
      L=LAST(GRNAME(I))
      WRITE (*,*) 'Enter name of final energy file ['//ENNAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') ENNAME(I)=NAM
      L=LAST(SDNAME(I))
      WRITE (*,*) 'Enter name of final gradient file ['//GRNAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') GRNAME(I)=NAM
      L=LAST(VENAME(I))
      WRITE (*,*) 'Enter name of final sec. derivative file ['//SDNAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') SDNAME(I)=NAM
      WRITE (*,*) ' '
      GOTO 213


211   WRITE (*,*) 'Enter name of coarse ABINIT output files: '
      READ (*,9997) NAM
      IF (NAM.NE.' ') ABINAMEC=NAM
      WRITE (*,*) 'Enter name of fine ABINIT output files: '
      READ (*,9997) NAM
      IF (NAM.NE.' ') ABINAMEF=NAM
      WRITE (*,*) ' '
      GOTO 213


214   WRITE (*,*) '**** SAVE AND QUIT ****'
      OPEN(60,FILE='ABINIT.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='OLD',RECL=360)
      WRITE (60,REC=1) ABINAMEC,ABINAMEF,ENNAMEA,ENNAME,GRNAME,SDNAME
      CLOSE(60)
      WRITE (*,*) ' '
      WRITE (*,*) 'DATA WRITTEN TO FILE ABINIT.INI.'


9900  FORMAT(' ',I1,'. ',A)
9910  FORMAT (' ',A,3(T14,3(F10.7,' '),/))
9920  FORMAT (' ',A,I3)
9930  FORMAT (' ',A,F5.3)
9940  FORMAT (' ',A,/,T15,A12,T30,A12,T45,A12,T60,A12)
9950  FORMAT (' ',A,T15,A12,T30,A12,T45,A12,T60,A12)
9960  FORMAT('+',T28,A12) 
9961  FORMAT('+',T41,A12)
9962  FORMAT('+',T54,A12)
9963  FORMAT('+',T67,A12) 
9965  FORMAT('+',T44,I1) 
9966  FORMAT('+',T46,I1) 
9967  FORMAT('+',T48,I1)  
9998  FORMAT(A1)
9997  FORMAT(A12)
9995  FORMAT(' ',A,I1)
9994  FORMAT(' ',A,' ',I1,' ',A) 

      END PROGRAM
