!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!     BE CAREFUL IF USING THIS PROGRAM TO CALCULATE BAND STRUCTURE IN A
!     SEMICONDUCTOR UNDER STRAIN. UNDER CERTAIN TYPES OF STRAIN, THE LIGHT
!     AND HEAVY HOLE WILL CROSS AND THIS LEADS TO THE ALGORITHM BEING UNABLE
!     TO TELL WHICH EIGENVALUE BELONGS TO WHICH BAND. SEE HALVORSEN'S
!     DIPLOMA THESIS, CH. 3.1.1.
!
!     THE GENERAL BAND LABELS USED BY THIS PROGRAM CORRESPOND TO REAL BANDS
!     AS FOLLOWS:
!     
!     BAND 1: SPLIT-OFF
!     BAND 2: LIGHT HOLE
!     BAND 3: HEAVY HOLE
!     BAND 4: 1ST CONDUCTION BAND
!     BAND 5: UPPER SPLIT-OFF LIKE CONDUCTION BAND
!     BAND 6: UPPER LIGHT HOLE LIKE CONDUCTION BAND
!     BAND 7: UPPER HEAVY HOLE LIKE CONDUCTION BAND
!
!     SEE APPENDIX D IN KARLSENS'S DIPLOMA THESIS FOR MORE INFORMATION.
!
!     THE FILE I/O IN THIS PROGRAM IS HARDCODED TO 64 BIT INTEGERS (8 BYTES).
!     OPEN STATEMENTS SPECIFY A RECORD LENGTH GIVEN BY THE TOTAL SIZE
!     (IN BYTES) OF ALL PARAMETERS WRITTEN TO FILE. IN THIS PROGRAM ALL OF
!     THESE ARE CALCULATED USING 1 INTEGER = 8 BYTES. THIS GIVES KPBAND.INI
!     A RECORD LENGTH OF 496. THEREFORE THIS PROGRAM MUST BE COMPILED WITH
!     THE COMPILER FLAG "-i8", TO ENSURE 64 BIT INTEGERS. IF THIS IS NOT DONE,
!     THIS PROGRAM WILL BE UNABLE TO READ THE .INI-FILE PRODUCED BY KPINI.f.
!
!     KPBAND2mkl.f90 MUST ALSO BE LINKED WITH MKL, SO THE COMPILER OPTIONS
!     FOR IFORT COMPOSER XE ARE AS FOLLOWS:
!
!     ifort halvorinit.f90 KPBAND2mkl.f90 /opt/intel/composer_xe_2013/mkl/lib/intel64/libmkl_lapack95_ilp64.a  -Wl,--start-group  /opt/intel/composer_xe_2013/mkl/lib/intel64/libmkl_intel_ilp64.a /opt/intel/composer_xe_2013/mkl/lib/intel64/libmkl_sequential.a /opt/intel/composer_xe_2013/mkl/lib/intel64/libmkl_core.a -Wl,--end-group -lpthread -i8 -I/opt/intel/composer_xe_2013/mkl/include/intel64/ilp64  -I/opt/intel/composer_xe_2013/mkl/include -o kpband2mkl.out

!     - Bjoernar Karlsen, December 2012
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!


 


      SUBROUTINE  EIGEN(K1,K2,K3,E,VR,VI)
      USE Halvorinit    
!     PARAMETERS CONCERNING THE LAPACK ROUTINE ZHEEV.............
      INTEGER :: IFAIL,INFO
      CHARACTER(LEN=1), PARAMETER :: JOBZ='V',UPLO='L'
      INTEGER, PARAMETER :: N=MS,IAR=MS,IAI=MS,IVR=MS,IVI=MS,LDA=MS,LWORK=MSD
      REAL(KIND=DBL) :: AR(IAR,MS),AI(IAI,MS),R(MS),VR(IVR,N),VI(IVI,N),WK1(MS),WK2(MS),WK3(MS),E(MSH)
      
      COMPLEX(KIND=DBL) :: HAM(N,N),WORK(MSD)
      REAL(KIND=DBL) :: RWORK(3*MS+2),W(N)
      
!     PARAMETERS APPEARING IN THE HAMILTONIAN (GENERIC NAMES F,G,H,I).....
!     THE LAST LETTER IN THEIR NAME DECIDE WHETHER IT IS A STRAIN-RELATED 
!     (S) PARAMETER OR A WAVEVECTOR RELATED (K) PARAMETER.
!     THE FIRST LETTER TELLS US IF IT IS THE REAL (R) OR THE IMAGINARY (I)
!     PART OF THE ACTUAL PARAMETER. IMMEDIATELY FOLLOWING THE FIRST LETTER,
!     THE GENERIC NAME (F,G,H,I) APPEARS, THEN THE ABBREVIATION STR (=STAR)
!     MAY FOLLOW, DECIDING WHETHER OR NOT THE PARAMETER IS THE COMPLEX
!     CONJUGATE OF THE GENERIC PARAMETER. F AND G ARE REAL, SO THEY ONLY 
!     NEED THE K OR S EXTENSION. AS AN EXAMPLE, IISTRS IS THE IMAGINARY PART
!     OF THE COMPLEX CONJUGATE OF THE STRAIN-RELATED I-PARAMETER.

      REAL(KIND=DBL) :: FS,GS,FK,GK,RHSTRK,IHSTRK,RHSTRS,IHSTRS,RISTRK,IISTRK,RISTRS,IISTRS

!     "TECHNICAL" PARAMETERS, NOT RELATED TO KP-THEORY. 
      REAL(KIND=DBL) :: EFREE,K1,K2,K3,KX,KY,KZ,K
      REAL(KIND=DBL), PARAMETER :: EFREPR=6.104D-39
      REAL(KIND=DBL), PARAMETER :: RESCALE=1.0D-20
      INTEGER :: I,J

!     THE LAPACK ROUTINE zheev WHICH FINDS EIGENVECTORS AND EIGENVALUES
      EXTERNAL ZHEEV

!     GENERATING REAL-AND IMAGINARY PART OF STRAIN HAMILTONIAN
 
100   FS=(AS+BS/2.0)*(EXX+EYY+EZZ)-3.0*(BS/2.0)*EZZ
      GS=(AS-BS/2.0)*(EXX+EYY+EZZ)+3.0*(BS/2.0)*EZZ
      IHSTRS= DS*EXZ
      RHSTRS=-DS*EYZ
      IISTRS= DS*EXY
      RISTRS= (1.73/2.0)*BS*(EXX-EYY)

      K = SQRT(K1**2+K2**2+K3**2)
      IF (K.NE.0) THEN
          KX= K1/K
          KY= K2/K
          KZ= K3/K
      ELSE
          KX=0
          KY=0
          KZ=0
      ENDIF

!     GENERATING REAL AND IMAGINARY PART OF K-DEPENDENT HAMILTONIAN    

      EFREE=EFREPR*K*K
      FK=EFREE*((A+B/2.0)-3.0*(B/2.0)*KZ*KZ)
      GK=EFREE*((A-B/2.0)+3.0*(B/2.0)*KZ*KZ)
      IHSTRK= D*KX*KZ*EFREE
      RHSTRK=-D*KY*KZ*EFREE
      IISTRK= D*KX*KY*EFREE
      RISTRK= (1.73/2.0)*B*(KX*KX-KY*KY)*EFREE


!     CALCULATING REAL AND IMAGINARY PARTS OF LOWER TRIANGLE IN TOTAL
!     HAMILTONIAN.
!     (ONLY THE LOWER TRIANGLE IS NEEDED, SINCE THE mkl ROUTINE USED
!     IS APPLICABLE ONLY FOR HERMITIAN MATRICES)
!     FIRST THE REAL PART IS CALCULATED:
      
      IF (MSH.GE.7) THEN
          AR(MS-13,MS-13)=E1+E0+DELTA1+EFREE
          AR(MS-12,MS-13)=0.0
          AR(MS-11,MS-13)=0.0
          AR(MS-10,MS-13)=0.0
          AR(MS-9,MS-13)=0.0
          AR(MS-8,MS-13)=0.0
          AR(MS-7,MS-13)=-P1*K2/SQRT(2.0)
          AR(MS-6,MS-13)=0.0
          AR(MS-5,MS-13)=0.0
          AR(MS-4,MS-13)=-Q*K1/SQRT(3.0)
          AR(MS-3,MS-13)=0.0
          AR(MS-2,MS-13)=0.0
          AR(MS-1,MS-13)=Q*K2/SQRT(6.0)
          AR(MS,MS-13)=-Q*K3*SQRT(2.0/3.0)

          AR(MS-12,MS-12)=E1+E0+DELTA1+EFREE
          AR(MS-11,MS-12)=0.0
          AR(MS-10,MS-12)=0.0
          AR(MS-9,MS-12)=0.0
          AR(MS-8,MS-12)=0.0
          AR(MS-7,MS-12)=P1*K3*SQRT(2.0/3.0)
          AR(MS-6,MS-12)=-P1*K1/SQRT(6.0)
          AR(MS-5,MS-12)=-Q*K1/SQRT(3.0)
          AR(MS-4,MS-12)=0.0
          AR(MS-3,MS-12)=0.0
          AR(MS-2,MS-12)=0.0
          AR(MS-1,MS-12)=0.0
          AR(MS,MS-12)=Q*K2/SQRT(2.0)

          AR(MS-11,MS-11)=E1+E0+DELTA1+EFREE
          AR(MS-10,MS-11)=0.0
          AR(MS-9,MS-11)=0.0
          AR(MS-8,MS-11)=0.0
          AR(MS-7,MS-11)=P1*K2/SQRT(6.0)
          AR(MS-6,MS-11)=0.0
          AR(MS-5,MS-11)=0.0
          AR(MS-4,MS-11)=0.0
          AR(MS-3,MS-11)=0.0
          AR(MS-2,MS-11)=Q*K1/SQRT(3.0)
          AR(MS-1,MS-11)=Q*K2/SQRT(2.0)
          AR(MS,MS-11)=0.0

          AR(MS-10,MS-10)=E1+E0+DELTA1+EFREE
          AR(MS-9,MS-10)=0.0
          AR(MS-8,MS-10)=0.0
          AR(MS-7,MS-10)=0.0
          AR(MS-6,MS-10)=-P1*K1/SQRT(2.0)
          AR(MS-5,MS-10)=0.0
          AR(MS-4,MS-10)=0.0
          AR(MS-3,MS-10)=Q*K1/SQRT(3.0)
          AR(MS-2,MS-10)=0.0
          AR(MS-1,MS-10)=Q*K3*SQRT(2.0/3.0)
          AR(MS,MS-10)=Q*K2/SQRT(6.0)

          AR(MS-9,MS-9)=E1+E0+EFREE
          AR(MS-8,MS-9)=0.0
          AR(MS-7,MS-9)=0.0
          AR(MS-6,MS-9)=-P1*K2/SQRT(3.0)
          AR(MS-5,MS-9)=Q*K2/SQRT(6.0)
          AR(MS-4,MS-9)=0.0
          AR(MS-3,MS-9)=Q*K2/SQRT(2.0)
          AR(MS-2,MS-9)=Q*K3*SQRT(2.0/3.0)
          AR(MS-1,MS-9)=0.0
          AR(MS,MS-9)=0.0

          AR(MS-8,MS-8)=E1+E0+EFREE
          AR(MS-7,MS-8)=P1*K1/SQRT(3.0)
          AR(MS-6,MS-8)=-P1*K3/SQRT(3.0)
          AR(MS-5,MS-8)=-Q*K3*SQRT(2.0/3.0)
          AR(MS-4,MS-8)=Q*K2/SQRT(2.0)
          AR(MS-3,MS-8)=0.0
          AR(MS-2,MS-8)=Q*K2/SQRT(6.0)
          AR(MS-1,MS-8)=0.0
          AR(MS,MS-8)=0.0
      ENDIF

      IF (MSH.GE.4) THEN
          AR(MS-7,MS-7)=E0+EFREE
          AR(MS-6,MS-7)=0.0
          AR(MS-5,MS-7)=P0*K1/SQRT(2.0)
          AR(MS-4,MS-7)=0.0
          AR(MS-3,MS-7)=P0*K1/SQRT(6.0)
          AR(MS-2,MS-7)=0.0
          AR(MS-1,MS-7)=P0*K3/SQRT(3.0)
          AR(MS,MS-7)=-P0*K2/SQRT(3.0)

          AR(MS-6,MS-6)=E0+EFREE
          AR(MS-5,MS-6)=0.0
          AR(MS-4,MS-6)=-P0*K2/SQRT(6.0)
          AR(MS-3,MS-6)=P0*K3*SQRT(2.0/3.0)
          AR(MS-2,MS-6)=P0*K2/SQRT(2.0)
          AR(MS-1,MS-6)=P0*K1/SQRT(3.0)
          AR(MS,MS-6)=0.0
      ENDIF

      AR(MS-5,MS-5)=FK+FS
      AR(MS-4,MS-5)=RHSTRK+RHSTRS
      AR(MS-3,MS-5)=RISTRK+RISTRS
      AR(MS-2,MS-5)=0.0
      AR(MS-1,MS-5)=(IHSTRK+IHSTRS)/SQRT(2.0)
      AR(MS,MS-5)=-SQRT(2.0)*(IISTRK+IISTRS)

      AR(MS-4,MS-4)=GK+GS
      AR(MS-3,MS-4)=0.0
      AR(MS-2,MS-4)=RISTRK+RISTRS
      AR(MS-1,MS-4)=0.0
      AR(MS,MS-4)=SQRT(1.50)*(IHSTRK+IHSTRS)

      AR(MS-3,MS-3)=GK+GS
      AR(MS-2,MS-3)=-(RHSTRK+RHSTRS)
      AR(MS-1,MS-3)=SQRT(1.50)*(IHSTRK+IHSTRS)
      AR(MS,MS-3)=0.0

      AR(MS-2,MS-2)=FK+FS
      AR(MS-1,MS-2)=SQRT(2.0)*(IISTRK+IISTRS)
      AR(MS,MS-2)=(IHSTRK+IHSTRS)/SQRT(2.0)

      AR(MS-1,MS-1)=(FK+FS+GK+GS)/2.0-DELTA0
      AR(MS,MS-1)=0.0

      AR(MS,MS)=AR(MS-1,MS-1)
    
!...  THEN THE IMAGINARY PART:..........
      IF (MSH.GE.7) THEN
          AI(MS-13,MS-13)=0.0
          AI(MS-12,MS-13)=0.0
          AI(MS-11,MS-13)=0.0
          AI(MS-10,MS-13)=0.0
          AI(MS-9,MS-13)=0.0
          AI(MS-8,MS-13)=0.0
          AI(MS-7,MS-13)=P1*K1/SQRT(2.0)
          AI(MS-6,MS-13)=0.0
          AI(MS-5,MS-13)=-DELTAQ/3.0
          AI(MS-4,MS-13)=Q*K2/SQRT(3.0)
          AI(MS-3,MS-13)=Q*K3/SQRT(3.0)
          AI(MS-2,MS-13)=0.0
          AI(MS-1,MS-13)=Q*K1/SQRT(6.0)
          AI(MS,MS-13)=0.0

          AI(MS-12,MS-12)=0.0
          AI(MS-11,MS-12)=0.0
          AI(MS-10,MS-12)=0.0
          AI(MS-9,MS-12)=0.0
          AI(MS-8,MS-12)=0.0
          AI(MS-7,MS-12)=0.0
          AI(MS-6,MS-12)=-P1*K2/SQRT(6.0)
          AI(MS-5,MS-12)=-Q*K2/SQRT(3.0)
          AI(MS-4,MS-12)=-DELTAQ/3.0
          AI(MS-3,MS-12)=0.0
          AI(MS-2,MS-12)=Q*K3/SQRT(3.0)
          AI(MS-1,MS-12)=0.0
          AI(MS,MS-12)=Q*K1/SQRT(2.0)

          AI(MS-11,MS-11)=0.0
          AI(MS-10,MS-11)=0.0
          AI(MS-9,MS-11)=0.0
          AI(MS-8,MS-11)=0.0
          AI(MS-7,MS-11)=P1*K1/SQRT(6.0)
          AI(MS-6,MS-11)=P1*K3*SQRT(2.0/3.0)
          AI(MS-5,MS-11)=-Q*K3/SQRT(3.0)
          AI(MS-4,MS-11)=0.0
          AI(MS-3,MS-11)=-DELTAQ/3.0
          AI(MS-2,MS-11)=-Q*K2/SQRT(3.0)
          AI(MS-1,MS-11)=-Q*K1/SQRT(2.0)
          AI(MS,MS-11)=0.0

          AI(MS-10,MS-10)=0.0
          AI(MS-9,MS-10)=0.0
          AI(MS-8,MS-10)=0.0
          AI(MS-7,MS-10)=0.0
          AI(MS-6,MS-10)=P1*K2/SQRT(2.0)
          AI(MS-5,MS-10)=0.0
          AI(MS-4,MS-10)=-Q*K3/SQRT(3.0)
          AI(MS-3,MS-10)=Q*K2/SQRT(3.0)
          AI(MS-2,MS-10)=-DELTAQ/3.0
          AI(MS-1,MS-10)=0.0
          AI(MS,MS-10)=-Q*K1/SQRT(6.0)

          AI(MS-9,MS-9)=0.0
          AI(MS-8,MS-9)=0.0
          AI(MS-7,MS-9)=P1*K3/SQRT(3.0)
          AI(MS-6,MS-9)=P1*K1/SQRT(3.0)
          AI(MS-5,MS-9)=-Q*K1/SQRT(6.0)
          AI(MS-4,MS-9)=0.0
          AI(MS-3,MS-9)=Q*K1/SQRT(2.0)
          AI(MS-2,MS-9)=0.0
          AI(MS-1,MS-9)=DELTAQ*2.0/3.0
          AI(MS,MS-9)=0.0

          AI(MS-8,MS-8)=0.0
          AI(MS-7,MS-8)=-P1*K2/SQRT(3.0)
          AI(MS-6,MS-8)=0.0
          AI(MS-5,MS-8)=0.0
          AI(MS-4,MS-8)=-Q*K1/SQRT(2.0)
          AI(MS-3,MS-8)=0.0
          AI(MS-2,MS-8)=Q*K1/SQRT(6.0)
          AI(MS-1,MS-8)=0.0
          AI(MS,MS-8)=DELTAQ*2.0/3.0
      ENDIF

      IF(MSH.GE.4) THEN
          AI(MS-7,MS-7)=0.0
          AI(MS-6,MS-7)=0.0
          AI(MS-5,MS-7)=-P0*K2/SQRT(2.0)
          AI(MS-4,MS-7)=P0*K3*SQRT(2.0/3.0)
          AI(MS-3,MS-7)=P0*K2/SQRT(6.0)
          AI(MS-2,MS-7)=0.0
          AI(MS-1,MS-7)=0.0
          AI(MS,MS-7)=P0*K1/SQRT(3.0)

          AI(MS-6,MS-6)=0.0
          AI(MS-5,MS-6)=0.0
          AI(MS-4,MS-6)=-P0*K1/SQRT(6.0)
          AI(MS-3,MS-6)=0.0
          AI(MS-2,MS-6)=-P0*K1/SQRT(2.0)
          AI(MS-1,MS-6)=-P0*K2/SQRT(3.0)
          AI(MS,MS-6)=-P0*K3/SQRT(3.0)
      ENDIF

      AI(MS-5,MS-5)=0.0  
      AI(MS-4,MS-5)=IHSTRK+IHSTRS
      AI(MS-3,MS-5)=IISTRK+IISTRS
      AI(MS-2,MS-5)=0.0
      AI(MS-1,MS-5)=-(RHSTRK+RHSTRS)/SQRT(2.0)
      AI(MS,MS-5)=SQRT(2.0)*(RISTRK+RISTRS)

      AI(MS-4,MS-4)=0.0
      AI(MS-3,MS-4)=0.0
      AI(MS-2,MS-4)=IISTRK+IISTRS
      AI(MS-1,MS-4)=-(GK+GS-FK-FS)/SQRT(2.0)
      AI(MS,MS-4)=-SQRT(1.50)*(RHSTRK+RHSTRS)

      AI(MS-3,MS-3)=0.0
      AI(MS-2,MS-3)=-(IHSTRS+IHSTRK)
      AI(MS-1,MS-3)=SQRT(1.5)*(RHSTRK+RHSTRS)
      AI(MS,MS-3)=-(GK+GS-FK-FS)/SQRT(2.0)

      AI(MS-2,MS-2)=0.0
      AI(MS-1,MS-2)=SQRT(2.0)*(RISTRK+RISTRS)
      AI(MS,MS-2)=(RHSTRK+RHSTRS)/SQRT(2.0)

      AI(MS-1,MS-1)=0.0
      AI(MS,MS-1)=0.0

      AI(MS,MS)=0.0

!     SCALING OF THE MATRIX DUE TO THE PRECISION OF THE COMPUTER
      DO I=1,N
         DO J=I,N
            AR(J,I)=AR(J,I)*1D19

            AI(J,I)=AI(J,I)*1D19
            ham(J,I)=DCMPLX(AR(J,I),AI(J,I))
         END DO
      END DO
          
!     THE LAPACK ROUTINE ZHEEV SOLVES THE EIGENVALUE PROBLEM 
      IFAIL=0
      INFO=0 
      
      CALL ZHEEV(JOBZ,UPLO,N,HAM,LDA,W,WORK,LWORK,RWORK,INFO)        
     
      DO I=1,N
         DO J=1,N
            VR(J,I)=DBLE(HAM(J,I))
            VI(J,I)=AIMAG(HAM(J,I))
         END DO
      END DO      

       
!     SCALING OF ENERGIES TO ELECTRONVOLTS
      DO I=1,MSH
          E(I)=-0.50*(W(2*I-1)+W(2*I))/1.60219
!          E(I)=-W(2*I-1)/1.60219
!          E(I)=-W(2*I)/1.60219
      END DO

!     PARABOLIC CORRECTION OF LH- AND HH-BANDS NEAR 1BZ-EDGE
      IF ((K.GE.(0.8*PI/A0)).AND.CORR) THEN
          E(2)=E(2)+LH1*EFREPR*K*K*K*K*RESCALE/1.6021D-19
          E(2)=E(2)+LH2A*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)/(K*K*1.6021D-19)
          E(2)=E(2)+LH2B*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*RESCALE/1.6021D-19
          E(2)=E(2)+LH2C*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*K*K*RESCALE*RESCALE/1.6021D-19
          E(2)=E(2)+LH3A*EFREPR*K1*K1*K2*K2*K3*K3/(K*K*K*K*1.6021D-19)
          E(2)=E(2)+LH3B*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE/(K*K*1.6021D-19)
          E(2)=E(2)+LH3C*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE*RESCALE/1.6021D-19

          E(3)=E(3)+HH1*EFREPR*K*K*K*K*RESCALE/1.6021D-19
          E(3)=E(3)+HH2A*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)/(K*K*1.6021D-19)
          E(3)=E(3)+HH2B*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*RESCALE/1.6021D-19
          E(3)=E(3)+HH2C*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*K*K*RESCALE*RESCALE/1.6021D-19
          E(3)=E(3)+HH3A*EFREPR*K1*K1*K2*K2*K3*K3/(K*K*K*K*1.6021D-19)
          E(3)=E(3)+HH3B*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE/(K*K*1.6021D-19)
          E(3)=E(3)+HH3C*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE*RESCALE/1.6021D-19

          E(4)=E(4)+CB1A*EFREPR*K*K/1.6021D-19
          E(4)=E(4)+CB1B*EFREPR*K*K*K*K*RESCALE/1.6021D-19
          E(4)=E(4)+CB1C*EFREPR*K*K*K*K*K*K*RESCALE*RESCALE/1.6021D-19
          E(4)=E(4)+CB2A*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)/(K*K*1.6021D-19)
          E(4)=E(4)+CB2B*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*RESCALE/1.6021D-19
          E(4)=E(4)+CB2C*EFREPR*(K1*K1*K2*K2+K1*K1*K3*K3+K2*K2*K3*K3)*K*K*RESCALE*RESCALE/1.6021D-19
          E(4)=E(4)+CB3A*EFREPR*K1*K1*K2*K2*K3*K3/(K*K*K*K*1.6021D-19)
          E(4)=E(4)+CB3B*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE/(K*K*1.6021D-19)
          E(4)=E(4)+CB3C*EFREPR*K1*K1*K2*K2*K3*K3*RESCALE*RESCALE/1.6021D-19
          E(4)=E(4)+CB3D*EFREPR*K1*K1*K2*K2*K3*K3*K*K*RESCALE*RESCALE*RESCALE/1.6021D-19
      ENDIF
         
      END SUBROUTINE EIGEN
!... END SUBROUTINE EIGEN   .................................
 


 
 
!     SUBROUTINE CALCULATING GRADIENT OF DISPERSION RELATION.
!     THE GRADIENT IS CALCULATED FROM THE EIGENVECTORS BY USING
!     A RELATION DERIVED FROM KNOWLEDGE OF THE NxN HAMILTONIAN 


      SUBROUTINE GRAD(KX,KY,KZ,VR,VI,EGRAD)
      USE Halvorinit       

      REAL(KIND=DBL) :: KX,KY,KZ,VR(MS,MS),VI(MS,MS)    
      REAL(KIND=DBL) :: EGRAD(MSH,3)
 
!     THE FIRST LETTER TELLS US IF IT IS THE REAL (R) OR THE IMAGINARY (I)
!     PART OF THE ACTUAL PARAMETER. IMMEDIATELY FOLLOWING THE FIRST LETTER,
!     THE GENERIC NAME (F,G,H,I) APPEARS, THEN THE ABBREVIATION ST (=STAR)
!     MAY FOLLOW, DECIDING WHETHER OR NOT THE PARAMETER IS THE COMPLEX
!     CONJUGATE OF THE GENERIC PARAMETER. THE FINAL LETTER DENOTES THAT
!     THEY ARE THE DERIVATIVES OF THE CORRESPOMDIN PARAMETERS IN "EIGEN" 
!     THE INDEX TELLS ALONG WHICH AXIS THE DERIVATIVE IS TAKEN. 

      REAL(KIND=DBL) :: FKD(3),GKD(3),RHSTRD(3),IHSTRD(3),RISTRD(3),IISTRD(3),PKD0(3),PKD1(3),QKD(3),KAX(3)

!     "TECHNICAL" PARAMETERS, NOT RELATED TO KP-THEORY. 
      REAL(KIND=DBL)            :: EFREE,AR(MS,MS,3),AI(MS,MS,3),AK(3),BK(3),K
      REAL(KIND=DBL), PARAMETER :: EFREPR=6.104D-39
      INTEGER :: AX,I,J,BAND
      COMPLEX(KIND=DBL) :: VECT(MS),EGR

      K=SQRT(KX**2+KY**2+KZ**2)

      PKD0(1)=P0
      PKD0(2)=0.0
      PKD0(3)=0.0

      PKD1(1)=P1
      PKD1(2)=0.0
      PKD1(3)=0.0

      QKD(1)=Q
      QKD(2)=0.0
      QKD(3)=0.0

      KAX(1)=KX
      KAX(2)=KY
      KAX(3)=KZ

      FKD(1)=EFREPR*(2*A+B)*KX
      FKD(2)=EFREPR*(2*A+B)*KY
      FKD(3)=EFREPR*2*(A-B)*KZ
 
      GKD(1)=EFREPR*(2*A-B)*KX
      GKD(2)=EFREPR*(2*A-B)*KY
      GKD(3)=EFREPR*2*(A+B)*KZ
 
      IHSTRD(1)=D*EFREPR*KZ
      IHSTRD(2)=0
      IHSTRD(3)=D*EFREPR*KX

      RHSTRD(1)=0
      RHSTRD(2)=-D*EFREPR*KZ
      RHSTRD(3)=-D*EFREPR*KY
 
      IISTRD(1)=D*EFREPR*KY
      IISTRD(2)=D*EFREPR*KX
      IISTRD(3)=0
 
      RISTRD(1)=SQRT(3.0)*B*EFREPR*KX
      RISTRD(2)=-SQRT(3.0)*B*EFREPR*KY
      RISTRD(3)=0
 
      DO AX=1,3
    
!     FIRST THE REAL PART IS CALCULATED:

         IF (MSH.GE.7) THEN
             AR(MS-13,MS-13,AX)=2*EFREPR*KAX(AX)
             AR(MS-12,MS-13,AX)=0.0
             AR(MS-11,MS-13,AX)=0.0
             AR(MS-10,MS-13,AX)=0.0
             AR(MS-9,MS-13,AX)=0.0
             AR(MS-8,MS-13,AX)=0.0
             AR(MS-7,MS-13,AX)=-PKD1(MOD(AX+1,3)+1)/SQRT(2.0)
             AR(MS-6,MS-13,AX)=0.0
             AR(MS-5,MS-13,AX)=0.0
             AR(MS-4,MS-13,AX)=-QKD(AX)/SQRT(3.0)
             AR(MS-3,MS-13,AX)=0.0
             AR(MS-2,MS-13,AX)=0.0
             AR(MS-1,MS-13,AX)=QKD(MOD(AX+1,3)+1)/SQRT(6.0)
             AR(MS,MS-13,AX)=-QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)

             AR(MS-12,MS-12,AX)=2*EFREPR*KAX(AX)
             AR(MS-11,MS-12,AX)=0.0
             AR(MS-10,MS-12,AX)=0.0
             AR(MS-9,MS-12,AX)=0.0
             AR(MS-8,MS-12,AX)=0.0
             AR(MS-7,MS-12,AX)=PKD1(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AR(MS-6,MS-12,AX)=-PKD1(AX)/SQRT(6.0)
             AR(MS-5,MS-12,AX)=-QKD(AX)/SQRT(3.0)
             AR(MS-4,MS-12,AX)=0.0
             AR(MS-3,MS-12,AX)=0.0
             AR(MS-2,MS-12,AX)=0.0
             AR(MS-1,MS-12,AX)=0.0
             AR(MS,MS-12,AX)=QKD(MOD(AX+1,3)+1)/SQRT(2.0)

             AR(MS-11,MS-11,AX)=2*EFREPR*KAX(AX)
             AR(MS-10,MS-11,AX)=0.0
             AR(MS-9,MS-11,AX)=0.0
             AR(MS-8,MS-11,AX)=0.0
             AR(MS-7,MS-11,AX)=PKD1(MOD(AX+1,3)+1)/SQRT(6.0)
             AR(MS-6,MS-11,AX)=0.0
             AR(MS-5,MS-11,AX)=0.0
             AR(MS-4,MS-11,AX)=0.0
             AR(MS-3,MS-11,AX)=0.0
             AR(MS-2,MS-11,AX)=QKD(AX)/SQRT(3.0)
             AR(MS-1,MS-11,AX)=QKD(MOD(AX+1,3)+1)/SQRT(2.0)
             AR(MS,MS-11,AX)=0.0

             AR(MS-10,MS-10,AX)=2*EFREPR*KAX(AX)
             AR(MS-9,MS-10,AX)=0.0
             AR(MS-8,MS-10,AX)=0.0
             AR(MS-7,MS-10,AX)=0.0
             AR(MS-6,MS-10,AX)=-PKD1(AX)/SQRT(2.0)
             AR(MS-5,MS-10,AX)=0.0
             AR(MS-4,MS-10,AX)=0.0
             AR(MS-3,MS-10,AX)=QKD(AX)/SQRT(3.0)
             AR(MS-2,MS-10,AX)=0.0
             AR(MS-1,MS-10,AX)=QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AR(MS,MS-10,AX)=QKD(MOD(AX+1,3)+1)/SQRT(6.0)

             AR(MS-9,MS-9,AX)=2*EFREPR*KAX(AX)
             AR(MS-8,MS-9,AX)=0.0
             AR(MS-7,MS-9,AX)=0.0
             AR(MS-6,MS-9,AX)=-PKD1(MOD(AX+1,3)+1)/SQRT(3.0)
             AR(MS-5,MS-9,AX)=QKD(MOD(AX+1,3)+1)/SQRT(6.0)
             AR(MS-4,MS-9,AX)=0.0
             AR(MS-3,MS-9,AX)=QKD(MOD(AX+1,3)+1)/SQRT(2.0)
             AR(MS-2,MS-9,AX)=QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AR(MS-1,MS-9,AX)=0.0
             AR(MS,MS-9,AX)=0.0

             AR(MS-8,MS-8,AX)=2*EFREPR*KAX(AX)
             AR(MS-7,MS-8,AX)=PKD1(AX)/SQRT(3.0)
             AR(MS-6,MS-8,AX)=-PKD1(MOD(AX,3)+1)/SQRT(3.0)
             AR(MS-5,MS-8,AX)=-QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AR(MS-4,MS-8,AX)=QKD(MOD(AX+1,3)+1)/SQRT(2.0)
             AR(MS-3,MS-8,AX)=0.0
             AR(MS-2,MS-8,AX)=QKD(MOD(AX+1,3)+1)/SQRT(6.0)
             AR(MS-1,MS-8,AX)=0.0
             AR(MS,MS-8,AX)=0.0
         ENDIF

         IF (MSH.GE.4) THEN
             AR(MS-7,MS-7,AX)=2*EFREPR*KAX(AX)
             AR(MS-6,MS-7,AX)=0.0
             AR(MS-5,MS-7,AX)=PKD0(AX)/SQRT(2.0)
             AR(MS-4,MS-7,AX)=0.0
             AR(MS-3,MS-7,AX)=PKD0(AX)/SQRT(6.0)
             AR(MS-2,MS-7,AX)=0.0
             AR(MS-1,MS-7,AX)=PKD0(MOD(AX,3)+1)/SQRT(3.0)
             AR(MS,MS-7,AX)=-PKD0(MOD(AX+1,3)+1)/SQRT(3.0)

             AR(MS-6,MS-6,AX)=2*EFREPR*KAX(AX)
             AR(MS-5,MS-6,AX)=0.0
             AR(MS-4,MS-6,AX)=-PKD0(MOD(AX+1,3)+1)/SQRT(6.0)
             AR(MS-3,MS-6,AX)=PKD0(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AR(MS-2,MS-6,AX)=PKD0(MOD(AX+1,3)+1)/SQRT(2.0)
             AR(MS-1,MS-6,AX)=PKD0(AX)/SQRT(3.0)
             AR(MS,MS-6,AX)=0.0
         ENDIF

         AR(MS-5,MS-5,AX)=FKD(AX)
         AR(MS-4,MS-5,AX)=RHSTRD(AX)
         AR(MS-3,MS-5,AX)=RISTRD(AX)
         AR(MS-2,MS-5,AX)=0.0
         AR(MS-1,MS-5,AX)=IHSTRD(AX)/SQRT(2.0)
         AR(MS,MS-5,AX)=-SQRT(2.0)*IISTRD(AX)

         AR(MS-4,MS-4,AX)=GKD(AX)
         AR(MS-3,MS-4,AX)=0.0
         AR(MS-2,MS-4,AX)=RISTRD(AX)
         AR(MS-1,MS-4,AX)=0.0
         AR(MS,MS-4,AX)=SQRT(1.50)*IHSTRD(AX)

         AR(MS-3,MS-3,AX)=GKD(AX)
         AR(MS-2,MS-3,AX)=-RHSTRD(AX)
         AR(MS-1,MS-3,AX)=SQRT(1.50)*IHSTRD(AX)
         AR(MS,MS-3,AX)=0.0

         AR(MS-2,MS-2,AX)=FKD(AX)
         AR(MS-1,MS-2,AX)=SQRT(2.0)*IISTRD(AX)
         AR(MS,MS-2,AX)=IHSTRD(AX)/SQRT(2.0)

         AR(MS-1,MS-1,AX)=(FKD(AX)+GKD(AX))/2.0
         AR(MS,MS-1,AX)=0.0

         AR(MS,MS,AX)=AR(MS-1,MS-1,AX)

!        THEN THE IMAGINARY PART:

         IF (MSH.GE.7) THEN
             AI(MS-13,MS-13,AX)=0.0
             AI(MS-12,MS-13,AX)=0.0
             AI(MS-11,MS-13,AX)=0.0
             AI(MS-10,MS-13,AX)=0.0
             AI(MS-9,MS-13,AX)=0.0
             AI(MS-8,MS-13,AX)=0.0
             AI(MS-7,MS-13,AX)=PKD1(AX)/SQRT(2.0)
             AI(MS-6,MS-13,AX)=0.0
             AI(MS-5,MS-13,AX)=0.0
             AI(MS-4,MS-13,AX)=QKD(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS-3,MS-13,AX)=QKD(MOD(AX,3)+1)/SQRT(3.0)
             AI(MS-2,MS-13,AX)=0.0
             AI(MS-1,MS-13,AX)=QKD(AX)/SQRT(6.0)
             AI(MS,MS-13,AX)=0.0

             AI(MS-12,MS-12,AX)=0.0
             AI(MS-11,MS-12,AX)=0.0
             AI(MS-10,MS-12,AX)=0.0
             AI(MS-9,MS-12,AX)=0.0
             AI(MS-8,MS-12,AX)=0.0
             AI(MS-7,MS-12,AX)=0.0
             AI(MS-6,MS-12,AX)=-PKD1(MOD(AX+1,3)+1)/SQRT(6.0)
             AI(MS-5,MS-12,AX)=-QKD(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS-4,MS-12,AX)=0.0
             AI(MS-3,MS-12,AX)=0.0
             AI(MS-2,MS-12,AX)=QKD(MOD(AX,3)+1)/SQRT(3.0)
             AI(MS-1,MS-12,AX)=0.0
             AI(MS,MS-12,AX)=-QKD(AX)/SQRT(2.0)

             AI(MS-11,MS-11,AX)=0.0
             AI(MS-10,MS-11,AX)=0.0
             AI(MS-9,MS-11,AX)=0.0
             AI(MS-8,MS-11,AX)=0.0
             AI(MS-7,MS-11,AX)=PKD1(AX)/SQRT(6.0)
             AI(MS-6,MS-11,AX)=PKD1(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AI(MS-5,MS-11,AX)=-QKD(MOD(AX,3)+1)/SQRT(3.0)
             AI(MS-4,MS-11,AX)=0.0
             AI(MS-3,MS-11,AX)=0.0
             AI(MS-2,MS-11,AX)=-QKD(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS-1,MS-11,AX)=-QKD(AX)/SQRT(2.0)
             AI(MS,MS-11,AX)=0.0

             AI(MS-10,MS-10,AX)=0.0
             AI(MS-9,MS-10,AX)=0.0
             AI(MS-8,MS-10,AX)=0.0
             AI(MS-7,MS-10,AX)=0.0
             AI(MS-6,MS-10,AX)=PKD1(MOD(AX+1,3)+1)/SQRT(2.0)
             AI(MS-5,MS-10,AX)=0.0
             AI(MS-4,MS-10,AX)=-QKD(MOD(AX,3)+1)/SQRT(3.0)
             AI(MS-3,MS-10,AX)=QKD(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS-2,MS-10,AX)=0.0
             AI(MS-1,MS-10,AX)=0.0
             AI(MS,MS-10,AX)=-QKD(AX)/SQRT(6.0)

             AI(MS-9,MS-9,AX)=0.0
             AI(MS-8,MS-9,AX)=0.0
             AI(MS-7,MS-9,AX)=PKD1(MOD(AX,3)+1)/SQRT(3.0)
             AI(MS-6,MS-9,AX)=PKD1(AX)/SQRT(3.0)
             AI(MS-5,MS-9,AX)=-QKD(AX)/SQRT(6.0)
             AI(MS-4,MS-9,AX)=0.0
             AI(MS-3,MS-9,AX)=QKD(AX)/SQRT(2.0)
             AI(MS-2,MS-9,AX)=0.0
             AI(MS-1,MS-9,AX)=0.0
             AI(MS,MS-9,AX)=0.0

             AI(MS-8,MS-8,AX)=0.0
             AI(MS-7,MS-8,AX)=-PKD1(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS-6,MS-8,AX)=0.0
             AI(MS-5,MS-8,AX)=0.0
             AI(MS-4,MS-8,AX)=-QKD(AX)/SQRT(2.0)
             AI(MS-3,MS-8,AX)=0.0
             AI(MS-2,MS-8,AX)=QKD(AX)/SQRT(6.0)
             AI(MS-1,MS-8,AX)=0.0
             AI(MS,MS-8,AX)=0.0
         ENDIF

         IF (MSH.GE.4) THEN
             AI(MS-7,MS-7,AX)=0.0
             AI(MS-6,MS-7,AX)=0.0
             AI(MS-5,MS-7,AX)=-PKD0(MOD(AX+1,3)+1)/SQRT(2.0)
             AI(MS-4,MS-7,AX)=PKD0(MOD(AX,3)+1)*SQRT(2.0/3.0)
             AI(MS-3,MS-7,AX)=PKD0(MOD(AX+1,3)+1)/SQRT(6.0)
             AI(MS-2,MS-7,AX)=0.0
             AI(MS-1,MS-7,AX)=0.0
             AI(MS,MS-7,AX)=PKD0(AX)/SQRT(3.0)

             AI(MS-6,MS-6,AX)=0.0
             AI(MS-5,MS-6,AX)=0.0
             AI(MS-4,MS-6,AX)=-PKD0(AX)/SQRT(6.0)
             AI(MS-3,MS-6,AX)=0.0
             AI(MS-2,MS-6,AX)=-PKD0(AX)/SQRT(2.0)
             AI(MS-1,MS-6,AX)=-PKD0(MOD(AX+1,3)+1)/SQRT(3.0)
             AI(MS,MS-6,AX)=-PKD0(MOD(AX,3)+1)/SQRT(3.0)
         ENDIF

         AI(MS-5,MS-5,AX)=0.0  
         AI(MS-4,MS-5,AX)=IHSTRD(AX)
         AI(MS-3,MS-5,AX)=IISTRD(AX)
         AI(MS-2,MS-5,AX)=0.0
         AI(MS-1,MS-5,AX)=-RHSTRD(AX)/SQRT(2.0)
         AI(MS,MS-5,AX)=SQRT(2.0)*RISTRD(AX)

         AI(MS-4,MS-4,AX)=0.0
         AI(MS-3,MS-4,AX)=0.0
         AI(MS-2,MS-4,AX)=IISTRD(AX)
         AI(MS-1,MS-4,AX)=-(GKD(AX)-FKD(AX))/SQRT(2.0)
         AI(MS,MS-4,AX)=-SQRT(1.50)*RHSTRD(AX)

         AI(MS-3,MS-3,AX)=0.0
         AI(MS-2,MS-3,AX)=-IHSTRD(AX)
         AI(MS-1,MS-3,AX)=SQRT(1.5)*RHSTRD(AX)
         AI(MS,MS-3,AX)=-(GKD(AX)-FKD(AX))/SQRT(2.0)

         AI(MS-2,MS-2,AX)=0.0
         AI(MS-1,MS-2,AX)=SQRT(2.0)*RISTRD(AX)
         AI(MS,MS-2,AX)=RHSTRD(AX)/SQRT(2.0)

         AI(MS-1,MS-1,AX)=0.0
         AI(MS,MS-1,AX)=0.0

         AI(MS,MS,AX)=0.0
      END DO
 
!     FILL ENTIRE MATRIX
      DO AX=1,3
         DO I=1,MS
            DO J=I+1,MS
               AR(I,J,AX)=AR(J,I,AX)

               AI(I,J,AX)=-AI(J,I,AX)
            END DO
         END DO
      END DO
     
!     CALCULATE GRADIENT 
 
      DO AX=1,3
         DO BAND=1,(MS-1),2

            DO I=1,MS
               VECT(I)=(0,0)
            END DO

            DO I=1,MS
               DO J=1,MS
                  VECT(I)=VECT(I)+DCMPLX(AR(I,J,AX),AI(I,J,AX))*DCMPLX(VR(J,BAND),VI(J,BAND))
               END DO
            END DO
            EGR=(0,0)
         
            DO I=1,MS
               EGR=EGR+VECT(I)*DCMPLX(VR(I,BAND),-VI(I,BAND))
            END DO
            EGRAD((BAND+1)/2,AX)=-DBLE(EGR)/1.60219D-19
         END DO
         IF ((K.GE.(0.8*PI/A0)).AND.CORR) THEN
            EGRAD(2,AX)=EGRAD(2,AX)+4.0*LH1*EFREPR*K*K*KAX(AX)*RESCALE/1.6021D-19

            EGRAD(2,AX)=EGRAD(2,AX)+LH2A*EFREPR*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K-KAX(AX)*KAX(AX))-2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*1.6021D-19)
            EGRAD(2,AX)=EGRAD(2,AX)+LH2B*EFREPR*RESCALE*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/1.6021D-19
            EGRAD(2,AX)=EGRAD(2,AX)+LH2C*EFREPR*RESCALE*RESCALE*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K+KAX(AX)*KAX(AX))+2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/1.6021D-19

            EGRAD(2,AX)=EGRAD(2,AX)+LH3A*EFREPR*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-4.0*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*1.6021D-19)
            EGRAD(2,AX)=EGRAD(2,AX)+LH3B*EFREPR*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-2.0*KAX(AX)*KAX(AX))/(K*K*K*K*1.6021D-19)
            EGRAD(2,AX)=EGRAD(2,AX)+2.0*LH3C*EFREPR*RESCALE*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/1.6021D-19


            EGRAD(3,AX)=EGRAD(3,AX)+4.0*HH1*EFREPR*K*K*KAX(AX)*RESCALE/1.6021D-19

            EGRAD(3,AX)=EGRAD(3,AX)+HH2A*EFREPR*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K-KAX(AX)*KAX(AX))-2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*1.6021D-19)
            EGRAD(3,AX)=EGRAD(3,AX)+HH2B*EFREPR*RESCALE*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/1.6021D-19
            EGRAD(3,AX)=EGRAD(3,AX)+HH2C*EFREPR*RESCALE*RESCALE*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K+KAX(AX)*KAX(AX))+2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/1.6021D-19

            EGRAD(3,AX)=EGRAD(3,AX)+HH3A*EFREPR*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-4.0*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*1.6021D-19)
            EGRAD(3,AX)=EGRAD(3,AX)+HH3B*EFREPR*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-2.0*KAX(AX)*KAX(AX))/(K*K*K*K*1.6021D-19)
            EGRAD(3,AX)=EGRAD(3,AX)+2.0*HH3C*EFREPR*RESCALE*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/1.6021D-19


            EGRAD(4,AX)=EGRAD(4,AX)+2.0*CB1A*EFREPR*KAX(AX)/1.6021D-19
            EGRAD(4,AX)=EGRAD(4,AX)+4.0*CB1B*EFREPR*K*K*KAX(AX)*RESCALE/1.6021D-19
            EGRAD(4,AX)=EGRAD(4,AX)+6.0*CB1C*EFREPR*K*K*K*K*KAX(AX)*RESCALE*RESCALE/1.6021D-19

            EGRAD(4,AX)=EGRAD(4,AX)+CB2A*EFREPR*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K-KAX(AX)*KAX(AX))-2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*1.6021D-19)
            EGRAD(4,AX)=EGRAD(4,AX)+CB2B*EFREPR*RESCALE*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/1.6021D-19
            EGRAD(4,AX)=EGRAD(4,AX)+CB2C*EFREPR*RESCALE*RESCALE*KAX(AX)*(2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))*(K*K+KAX(AX)*KAX(AX))+2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/1.6021D-19

            EGRAD(4,AX)=EGRAD(4,AX)+CB3A*EFREPR*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-4.0*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*1.6021D-19)
            EGRAD(4,AX)=EGRAD(4,AX)+CB3B*EFREPR*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K-2.0*KAX(AX)*KAX(AX))/(K*K*K*K*1.6021D-19)
            EGRAD(4,AX)=EGRAD(4,AX)+2.0*CB3C*EFREPR*RESCALE*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/1.6021D-19
            EGRAD(4,AX)=EGRAD(4,AX)+CB3D*EFREPR*RESCALE*RESCALE*RESCALE*KAX(AX)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K+2.0*KAX(AX)*KAX(AX))/1.6021D-19
         ENDIF
      END DO

      END SUBROUTINE GRAD
 
 
!----------------------------------------------------------------------C 
!     THIS SUBROUTINE CALCULATES THE SECOND DERIVATIVES OF THE ENERGY  C 
!     FOR EACH BAND. IT IS DONE BY EXPRESSIONS BASED ON EIGENVECTORS,  C
!     EIGENVALUES AND THE FIRST AND SECOND DERIVATIVE OF NxN-          C
!     HAMILTONIAN MATRIX.                                              C
!     THE SECOND DERIVATIVES ARE RETURNED IN THE FOLLOWING ORDER:      C
!                       XX,YY,ZZ,XY,XZ,YZ                              C
!----------------------------------------------------------------------C 


      SUBROUTINE SECNDD(KX,KY,KZ,VR,VI,E,SD)
      USE Halvorinit 

      REAL(KIND=DBL) :: KX,KY,KZ,VR(MS,MS),VI(MS,MS),E(MSH),SD(6,MSH)    
!      REAL SD(6,3)
 
!     THE FIRST LETTER TELLS US IF IT IS THE REAL (R) OR THE IMAGINARY (I)
!     PART OF THE ACTUAL PARAMETER. IMMEDIATELY FOLLOWING THE FIRST LETTER,
!     THE GENERIC NAME (F,G,H,I) APPEARS, THEN THE ABBREVIATION STR (=STAR)
!     MAY FOLLOW, DECIDING WHETHER OR NOT THE PARAMETER IS THE COMPLEX
!     CONJUGATE OF THE GENERIC PARAMETER. THE FINAL LETTER DENOTES THAT
!     THEY ARE THE DERIVATIVES OF THE CORRESPONDING PARAMETERS IN "EIGEN" 
!     THE INDEX TELLS ALONG WHICH AXIS THE DERIVATIVE IS TAKEN. 

      REAL(KIND=DBL) :: PKD0(3),PKD1(3),QKD(3),KAX(3),FKD(3),GKD(3),RHSTRD(3),IHSTRD(3),RISTRD(3),IISTRD(3)
      REAL(KIND=DBL) :: KKSD(6),FKSD(6),GKSD(6),RHSTSD(6),IHSTSD(6),RISTSD(6),IISTSD(6)
 

!     DEFORMATION POTENTIAL AND VALENCE BAND PARAMETERS........
      REAL(KIND=DBL), PARAMETER :: EV = 1.60219D-19


!     "TECHNICAL" PARAMETERS, NOT RELATED TO KP-THEORY. 
      REAL(KIND=DBL) :: EFREE,AR(MS,MS,3),AI(MS,MS,3),K
      REAL(KIND=DBL), PARAMETER :: EFREPR = 6.104D-39
      REAL(KIND=DBL) :: BR(MS,MS,6),BI(MS,MS,6),EN,EO(MS-2)
      INTEGER :: AX,I,J,BAND,S1,S2,O(MS-2),NONDEG
      COMPLEX(KIND=DBL) :: VECT(MS),TEMP,V1(MS),V2(MS),TEMPA,TEMPB,TEMPC
      REAL(KIND=DBL), PARAMETER :: ZERLIM = 1D-30

      K=SQRT(KX*KX+KY*KY+KZ*KZ)

!     GENERATE FIRST DERIVATIVE OF HAMILTONIAN 

      PKD0(1)=P0
      PKD0(2)=0.0
      PKD0(3)=0.0

      PKD1(1)=P1
      PKD1(2)=0.0
      PKD1(3)=0.0

      QKD(1)=Q
      QKD(2)=0.0
      QKD(3)=0.0

      KAX(1)=KX
      KAX(2)=KY
      KAX(3)=KZ

      FKD(1)=EFREPR*(2*A+B)*KX
      FKD(2)=EFREPR*(2*A+B)*KY
      FKD(3)=EFREPR*2*(A-B)*KZ
 
      GKD(1)=EFREPR*(2*A-B)*KX
      GKD(2)=EFREPR*(2*A-B)*KY
      GKD(3)=EFREPR*2*(A+B)*KZ
 
      IHSTRD(1)=D*EFREPR*KZ
      IHSTRD(2)=0
      IHSTRD(3)=D*EFREPR*KX

      RHSTRD(1)=0
      RHSTRD(2)=-D*EFREPR*KZ
      RHSTRD(3)=-D*EFREPR*KY
 !        THEN THE IMAGINARY PART:
      IISTRD(1)=D*EFREPR*KY
      IISTRD(2)=D*EFREPR*KX
      IISTRD(3)=0
 
      RISTRD(1)=SQRT(3.0)*B*EFREPR*KX
      RISTRD(2)=-SQRT(3.0)*B*EFREPR*KY
      RISTRD(3)=0
 
 
!     (THE MATRIX IS SCALED BY AN ELECTRONVOLT)
      DO AX=1,3
    
!     FIRST THE REAL PART IS CALCULATED:

          IF (MSH.GE.7) THEN
              AR(MS-13,MS-13,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-12,MS-13,AX)=0.0
              AR(MS-11,MS-13,AX)=0.0
              AR(MS-10,MS-13,AX)=0.0
              AR(MS-9,MS-13,AX)=0.0
              AR(MS-8,MS-13,AX)=0.0
              AR(MS-7,MS-13,AX)=-PKD1(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
              AR(MS-6,MS-13,AX)=0.0
              AR(MS-5,MS-13,AX)=0.0
              AR(MS-4,MS-13,AX)=-QKD(AX)/(SQRT(3.0)*EV)
              AR(MS-3,MS-13,AX)=0.0
              AR(MS-2,MS-13,AX)=0.0
              AR(MS-1,MS-13,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
              AR(MS,MS-13,AX)=-QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV

              AR(MS-12,MS-12,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-11,MS-12,AX)=0.0
              AR(MS-10,MS-12,AX)=0.0
              AR(MS-9,MS-12,AX)=0.0
              AR(MS-8,MS-12,AX)=0.0
              AR(MS-7,MS-12,AX)=PKD1(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
              AR(MS-6,MS-12,AX)=-PKD1(AX)/(SQRT(6.0)*EV)
              AR(MS-5,MS-12,AX)=-QKD(AX)/(SQRT(3.0)*EV)
              AR(MS-4,MS-12,AX)=0.0
              AR(MS-3,MS-12,AX)=0.0
              AR(MS-2,MS-12,AX)=0.0
              AR(MS-1,MS-12,AX)=0.0
              AR(MS,MS-12,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)

              AR(MS-11,MS-11,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-10,MS-11,AX)=0.0
              AR(MS-9,MS-11,AX)=0.0
              AR(MS-8,MS-11,AX)=0.0
              AR(MS-7,MS-11,AX)=PKD1(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
              AR(MS-6,MS-11,AX)=0.0
              AR(MS-5,MS-11,AX)=0.0
              AR(MS-4,MS-11,AX)=0.0
              AR(MS-3,MS-11,AX)=0.0
              AR(MS-2,MS-11,AX)=QKD(AX)/(SQRT(3.0)*EV)
              AR(MS-1,MS-11,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
              AR(MS,MS-11,AX)=0.0

              AR(MS-10,MS-10,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-9,MS-10,AX)=0.0
              AR(MS-8,MS-10,AX)=0.0
              AR(MS-7,MS-10,AX)=0.0
              AR(MS-6,MS-10,AX)=-PKD1(AX)/(SQRT(2.0)*EV)
              AR(MS-5,MS-10,AX)=0.0
              AR(MS-4,MS-10,AX)=0.0
              AR(MS-3,MS-10,AX)=QKD(AX)/(SQRT(3.0)*EV)
              AR(MS-2,MS-10,AX)=0.0
              AR(MS-1,MS-10,AX)=QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
              AR(MS,MS-10,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)

              AR(MS-9,MS-9,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-8,MS-9,AX)=0.0
              AR(MS-7,MS-9,AX)=0.0
              AR(MS-6,MS-9,AX)=-PKD1(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AR(MS-5,MS-9,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
              AR(MS-4,MS-9,AX)=0.0
              AR(MS-3,MS-9,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
              AR(MS-2,MS-9,AX)=QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
              AR(MS-1,MS-9,AX)=0.0
              AR(MS,MS-9,AX)=0.0

              AR(MS-8,MS-8,AX)=2*EFREPR*KAX(AX)/EV
              AR(MS-7,MS-8,AX)=PKD1(AX)/(SQRT(3.0)*EV)
              AR(MS-6,MS-8,AX)=-PKD1(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AR(MS-5,MS-8,AX)=-QKD(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
              AR(MS-4,MS-8,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
              AR(MS-3,MS-8,AX)=0.0
              AR(MS-2,MS-8,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
              AR(MS-1,MS-8,AX)=0.0
              AR(MS,MS-8,AX)=0.0
         ENDIF

         IF (MSH.GE.4) THEN
             AR(MS-7,MS-7,AX)=2*EFREPR*KAX(AX)/EV
             AR(MS-6,MS-7,AX)=0.0
             AR(MS-5,MS-7,AX)=PKD0(AX)/(SQRT(2.0)*EV)
             AR(MS-4,MS-7,AX)=0.0
             AR(MS-3,MS-7,AX)=PKD0(AX)/(SQRT(6.0)*EV)
             AR(MS-2,MS-7,AX)=0.0
             AR(MS-1,MS-7,AX)=PKD0(MOD(AX,3)+1)/(SQRT(3.0)*EV)
             AR(MS,MS-7,AX)=-PKD0(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)

             AR(MS-6,MS-6,AX)=2*EFREPR*KAX(AX)/EV
             AR(MS-5,MS-6,AX)=0.0
             AR(MS-4,MS-6,AX)=-PKD0(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
             AR(MS-3,MS-6,AX)=PKD0(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
             AR(MS-2,MS-6,AX)=PKD0(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
             AR(MS-1,MS-6,AX)=PKD0(AX)/(SQRT(3.0)*EV)
             AR(MS,MS-6,AX)=0.0
         ENDIF

         AR(MS-5,MS-5,AX)=FKD(AX)/EV
         AR(MS-4,MS-5,AX)=RHSTRD(AX)/EV
         AR(MS-3,MS-5,AX)=RISTRD(AX)/EV
         AR(MS-2,MS-5,AX)=0.0
         AR(MS-1,MS-5,AX)=IHSTRD(AX)/(SQRT(2.0)*EV)
         AR(MS,MS-5,AX)=-SQRT(2.0)*IISTRD(AX)/EV

         AR(MS-4,MS-4,AX)=GKD(AX)/EV
         AR(MS-3,MS-4,AX)=0.0
         AR(MS-2,MS-4,AX)=RISTRD(AX)/EV
         AR(MS-1,MS-4,AX)=0.0
         AR(MS,MS-4,AX)=SQRT(1.50)*IHSTRD(AX)/EV

         AR(MS-3,MS-3,AX)=GKD(AX)/EV
         AR(MS-2,MS-3,AX)=-RHSTRD(AX)/EV
         AR(MS-1,MS-3,AX)=SQRT(1.50)*IHSTRD(AX)/EV
         AR(MS,MS-3,AX)=0.0


         AR(MS-2,MS-2,AX)=FKD(AX)/EV
         AR(MS-1,MS-2,AX)=SQRT(2.0)*IISTRD(AX)/EV
         AR(MS,MS-2,AX)=IHSTRD(AX)/(SQRT(2.0)*EV)

         AR(MS-1,MS-1,AX)=(FKD(AX)+GKD(AX))/(2.0*EV)
         AR(MS,MS-1,AX)=0.0

         AR(MS,MS,AX)=AR(MS-1,MS-1,AX)

!        THEN THE IMAGINARY PART:

          IF (MSH.GE.7) THEN
              AI(MS-13,MS-13,AX)=0.0
              AI(MS-12,MS-13,AX)=0.0
              AI(MS-11,MS-13,AX)=0.0
              AI(MS-10,MS-13,AX)=0.0
              AI(MS-9,MS-13,AX)=0.0
              AI(MS-8,MS-13,AX)=0.0
              AI(MS-7,MS-13,AX)=PKD1(AX)/(SQRT(2.0)*EV)
              AI(MS-6,MS-13,AX)=0.0
              AI(MS-5,MS-13,AX)=0.0
              AI(MS-4,MS-13,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AI(MS-3,MS-13,AX)=QKD(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AI(MS-2,MS-13,AX)=0.0
              AI(MS-1,MS-13,AX)=QKD(AX)/(SQRT(6.0)*EV)
              AI(MS,MS-13,AX)=0.0

              AI(MS-12,MS-12,AX)=0.0
              AI(MS-11,MS-12,AX)=0.0
              AI(MS-10,MS-12,AX)=0.0
              AI(MS-9,MS-12,AX)=0.0
              AI(MS-8,MS-12,AX)=0.0
              AI(MS-7,MS-12,AX)=0.0
              AI(MS-6,MS-12,AX)=-PKD1(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
              AI(MS-5,MS-12,AX)=-QKD(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AI(MS-4,MS-12,AX)=0.0
              AI(MS-3,MS-12,AX)=0.0
              AI(MS-2,MS-12,AX)=QKD(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AI(MS-1,MS-12,AX)=0.0
              AI(MS,MS-12,AX)=-QKD(AX)/(SQRT(2.0)*EV)

              AI(MS-11,MS-11,AX)=0.0
              AI(MS-10,MS-11,AX)=0.0
              AI(MS-9,MS-11,AX)=0.0
              AI(MS-8,MS-11,AX)=0.0
              AI(MS-7,MS-11,AX)=PKD1(AX)/(SQRT(6.0)*EV)
              AI(MS-6,MS-11,AX)=PKD1(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
              AI(MS-5,MS-11,AX)=-QKD(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AI(MS-4,MS-11,AX)=0.0
              AI(MS-3,MS-11,AX)=0.0
              AI(MS-2,MS-11,AX)=-QKD(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AI(MS-1,MS-11,AX)=-QKD(AX)/(SQRT(2.0)*EV)
              AI(MS,MS-11,AX)=0.0

              AI(MS-10,MS-10,AX)=0.0
              AI(MS-9,MS-10,AX)=0.0
              AI(MS-8,MS-10,AX)=0.0
              AI(MS-7,MS-10,AX)=0.0
              AI(MS-6,MS-10,AX)=PKD1(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
              AI(MS-5,MS-10,AX)=0.0
              AI(MS-4,MS-10,AX)=-QKD(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AI(MS-3,MS-10,AX)=QKD(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AI(MS-2,MS-10,AX)=0.0
              AI(MS-1,MS-10,AX)=0.0
              AI(MS,MS-10,AX)=-QKD(AX)/(SQRT(6.0)*EV)

              AI(MS-9,MS-9,AX)=0.0
              AI(MS-8,MS-9,AX)=0.0
              AI(MS-7,MS-9,AX)=PKD1(MOD(AX,3)+1)/(SQRT(3.0)*EV)
              AI(MS-6,MS-9,AX)=PKD1(AX)/(SQRT(3.0)*EV)
              AI(MS-5,MS-9,AX)=-QKD(AX)/(SQRT(6.0)*EV)
              AI(MS-4,MS-9,AX)=0.0
              AI(MS-3,MS-9,AX)=QKD(AX)/(SQRT(2.0)*EV)
              AI(MS-2,MS-9,AX)=0.0
              AI(MS-1,MS-9,AX)=0.0
              AI(MS,MS-9,AX)=0.0

              AI(MS-8,MS-8,AX)=0.0
              AI(MS-7,MS-8,AX)=-PKD1(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
              AI(MS-6,MS-8,AX)=0.0
              AI(MS-5,MS-8,AX)=0.0
              AI(MS-4,MS-8,AX)=-QKD(AX)/(SQRT(2.0)*EV)
              AI(MS-3,MS-8,AX)=0.0
              AI(MS-2,MS-8,AX)=QKD(AX)/(SQRT(6.0)*EV)
              AI(MS-1,MS-8,AX)=0.0
              AI(MS,MS-8,AX)=0.0
         ENDIF

         IF (MSH.GE.4) THEN
             AI(MS-7,MS-7,AX)=0.0
             AI(MS-6,MS-7,AX)=0.0
             AI(MS-5,MS-7,AX)=-PKD0(MOD(AX+1,3)+1)/(SQRT(2.0)*EV)
             AI(MS-4,MS-7,AX)=PKD0(MOD(AX,3)+1)*SQRT(2.0/3.0)/EV
             AI(MS-3,MS-7,AX)=PKD0(MOD(AX+1,3)+1)/(SQRT(6.0)*EV)
             AI(MS-2,MS-7,AX)=0.0
             AI(MS-1,MS-7,AX)=0.0
             AI(MS,MS-7,AX)=PKD0(AX)/(SQRT(3.0)*EV)

             AI(MS-6,MS-6,AX)=0.0
             AI(MS-5,MS-6,AX)=0.0
             AI(MS-4,MS-6,AX)=-PKD0(AX)/(SQRT(6.0)*EV)
             AI(MS-3,MS-6,AX)=0.0
             AI(MS-2,MS-6,AX)=-PKD0(AX)/(SQRT(2.0)*EV)
             AI(MS-1,MS-6,AX)=-PKD0(MOD(AX+1,3)+1)/(SQRT(3.0)*EV)
             AI(MS,MS-6,AX)=-PKD0(MOD(AX,3)+1)/(SQRT(3.0)*EV)
         ENDIF

         AI(MS-5,MS-5,AX)=0.0  
         AI(MS-4,MS-5,AX)=IHSTRD(AX)/EV
         AI(MS-3,MS-5,AX)=IISTRD(AX)/EV
         AI(MS-2,MS-5,AX)=0.0
         AI(MS-1,MS-5,AX)=-RHSTRD(AX)/(SQRT(2.0)*EV)
         AI(MS,MS-5,AX)=SQRT(2.0)*RISTRD(AX)/EV

         AI(MS-4,MS-4,AX)=0.0
         AI(MS-3,MS-4,AX)=0.0
         AI(MS-2,MS-4,AX)=IISTRD(AX)/EV
         AI(MS-1,MS-4,AX)=-(GKD(AX)-FKD(AX))/(SQRT(2.0)*EV)
         AI(MS,MS-4,AX)=-SQRT(1.50)*RHSTRD(AX)/EV

         AI(MS-3,MS-3,AX)=0.0
         AI(MS-2,MS-3,AX)=-IHSTRD(AX)/EV
         AI(MS-1,MS-3,AX)=SQRT(1.5)*RHSTRD(AX)/EV
         AI(MS,MS-3,AX)=-(GKD(AX)-FKD(AX))/(SQRT(2.0)*EV)

         AI(MS-2,MS-2,AX)=0.0
         AI(MS-1,MS-2,AX)=SQRT(2.0)*RISTRD(AX)/EV
         AI(MS,MS-2,AX)=RHSTRD(AX)/(SQRT(2.0)*EV)

         AI(MS-1,MS-1,AX)=0.0
         AI(MS,MS-1,AX)=0.0

         AI(MS,MS,AX)=0.0
      END DO
 
!     FILL ENTIRE MATRIX
      DO AX=1,3
         DO I=1,MS
            DO J=I+1,MS
               AR(I,J,AX)=AR(J,I,AX)
               AI(I,J,AX)=-AI(J,I,AX)
            END DO
         END DO
      END DO
 
!     GENERATE SECOND DERIVATIVES OF THE HAMILTONIAN
      KKSD(1)=2.0*EFREPR
      KKSD(2)=2.0*EFREPR
      KKSD(3)=2.0*EFREPR
      KKSD(4)=0.0
      KKSD(5)=0.0
      KKSD(6)=0.0

      FKSD(1)=EFREPR*(2*A+B)
      FKSD(2)=EFREPR*(2*A+B)
      FKSD(3)=EFREPR*2*(A-B)
      FKSD(4)=0
      FKSD(5)=0
      FKSD(6)=0

      GKSD(1)=EFREPR*(2*A-B)
      GKSD(2)=EFREPR*(2*A-B)
      GKSD(3)=EFREPR*2*(A+B)
      GKSD(4)=0
      GKSD(5)=0
      GKSD(6)=0

      IHSTSD(1)=0
      IHSTSD(2)=0
      IHSTSD(3)=0
      IHSTSD(4)=0
      IHSTSD(5)=EFREPR*D
      IHSTSD(6)=0

      RHSTSD(1)=0
      RHSTSD(2)=0
      RHSTSD(3)=0
      RHSTSD(4)=0
      RHSTSD(5)=0
      RHSTSD(6)=-EFREPR*D

      IISTSD(1)=0
      IISTSD(2)=0
      IISTSD(3)=0
      IISTSD(4)=EFREPR*D
      IISTSD(5)=0
      IISTSD(6)=0
 
      RISTSD(1)=SQRT(3.0)*B*EFREPR
      RISTSD(2)=-SQRT(3.0)*B*EFREPR
      RISTSD(3)=0
      RISTSD(4)=0
      RISTSD(5)=0
      RISTSD(6)=0

 
!     (THE MATRIX IS SCALED BY AN ELECTRONVOLT)
      DO AX=1,6

!        FIRST THE REAL PART IS CALCULATED:

         IF (MSH.GE.7) THEN
             DO I=8,13
                 BR(MS-I,MS-I,AX)=KKSD(AX)/EV
                 DO J=0,(I-1)
                     BR(MS-J,MS-I,AX)=0.0
                 ENDDO
             ENDDO
         ENDIF

         IF (MSH.GE.4) THEN
             BR(MS-7,MS-7,AX)=KKSD(AX)/EV
             BR(MS-6,MS-7,AX)=0.0
             BR(MS-5,MS-7,AX)=0.0
             BR(MS-4,MS-7,AX)=0.0
             BR(MS-3,MS-7,AX)=0.0
             BR(MS-2,MS-7,AX)=0.0
             BR(MS-1,MS-7,AX)=0.0
             BR(MS,MS-7,AX)=0.0

             BR(MS-6,MS-6,AX)=KKSD(AX)/EV
             BR(MS-5,MS-6,AX)=0.0
             BR(MS-4,MS-6,AX)=0.0
             BR(MS-3,MS-6,AX)=0.0
             BR(MS-2,MS-6,AX)=0.0
             BR(MS-1,MS-6,AX)=0.0
             BR(MS,MS-6,AX)=0.0
         ENDIF

         BR(MS-5,MS-5,AX)=FKSD(AX)/EV
         BR(MS-4,MS-5,AX)=RHSTSD(AX)/EV
         BR(MS-3,MS-5,AX)=RISTSD(AX)/EV
         BR(MS-2,MS-5,AX)=0.0
         BR(MS-1,MS-5,AX)=IHSTSD(AX)/(SQRT(2.0)*EV)
         BR(MS,MS-5,AX)=-SQRT(2.0)*IISTSD(AX)/EV

         BR(MS-4,MS-4,AX)=GKSD(AX)/EV
         BR(MS-3,MS-4,AX)=0.0
         BR(MS-2,MS-4,AX)=RISTSD(AX)/EV
         BR(MS-1,MS-4,AX)=0.0
         BR(MS,MS-4,AX)=SQRT(1.50)*IHSTSD(AX)/EV

         BR(MS-3,MS-3,AX)=GKSD(AX)/EV
         BR(MS-2,MS-3,AX)=-RHSTSD(AX)/EV
         BR(MS-1,MS-3,AX)=SQRT(1.50)*IHSTSD(AX)/EV
         BR(MS,MS-3,AX)=0.0

         BR(MS-2,MS-2,AX)=FKSD(AX)/EV
         BR(MS-1,MS-2,AX)=SQRT(2.0)*IISTSD(AX)/EV
         BR(MS,MS-2,AX)=IHSTSD(AX)/(SQRT(2.0)*EV)

         BR(MS-1,MS-1,AX)=(FKSD(AX)+GKSD(AX))/(2.0*EV)
         BR(MS,MS-1,AX)=0.0

         BR(MS,MS,AX)=BR(MS-1,MS-1,AX)
    
!        THEN THE IMAGINARY PART:

         IF (MSH.GE.7) THEN
             DO I=8,13
                 BR(MS-I,MS-I,AX)=KKSD(AX)/EV
                 DO J=0,(I-1)
                     BR(MS-J,MS-I,AX)=0.0
                 ENDDO
             ENDDO
         ENDIF

         IF (MSH.GE.4) THEN
             BR(MS-7,MS-7,AX)=KKSD(AX)/EV
             BR(MS-6,MS-7,AX)=0.0
             BR(MS-5,MS-7,AX)=0.0
             BR(MS-4,MS-7,AX)=0.0
             BR(MS-3,MS-7,AX)=0.0
             BR(MS-2,MS-7,AX)=0.0
             BR(MS-1,MS-7,AX)=0.0
             BR(MS,MS-7,AX)=0.0

             BR(MS-6,MS-6,AX)=KKSD(AX)/EV
             BR(MS-5,MS-6,AX)=0.0
             BR(MS-4,MS-6,AX)=0.0
             BR(MS-3,MS-6,AX)=0.0
             BR(MS-2,MS-6,AX)=0.0
             BR(MS-1,MS-6,AX)=0.0
             BR(MS,MS-6,AX)=0.0
         ENDIF

         BR(MS-5,MS-5,AX)=FKSD(AX)/EV
         BR(MS-4,MS-5,AX)=RHSTSD(AX)/EV
         BR(MS-3,MS-5,AX)=RISTSD(AX)/EV
         BR(MS-2,MS-5,AX)=0.0
         BR(MS-1,MS-5,AX)=IHSTSD(AX)/(SQRT(2.0)*EV)
         BR(MS,MS-5,AX)=-SQRT(2.0)*IISTSD(AX)/EV

         BR(MS-4,MS-4,AX)=GKSD(AX)/EV
         BR(MS-3,MS-4,AX)=0.0
         BR(MS-2,MS-4,AX)=RISTSD(AX)/EV
         BR(MS-1,MS-4,AX)=0.0
         BR(MS,MS-4,AX)=SQRT(1.50)*IHSTSD(AX)/EV

         BR(MS-3,MS-3,AX)=GKSD(AX)/EV
         BR(MS-2,MS-3,AX)=-RHSTSD(AX)/EV
         BR(MS-1,MS-3,AX)=SQRT(1.50)*IHSTSD(AX)/EV
         BR(MS,MS-3,AX)=0.0

         BR(MS-2,MS-2,AX)=FKSD(AX)/EV
         BR(MS-1,MS-2,AX)=SQRT(2.0)*IISTSD(AX)/EV
         BR(MS,MS-2,AX)=IHSTSD(AX)/(SQRT(2.0)*EV)

         BR(MS-1,MS-1,AX)=(FKSD(AX)+GKSD(AX))/(2.0*EV)
         BR(MS,MS-1,AX)=0.0

         BR(MS,MS,AX)=BR(MS-1,MS-1,AX)
    
!        THEN THE IMAGINARY PART:

         IF (MSH.GE.7) THEN
             DO I=8,13
                 DO J=0,I
                     BI(MS-J,MS-I,AX)=0.0
                 ENDDO
             ENDDO
         ENDIF

         IF (MSH.GE.4) THEN
             BI(MS-7,MS-7,AX)=0.0
             BI(MS-6,MS-7,AX)=0.0
             BI(MS-5,MS-7,AX)=0.0
             BI(MS-4,MS-7,AX)=0.0
             BI(MS-3,MS-7,AX)=0.0
             BI(MS-2,MS-7,AX)=0.0
             BI(MS-1,MS-7,AX)=0.0
             BI(MS,MS-7,AX)=0.0

             BI(MS-6,MS-6,AX)=0.0
             BI(MS-5,MS-6,AX)=0.0
             BI(MS-4,MS-6,AX)=0.0
             BI(MS-3,MS-6,AX)=0.0
             BI(MS-2,MS-6,AX)=0.0
             BI(MS-1,MS-6,AX)=0.0
             BI(MS,MS-6,AX)=0.0
         ENDIF

         BI(MS-5,MS-5,AX)=0.0  
         BI(MS-4,MS-5,AX)=IHSTSD(AX)/EV
         BI(MS-3,MS-5,AX)=IISTSD(AX)/EV
         BI(MS-2,MS-5,AX)=0.0
         BI(MS-1,MS-5,AX)=-RHSTSD(AX)/(SQRT(2.0)*EV)
         BI(MS,MS-5,AX)=SQRT(2.0)*RISTSD(AX)/EV

         BI(MS-4,MS-4,AX)=0.0
         BI(MS-3,MS-4,AX)=0.0
         BI(MS-2,MS-4,AX)=IISTSD(AX)/EV
         BI(MS-1,MS-4,AX)=-(GKSD(AX)-FKSD(AX))/(SQRT(2.0)*EV)
         BI(MS,MS-4,AX)=-SQRT(1.50)*RHSTSD(AX)/EV

         BI(MS-3,MS-3,AX)=0.0
         BI(MS-2,MS-3,AX)=-IHSTSD(AX)/EV
         BI(MS-1,MS-3,AX)=SQRT(1.5)*RHSTSD(AX)/EV
         BI(MS,MS-3,AX)=-(GKSD(AX)-FKSD(AX))/(SQRT(2.0)*EV)

         BI(MS-2,MS-2,AX)=0.0
         BI(MS-1,MS-2,AX)=SQRT(2.0)*RISTSD(AX)/EV
         BI(MS,MS-2,AX)=RHSTSD(AX)/(SQRT(2.0)*EV)

         BI(MS-1,MS-1,AX)=0.0
         BI(MS,MS-1,AX)=0.0

         BI(MS,MS,AX)=0.0
      END DO 

!     FILL ENTIRE MATRIX
      DO AX=1,6
         DO I=1,MS
            DO J=I+1,MS
               BR(I,J,AX)=BR(J,I,AX)
               BI(I,J,AX)=-BI(J,I,AX)
            END DO
         END DO
      END DO

      DO BAND=1,(MS-1),2
         IF (BAND.GT.1) THEN
            DO I=1,(BAND-1)
               O(I)=I
            END DO
         ENDIF

         IF (BAND.LT.(MS-1)) THEN
            DO I=(BAND+2),MS
               O(I-2)=I
            END DO
         ENDIF
         NONDEG=MS-2
         IF (MSH.GE.7) THEN
            IF ((BAND.EQ.13).OR.(BAND.EQ.11)) THEN
               IF(ABS(E(7)-E(6)).LT.ZERLIM) THEN
                  WRITE (*,*) 'DEGENERATE!!!!!!'
                  NONDEG=NONDEG-2
               ENDIF
            ENDIF
         ENDIF
         IF ((BAND.EQ.3).OR.(BAND.EQ.1)) THEN
            IF(ABS(E(2)-E(1)).LT.ZERLIM) THEN
               WRITE (*,*) 'DEGENERATE!!!!!!'
               NONDEG=NONDEG-2
            ENDIF
         ENDIF

         DO AX=1,6
    
            IF (AX.LE.3) THEN
               S1=AX
               S2=AX
            ELSEIF (AX.EQ.4) THEN
               S1=1
               S2=2
            ELSEIF (AX.EQ.5) THEN
               S1=1
               S2=3
            ELSE
               S1=2
               S2=3
            ENDIF
 
            EN=-E(BAND/2+1)
            DO I=1,(MS-2)
               EO(I)=-E((O(I)+1)/2)
            END DO

!           FIRST TERM
            DO I=1,MS
               VECT(I)=(0,0)
               DO J=1,MS
                  VECT(I)=VECT(I)+DCMPLX(VR(J,BAND),VI(J,BAND))*DCMPLX(BR(I,J,AX),BI(I,J,AX)) 
               END DO
            END DO
       
            TEMP=(0,0)
            DO I=1,MS
               TEMP=TEMP+DCMPLX(VR(I,BAND),-VI(I,BAND))*VECT(I)
            END DO
            SD(AX,BAND/2+1)=DBLE(TEMP)

!           SECOND TERM
            DO I=1,MS
               V1(I)=(0,0)
               V2(I)=(0,0)
               DO J=1,MS
                  V1(I)=V1(I)+DCMPLX(VR(J,BAND),VI(J,BAND))*DCMPLX(AR(I,J,S1),AI(I,J,S1)) 
                  V2(I)=V2(I)+DCMPLX(VR(J,BAND),VI(J,BAND))*DCMPLX(AR(I,J,S2),AI(I,J,S2)) 
               END DO
            END DO

            TEMP=(0,0)
            DO J=1,(MS-2)
               TEMPA=(0,0)
               TEMPB=(0,0)
               DO I=1,MS
                  TEMPA=TEMPA+CONJG(DCMPLX(VR(I,O(J)),VI(I,O(J))))*V1(I)
                  TEMPB=TEMPB+CONJG(DCMPLX(VR(I,O(J)),VI(I,O(J))))*V2(I)
               END DO
               TEMPC=TEMPA*CONJG(TEMPB) 
               IF (DBLE(TEMPC).NE.0) TEMP=TEMP+TEMPC/(EN-EO(J))
            END DO
       
!     SUM TERMS 
            SD(AX,BAND/2+1)=-SD(AX,BAND/2+1)-2*DBLE(TEMP)
         END DO ! AX
      END DO ! BAND

      IF ((K.GE.(0.8*PI/A0)).AND.CORR) THEN
         DO AX=1,6
            IF ((AX.EQ.1).OR.(AX.EQ.2).OR.(AX.EQ.3)) THEN
               SD(AX,2)=SD(AX,2)+4.0*LH1*EFREPR*RESCALE*(2.0*KAX(AX)*KAX(AX)+K*K)/EV

               SD(AX,2)=SD(AX,2)+LH2A*EFREPR*(2.0*K*K*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))-2.0*K*K*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+8.0*KAX(AX)*KAX(AX)*(KAX(AX)*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH2B*EFREPR*RESCALE*2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/EV
               SD(AX,2)=SD(AX,2)+LH2C*EFREPR*RESCALE*RESCALE*(2.0*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+2.0*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/EV

               SD(AX,2)=SD(AX,2)+LH3A*EFREPR*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-20.0*K*K*KAX(AX)*KAX(AX)+24.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH3B*EFREPR*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-10.0*K*K*KAX(AX)*KAX(AX)+8.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH3C*EFREPR*RESCALE*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/EV


               SD(AX,3)=SD(AX,3)+4.0*HH1*EFREPR*RESCALE*(2.0*KAX(AX)*KAX(AX)+K*K)/EV

               SD(AX,3)=SD(AX,3)+HH2A*EFREPR*(2.0*K*K*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))-2.0*K*K*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+8.0*KAX(AX)*KAX(AX)*(KAX(AX)*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH2B*EFREPR*RESCALE*2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/EV
               SD(AX,3)=SD(AX,3)+HH2C*EFREPR*RESCALE*RESCALE*(2.0*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+2.0*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/EV

               SD(AX,3)=SD(AX,3)+HH3A*EFREPR*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-20.0*K*K*KAX(AX)*KAX(AX)+24.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH3B*EFREPR*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-10.0*K*K*KAX(AX)*KAX(AX)+8.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH3C*EFREPR*RESCALE*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/EV


               SD(AX,4)=SD(AX,4)+2.0*CB1A*EFREPR/EV
               SD(AX,4)=SD(AX,4)+4.0*CB1B*EFREPR*RESCALE*(2.0*KAX(AX)*KAX(AX)+K*K)/EV
               SD(AX,4)=SD(AX,4)+6.0*CB1C*EFREPR*K*K*RESCALE*RESCALE*(4.0*KAX(AX)*KAX(AX)+K*K)/EV

               SD(AX,4)=SD(AX,4)+CB2A*EFREPR*(2.0*K*K*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))-2.0*K*K*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+8.0*KAX(AX)*KAX(AX)*(KAX(AX)*KAX(AX)*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+CB2B*EFREPR*RESCALE*2.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))/EV
               SD(AX,4)=SD(AX,4)+CB2C*EFREPR*RESCALE*RESCALE*(2.0*K*K*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+2.0*(5.0*(KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)+KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1))+KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)))/EV

               SD(AX,4)=SD(AX,4)+CB3A*EFREPR*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-20.0*K*K*KAX(AX)*KAX(AX)+24.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+CB3B*EFREPR*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K*K*K-10.0*K*K*KAX(AX)*KAX(AX)+8.0*KAX(AX)*KAX(AX)*KAX(AX)*KAX(AX))/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+2.0*CB3C*EFREPR*RESCALE*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)/EV
               SD(AX,4)=SD(AX,4)+CB3D*EFREPR*RESCALE*RESCALE*RESCALE*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX+1,3)+1)*KAX(MOD(AX,3)+1)*KAX(MOD(AX,3)+1)*(2.0*K*K+10.0*KAX(AX)*KAX(AX))/EV
            ELSEIF (AX.EQ.4) THEN
               SD(AX,2)=SD(AX,2)+8.0*LH1*EFREPR*RESCALE*KX*KY/EV

               SD(AX,2)=SD(AX,2)+LH2A*EFREPR*KX*KY*(4.0*K*K*K*K-4.0*K*K*(K*K+KZ*KZ)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH2B*EFREPR*RESCALE*KX*KY/EV
               SD(AX,2)=SD(AX,2)+LH2C*EFREPR*RESCALE*RESCALE*KX*KY*(4.0*K*K+4.0*(K*K+KZ*KZ))/EV

               SD(AX,2)=SD(AX,2)+LH3A*EFREPR*KX*KY*KZ*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KY*KY)+24.0*KX*KX*KY*KY)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH3B*EFREPR*RESCALE*KX*KY*KZ*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KY*KY)+8.0*KX*KX*KY*KY)/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH3C*EFREPR*RESCALE*RESCALE*KX*KY*KZ*KZ/EV


               SD(AX,3)=SD(AX,3)+8.0*HH1*EFREPR*RESCALE*KX*KY/EV

               SD(AX,3)=SD(AX,3)+HH2A*EFREPR*KX*KY*(4.0*K*K*K*K-4.0*K*K*(K*K+KZ*KZ)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH2B*EFREPR*RESCALE*KX*KY/EV
               SD(AX,3)=SD(AX,3)+HH2C*EFREPR*RESCALE*RESCALE*KX*KY*(4.0*K*K+4.0*(K*K+KZ*KZ))/EV

               SD(AX,3)=SD(AX,3)+HH3A*EFREPR*KX*KY*KZ*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KY*KY)+24.0*KX*KX*KY*KY)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH3B*EFREPR*RESCALE*KX*KY*KZ*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KY*KY)+8.0*KX*KX*KY*KY)/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH3C*EFREPR*RESCALE*RESCALE*KX*KY*KZ*KZ/EV


               SD(AX,4)=SD(AX,4)+0.0
               SD(AX,4)=SD(AX,4)+8.0*CB1B*EFREPR*RESCALE*KX*KY/EV
               SD(AX,4)=SD(AX,4)+24.0*CB1C*EFREPR*RESCALE*RESCALE*K*K*KX*KY/EV

               SD(AX,4)=SD(AX,4)+CB2A*EFREPR*KX*KY*(4.0*K*K*K*K-4.0*K*K*(K*K+KZ*KZ)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB2B*EFREPR*RESCALE*KX*KY/EV
               SD(AX,4)=SD(AX,4)+CB2C*EFREPR*RESCALE*RESCALE*KX*KY*(4.0*K*K+4.0*(K*K+KZ*KZ))/EV

               SD(AX,4)=SD(AX,4)+CB3A*EFREPR*KX*KY*KZ*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KY*KY)+24.0*KX*KX*KY*KY)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+CB3B*EFREPR*RESCALE*KX*KY*KZ*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KY*KY)+8.0*KX*KX*KY*KY)/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB3C*EFREPR*RESCALE*RESCALE*KX*KY*KZ*KZ/EV
               SD(AX,4)=SD(AX,4)+4.0*CB3D*EFREPR*RESCALE*RESCALE*RESCALE*KX*KY*KZ*KZ*(K*K+KX*KX+KY*KY)/EV
            ELSEIF (AX.EQ.5) THEN
               SD(AX,2)=SD(AX,2)+8.0*LH1*EFREPR*RESCALE*KX*KZ/EV

               SD(AX,2)=SD(AX,2)+LH2A*EFREPR*KX*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KY*KY)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH2B*EFREPR*RESCALE*KX*KZ
               SD(AX,2)=SD(AX,2)+LH2C*EFREPR*RESCALE*RESCALE*KX*KZ*(4.0*K*K+4.0*(K*K+KY*KY))/EV

               SD(AX,2)=SD(AX,2)+LH3A*EFREPR*KX*KY*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KZ*KZ)+24.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH3B*EFREPR*RESCALE*KX*KY*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KZ*KZ)+8.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH3C*EFREPR*RESCALE*RESCALE*KX*KY*KY*KZ/EV


               SD(AX,3)=SD(AX,3)+8.0*HH1*EFREPR*RESCALE*KX*KZ/EV

               SD(AX,3)=SD(AX,3)+HH2A*EFREPR*KX*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KY*KY)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH2B*EFREPR*RESCALE*KX*KZ
               SD(AX,3)=SD(AX,3)+HH2C*EFREPR*RESCALE*RESCALE*KX*KZ*(4.0*K*K+4.0*(K*K+KY*KY))/EV

               SD(AX,3)=SD(AX,3)+HH3A*EFREPR*KX*KY*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KZ*KZ)+24.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH3B*EFREPR*RESCALE*KX*KY*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KZ*KZ)+8.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH3C*EFREPR*RESCALE*RESCALE*KX*KY*KY*KZ/EV


               SD(AX,4)=SD(AX,4)+0.0
               SD(AX,4)=SD(AX,4)+8.0*CB1B*EFREPR*RESCALE*KX*KZ/EV
               SD(AX,4)=SD(AX,4)+24.0*CB1C*EFREPR*RESCALE*RESCALE*K*K*KX*KZ/EV

               SD(AX,4)=SD(AX,4)+CB2A*EFREPR*KX*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KY*KY)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB2B*EFREPR*RESCALE*KX*KZ
               SD(AX,4)=SD(AX,4)+CB2C*EFREPR*RESCALE*RESCALE*KX*KZ*(4.0*K*K+4.0*(K*K+KY*KY))/EV

               SD(AX,4)=SD(AX,4)+CB3A*EFREPR*KX*KY*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KX*KX+KZ*KZ)+24.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+CB3B*EFREPR*RESCALE*KX*KY*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KX*KX+KZ*KZ)+8.0*KX*KX*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB3C*EFREPR*RESCALE*RESCALE*KX*KY*KY*KZ/EV
               SD(AX,4)=SD(AX,4)+4.0*CB3D*EFREPR*RESCALE*RESCALE*RESCALE*KX*KY*KZ*KZ*(K*K+KX*KX+KZ*KZ)/EV
            ELSE
               SD(AX,2)=SD(AX,2)+8.0*LH1*EFREPR*RESCALE*KY*KZ/EV

               SD(AX,2)=SD(AX,2)+LH2A*EFREPR*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KX*KX)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH2B*EFREPR*RESCALE*KY*KZ
               SD(AX,2)=SD(AX,2)+LH2C*EFREPR*RESCALE*RESCALE*KY*KZ*(4.0*K*K+4.0*(K*K+KX*KX))/EV

               SD(AX,2)=SD(AX,2)+LH3A*EFREPR*KX*KX*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KY*KY+KZ*KZ)+24.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+LH3B*EFREPR*RESCALE*KX*KX*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KY*KY+KZ*KZ)+8.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,2)=SD(AX,2)+4.0*LH3C*EFREPR*RESCALE*RESCALE*KX*KX*KY*KZ/EV


               SD(AX,3)=SD(AX,3)+8.0*HH1*EFREPR*RESCALE*KY*KZ/EV

               SD(AX,3)=SD(AX,3)+HH2A*EFREPR*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KX*KX)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH2B*EFREPR*RESCALE*KY*KZ
               SD(AX,3)=SD(AX,3)+HH2C*EFREPR*RESCALE*RESCALE*KY*KZ*(4.0*K*K+4.0*(K*K+KX*KX))/EV

               SD(AX,3)=SD(AX,3)+HH3A*EFREPR*KX*KX*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KY*KY+KZ*KZ)+24.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+HH3B*EFREPR*RESCALE*KX*KX*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KY*KY+KZ*KZ)+8.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,3)=SD(AX,3)+4.0*HH3C*EFREPR*RESCALE*RESCALE*KX*KX*KY*KZ/EV


               SD(AX,4)=SD(AX,4)+0.0
               SD(AX,4)=SD(AX,4)+8.0*CB1B*EFREPR*RESCALE*KY*KZ/EV
               SD(AX,4)=SD(AX,4)+24.0*CB1C*EFREPR*RESCALE*RESCALE*K*K*KY*KZ/EV

               SD(AX,4)=SD(AX,4)+CB2A*EFREPR*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(K*K+KX*KX)+8.0*(KX*KX*KY*KY+KX*KX*KZ*KZ+KY*KY*KZ*KZ))/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB2B*EFREPR*RESCALE*KY*KZ
               SD(AX,4)=SD(AX,4)+CB2C*EFREPR*RESCALE*RESCALE*KY*KZ*(4.0*K*K+4.0*(K*K+KX*KX))/EV

               SD(AX,4)=SD(AX,4)+CB3A*EFREPR*KX*KX*KY*KZ*(4.0*K*K*K*K-8.0*K*K*(KY*KY+KZ*KZ)+24.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+CB3B*EFREPR*RESCALE*KX*KX*KY*KZ*(4.0*K*K*K*K-4.0*K*K*(KY*KY+KZ*KZ)+8.0*KY*KY*KZ*KZ)/(K*K*K*K*K*K*EV)
               SD(AX,4)=SD(AX,4)+4.0*CB3C*EFREPR*RESCALE*RESCALE*KX*KX*KY*KZ/EV
               SD(AX,4)=SD(AX,4)+4.0*CB3D*EFREPR*RESCALE*RESCALE*RESCALE*KX*KY*KZ*KZ*(K*K+KY*KY+KZ*KZ)/EV
            ENDIF
         END DO
      ENDIF

!      DO AX=1,6
!         DO I=1,MS
!            WRITE(*,*) AR(I,1,AX)," ",AR(I,2,AX)," ",AR(I,3,AX)," ",AR(I,4,AX)," ",AR(I,5,AX)," ",AR(I,6,AX)," ",AR(I,7,AX)," ",AR(I,8,AX)," ",AR(I,9,AX)," ",AR(I,10,AX)," ",AR(I,11,AX)," ",AR(I,12,AX)," ",AR(I,13,AX)," ",AR(I,14,AX)
!         END DO
!         WRITE(*,*)
!      END DO
!      WRITE(*,*)

      END SUBROUTINE SECNDD

 
 
      PROGRAM KPBAND
!     CALCULATIONS OF VALENCE-BAND ENERGIES AND EIGENVECTORS
!     UNDER STRAIN. THE PROGRAM ALSO CALCULATES THE GRADIENT
!     AND SECOND DERIVATIVES OF THE DISPERSION RELATION          

!      USE Halvorinit, ONLY : A0, DBL, EXX, EXY, EYY, EXZ, EYZ, EZZ
      USE Halvorinit

!     "TECHNICAL" PARAMETERS 
      INTEGER            :: X,Y,Z,ANTCUB
      INTEGER, PARAMETER :: N=MS
      REAL(KIND=DBL)     :: KX,KY,KZ,E(MSH),MSIZE,WAVMAX,VR(MS,MS),VI(MS,MS),EMIN(MSH)
      LOGICAL            :: CALCSELECT
      REAL(KIND=DBL)     :: EGRAD(MSH,3),SD(6,MSH)
      INTEGER :: I,J,ERROR,CH

!     ABORT PROGRAM IF MODEL SIZE PARAMETERS ARE INCONSISTENT
      IF ((MS.EQ.6).OR.(MS.EQ.8).OR.(MS.EQ.14)) THEN
          IF ((MS/MSH).EQ.2) THEN
              IF ((MSD/MS).EQ.2) THEN
              ELSE
                  WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
                  STOP
              ENDIF
          ELSE
              WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
              STOP
          ENDIF
      ELSE
          WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
          STOP
      ENDIF
 
!     INITIALIZING PARAMETERS WITH VALUES STORED ON FILE

      OPEN(50,FILE='KPBAND.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='OLD',RECL=496)
      READ (50,REC=1) EXX,EXY,EYY,EXZ,EYZ,EZZ,ENCALC,GRCALC,VECALC,SDCALC, &
                      ENNAME,GRNAME,VENAME,SDNAME,MESHNU,BFRAC,BANDSELECT,FINE,FBFRAC,FMESHN
      CLOSE(50)
      
 
!     REPORT WHAT IS DONE
      WRITE (*,*) ' ' 
      WRITE (*,*) '**************** KPBAND ****************'
      WRITE (*,*) 'Program calculating bandstructure and'
      WRITE (*,*) 'related quantities for MCT.'
      WRITE (*,*) ' '
      WRITE (*,9910) 'Strain: ',EXX,EXY,EXZ,EXY,EYY,EYZ,EXZ,EYZ,EZZ
 
      WRITE (*,*) 'Coarse mesh:'
      WRITE (*,9920) '   Number of meshpoints : ',MESHNU
      WRITE (*,9930) '   Fraction of 2*PI/A0  : ',BFRAC
      WRITE (*,*) ' '
      WRITE (*,*) 'Fine mesh:'
      IF (FINE) THEN
          WRITE (*,*)    '   Fine mesh is generated.'
          WRITE (*,9920) '   Number of meshpoints : ',FMESHN
          WRITE (*,9930) '   Fraction of 2*PI/A0  : ',FBFRAC
      ELSE 
          WRITE (*,*) '   No fine mesh is generated.'
      ENDIF
      WRITE (*,*) ' '

      WRITE(*,*) 'Results are stored for bands : '
      IF (BANDSELECT(1)) WRITE(*,*) 1
      IF (BANDSELECT(2)) WRITE(*,*) 2
      IF (BANDSELECT(3)) WRITE(*,*) 3
      IF (MSH.GE.4) THEN
          IF (BANDSELECT(4)) WRITE(*,*) 4
      ENDIF
      IF (MSH.GE.7) THEN
          IF (BANDSELECT(5)) WRITE(*,*) 5
          IF (BANDSELECT(6)) WRITE(*,*) 6
          IF (BANDSELECT(7)) WRITE(*,*) 7
      ENDIF
      WRITE (*,*) ' '

      WRITE (*,*) 'The program calculates : '
      IF (ENCALC) WRITE (*,*) 'ENERGIES'
      IF (GRCALC) WRITE (*,*) 'GRADIENTS'
      IF (SDCALC) WRITE (*,*) 'SECOND DERIVATIVES'
      IF (VECALC) WRITE (*,*) 'EIGENVECTORS'

      WRITE (*,*) 'Output files are: '
      IF (BANDSELECT(1)) THEN
          WRITE (*,*) 'Output files for band 1 are: '
          IF (ENCALC) WRITE (*,*) ENNAME(1)
          IF (GRCALC) WRITE (*,*) GRNAME(1)
          IF (SDCALC) WRITE (*,*) SDNAME(1)
          IF (VECALC) WRITE (*,*) VENAME(1)
      ENDIF
      IF (BANDSELECT(2)) THEN
          WRITE (*,*) 'Output files for band 2 are: '
          IF (ENCALC) WRITE (*,*) ENNAME(2)
          IF (GRCALC) WRITE (*,*) GRNAME(2)
          IF (SDCALC) WRITE (*,*) SDNAME(2)
          IF (VECALC) WRITE (*,*) VENAME(2)
      ENDIF
      IF (BANDSELECT(3)) THEN
          WRITE (*,*) 'Output files for band 3 are: '
          IF (ENCALC) WRITE (*,*) ENNAME(3)
          IF (GRCALC) WRITE (*,*) GRNAME(3)
          IF (SDCALC) WRITE (*,*) SDNAME(3)
          IF (VECALC) WRITE (*,*) VENAME(3)
      ENDIF
      IF (MSH.GE.4) THEN
          IF (BANDSELECT(4)) THEN
              WRITE (*,*) 'Output files for band 4 are: '
              IF (ENCALC) WRITE (*,*) ENNAME(4)
              IF (GRCALC) WRITE (*,*) GRNAME(4)
              IF (SDCALC) WRITE (*,*) SDNAME(4)
              IF (VECALC) WRITE (*,*) VENAME(4)
          ENDIF
      ENDIF
      IF (MSH.GE.7) THEN
          IF (BANDSELECT(5)) THEN
              WRITE (*,*) 'Output files for band 5 are: '
              IF (ENCALC) WRITE (*,*) ENNAME(5)
              IF (GRCALC) WRITE (*,*) GRNAME(5)
              IF (SDCALC) WRITE (*,*) SDNAME(5)
              IF (VECALC) WRITE (*,*) VENAME(5)
          ENDIF
          IF (BANDSELECT(6)) THEN
              WRITE (*,*) 'Output files for band 6 are: '
              IF (ENCALC) WRITE (*,*) ENNAME(6)
              IF (GRCALC) WRITE (*,*) GRNAME(6)
              IF (SDCALC) WRITE (*,*) SDNAME(6)
              IF (VECALC) WRITE (*,*) VENAME(6)
          ENDIF
          IF (BANDSELECT(7)) THEN
              WRITE (*,*) 'Output files for band 7 are: '
              IF (ENCALC) WRITE (*,*) ENNAME(7)
              IF (GRCALC) WRITE (*,*) GRNAME(7)
              IF (SDCALC) WRITE (*,*) SDNAME(7)
              IF (VECALC) WRITE (*,*) VENAME(7)
          ENDIF
      ENDIF
      WRITE (*,*) ' '

 
!     ABORT PROGRAM IF IT ISN'T SET TO COMPUTE ANYTHING 
      CALCSELECT=ENCALC.OR.GRCALC.OR.VECALC.OR.SDCALC
      IF (.NOT.CALCSELECT) THEN
          WRITE (*,*) 'NOTHING IS CALCULATED ! '
          STOP
      ENDIF
      CALCSELECT=BANDSELECT(1).OR.BANDSELECT(2).OR.BANDSELECT(3)
      IF (.NOT.CALCSELECT) THEN
          IF (MSH.GE.4) THEN
              CALCSELECT=BANDSELECT(4)
              IF(.NOT.CALCSELECT) THEN
                  IF (MSH.GE.7) THEN
                      CALCSELECT=BANDSELECT(5).OR.BANDSELECT(6).OR.BANDSELECT(7)
                      IF (.NOT.CALCSELECT) THEN
                          WRITE (*,*) 'NOTHING IS CALCULATED ! '
                          STOP
                      ENDIF
                  ELSE
                      WRITE (*,*) 'NOTHING IS CALCULATED ! '
                      STOP
                  ENDIF
              ENDIF
          ELSE
              WRITE (*,*) 'NOTHING IS CALCULATED ! '
              STOP
          ENDIF
      ENDIF
 
!     INITIALIZATION OF MAXIMUM K-VECTOR, WAVMAX, DISTANCE BETWEEN 
!     MESHPOINTS, MSIZE, AND RECORD NUMBER, I. 
      WAVMAX=2*BFRAC*PI/A0
      MSIZE=2*WAVMAX/(MESHNU-1)

!----------------------------------------------------------------------
!     CALCULATING EIGENVECTORS,EIGENVALUES AND GRADIENTS.
!     THE FIRST RECORD(S) OF EACH FILE CONTAINS THE VALUE OF 
!     MESHNU AND WAVMAX. THERE IS ONE FILE FOR EACH TYPE OF
!     DATA FOR EACH BAND. THE FILES ARE ONLY OPENED IF THE
!     CORRESPONDING SET OF DATA IS GOING TO BE COMPUTED .
!     ALL FLOATING POINT DATA IS WRITTEN TO FILE IN SINGLE PRECISION
!---------------------------------------------------------------------- 
 
      IF (ENCALC) THEN
          IF (BANDSELECT(1)) THEN 
              OPEN(21,FILE=ENNAME(1),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
              WRITE(21,REC=1) MESHNU
              WRITE(21,REC=2) WAVMAX
              WRITE(21,REC=4) FINE
              WRITE(21,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(2)) THEN
              OPEN(22,FILE=ENNAME(2),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
              WRITE(22,REC=1) MESHNU
              WRITE(22,REC=2) WAVMAX
              WRITE(22,REC=4) FINE
              WRITE(22,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(3)) THEN
              OPEN(23,FILE=ENNAME(3),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
              WRITE(23,REC=1) MESHNU
              WRITE(23,REC=2) WAVMAX
              WRITE(23,REC=4) FINE
              WRITE(23,REC=5) 2*PI/A0
          ENDIF
          IF (MSH.GE.4) THEN
              IF (BANDSELECT(4)) THEN
                  OPEN(24,FILE=ENNAME(4),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
                  WRITE(24,REC=1) MESHNU
                  WRITE(24,REC=2) WAVMAX
                  WRITE(24,REC=4) FINE
                  WRITE(24,REC=5) 2*PI/A0
              ENDIF
          ENDIF
          IF (MSH.GE.7) THEN
              IF (BANDSELECT(5)) THEN
                  OPEN(25,FILE=ENNAME(5),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
                  WRITE(25,REC=1) MESHNU
                  WRITE(25,REC=2) WAVMAX
                  WRITE(25,REC=4) FINE
                  WRITE(25,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(6)) THEN
                  OPEN(26,FILE=ENNAME(6),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
                  WRITE(26,REC=1) MESHNU
                  WRITE(26,REC=2) WAVMAX
                  WRITE(26,REC=4) FINE
                  WRITE(26,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(7)) THEN
                  OPEN(27,FILE=ENNAME(7),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=8,STATUS='UNKNOWN')
                  WRITE(27,REC=1) MESHNU
                  WRITE(27,REC=2) WAVMAX
                  WRITE(27,REC=4) FINE
                  WRITE(27,REC=5) 2*PI/A0
              ENDIF
          ENDIF
      ENDIF
 
      IF (VECALC) THEN
          IF (BANDSELECT(1)) THEN
              OPEN(31,FILE=VENAME(1),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
              WRITE(31,REC=1) MESHNU
              WRITE(31,REC=2) WAVMAX
              WRITE(31,REC=4) FINE
              WRITE(31,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(2)) THEN
              OPEN(32,FILE=VENAME(2),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
              WRITE(32,REC=1) MESHNU
              WRITE(32,REC=2) WAVMAX
              WRITE(32,REC=4) FINE
              WRITE(32,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(3)) THEN
              OPEN(33,FILE=VENAME(3),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
              WRITE(33,REC=1) MESHNU
              WRITE(33,REC=2) WAVMAX
              WRITE(33,REC=4) FINE
              WRITE(33,REC=5) 2*PI/A0
          ENDIF
          IF (MSH.GE.4) THEN
              IF (BANDSELECT(4)) THEN
                  OPEN(34,FILE=VENAME(4),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
                  WRITE(34,REC=1) MESHNU
                  WRITE(34,REC=2) WAVMAX
                  WRITE(34,REC=4) FINE
                  WRITE(34,REC=5) 2*PI/A0
              ENDIF
          ENDIF
          IF (MSH.GE.7) THEN
              IF (BANDSELECT(5)) THEN
                  OPEN(35,FILE=VENAME(5),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
                  WRITE(35,REC=1) MESHNU
                  WRITE(35,REC=2) WAVMAX
                  WRITE(35,REC=4) FINE
                  WRITE(35,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(6)) THEN
                  OPEN(36,FILE=VENAME(6),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
                  WRITE(36,REC=1) MESHNU
                  WRITE(36,REC=2) WAVMAX
                  WRITE(36,REC=4) FINE
                  WRITE(36,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(7)) THEN
                  OPEN(37,FILE=VENAME(7),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=32*MS,STATUS='UNKNOWN')
                  WRITE(37,REC=1) MESHNU
                  WRITE(37,REC=2) WAVMAX
                  WRITE(37,REC=4) FINE
                  WRITE(37,REC=5) 2*PI/A0
              ENDIF
          ENDIF
      ENDIF
 
      IF (GRCALC) THEN
          IF (BANDSELECT(1)) THEN
              OPEN(41,FILE=GRNAME(1),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
              WRITE(41,REC=1) MESHNU
              WRITE(41,REC=2) WAVMAX
              WRITE(41,REC=4) FINE
              WRITE(41,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(2)) THEN
              OPEN(42,FILE=GRNAME(2),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
              WRITE(42,REC=1) MESHNU
              WRITE(42,REC=2) WAVMAX
              WRITE(42,REC=4) FINE
              WRITE(42,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(3)) THEN
              OPEN(43,FILE=GRNAME(3),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
              WRITE(43,REC=1) MESHNU
              WRITE(43,REC=2) WAVMAX
              WRITE(43,REC=4) FINE
              WRITE(43,REC=5) 2*PI/A0
          ENDIF
          IF (MSH.GE.4) THEN
              IF (BANDSELECT(4)) THEN
                  OPEN(44,FILE=GRNAME(4),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
                  WRITE(44,REC=1) MESHNU
                  WRITE(44,REC=2) WAVMAX
                  WRITE(44,REC=4) FINE
                  WRITE(44,REC=5) 2*PI/A0
              ENDIF
          ENDIF
          IF (MSH.GE.7) THEN
              IF (BANDSELECT(5)) THEN
                  OPEN(45,FILE=GRNAME(5),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
                  WRITE(45,REC=1) MESHNU
                  WRITE(45,REC=2) WAVMAX
                  WRITE(45,REC=4) FINE
                  WRITE(45,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(6)) THEN
                  OPEN(46,FILE=GRNAME(6),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
                  WRITE(46,REC=1) MESHNU
                  WRITE(46,REC=2) WAVMAX
                  WRITE(46,REC=4) FINE
                  WRITE(46,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(7)) THEN
                  OPEN(47,FILE=GRNAME(7),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=24,STATUS='UNKNOWN')
                  WRITE(47,REC=1) MESHNU
                  WRITE(47,REC=2) WAVMAX
                  WRITE(47,REC=4) FINE
                  WRITE(47,REC=5) 2*PI/A0
              ENDIF
          ENDIF
      ENDIF
 
      IF (SDCALC) THEN
          IF (BANDSELECT(1)) THEN
              OPEN(51,FILE=SDNAME(1),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
              WRITE(51,REC=1) MESHNU
              WRITE(51,REC=2) WAVMAX
              WRITE(51,REC=4) FINE
              WRITE(51,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(2)) THEN
              OPEN(52,FILE=SDNAME(2),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
              WRITE(52,REC=1) MESHNU
              WRITE(52,REC=2) WAVMAX
              WRITE(52,REC=4) FINE
              WRITE(52,REC=5) 2*PI/A0
          ENDIF
          IF (BANDSELECT(3)) THEN
              OPEN(53,FILE=SDNAME(3),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
              WRITE(53,REC=1) MESHNU
              WRITE(53,REC=2) WAVMAX
              WRITE(53,REC=4) FINE
              WRITE(53,REC=5) 2*PI/A0
          ENDIF
          IF (MSH.GE.4) THEN
              IF (BANDSELECT(4)) THEN
                  OPEN(54,FILE=SDNAME(4),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
                  WRITE(54,REC=1) MESHNU
                  WRITE(54,REC=2) WAVMAX
                  WRITE(54,REC=4) FINE
                  WRITE(54,REC=5) 2*PI/A0
              ENDIF
          ENDIF
          IF (MSH.GE.7) THEN
              IF (BANDSELECT(5)) THEN
                  OPEN(55,FILE=SDNAME(5),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
                  WRITE(55,REC=1) MESHNU
                  WRITE(55,REC=2) WAVMAX
                  WRITE(55,REC=4) FINE
                  WRITE(55,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(6)) THEN
                  OPEN(56,FILE=SDNAME(6),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
                  WRITE(56,REC=1) MESHNU
                  WRITE(56,REC=2) WAVMAX
                  WRITE(56,REC=4) FINE
                  WRITE(56,REC=5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(7)) THEN
                  OPEN(57,FILE=SDNAME(7),ACCESS='DIRECT',FORM='UNFORMATTED',RECL=48,STATUS='UNKNOWN')
                  WRITE(57,REC=1) MESHNU
                  WRITE(57,REC=2) WAVMAX
                  WRITE(57,REC=4) FINE
                  WRITE(57,REC=5) 2*PI/A0
              ENDIF
          ENDIF
      ENDIF
 
      
!-----CALCULATE AND STORE VALUES AT K=0.------------------------------- 
      WRITE (*,*) 'COARSE MESH:'
 
      I=0
      KX=0.0D0
      KY=0.0D0
      KZ=0.0D0
 
!-----A GOTO IS PERFORMED TO THIS STATEMENT FOR CALCULATION OF THE FINE MESH 
200   CALL EIGEN(KX,KY,KZ,E,VR,VI)
 
 
!     STORING ENERGIES AT K=0
      IF (ENCALC) THEN
          IF (BANDSELECT(1)) WRITE(21,REC=I+3) E(1)
          IF (BANDSELECT(2)) WRITE(22,REC=I+3) E(2)
          IF (BANDSELECT(3)) WRITE(23,REC=I+3) E(3)
          IF (MSH.GE.4) THEN
              IF (BANDSELECT(4)) WRITE(24,REC=I+3) E(4)
          ENDIF
          IF (MSH.GE.7) THEN
              IF (BANDSELECT(5)) WRITE(25,REC=I+3) E(5)
              IF (BANDSELECT(6)) WRITE(26,REC=I+3) E(6)
              IF (BANDSELECT(7)) WRITE(27,REC=I+3) E(7)
          ENDIF
      ENDIF
 


!-----FIND ENERGY LIMIT------------------------------------------------- 
      IF (ENCALC) THEN
 
         WRITE(*,*) 'COMPUTING ENERGY LIMIT...'
 
         EMIN(1)=1D30
         EMIN(2)=1D30
         EMIN(3)=1D30
         IF (MSH.GE.4) EMIN(4)=1D30
         IF (MSH.GE.7) THEN
            EMIN(5)=1D30
            EMIN(6)=1D30
            EMIN(7)=1D30
         ENDIF
 
         DO X=-1,MESHNU,MESHNU+1
            DO Y=-1,MESHNU
               DO Z=-1,MESHNU
                  KX= X*MSIZE-WAVMAX
                  KY= Y*MSIZE-WAVMAX
                  KZ= Z*MSIZE-WAVMAX
                  CALL EIGEN(KX,KY,KZ,E,VR,VI)
                  CALL GRAD(KX,KY,KZ,VR,VI,EGRAD)
                  DO J=1,MSH
                      EMIN(J)=MIN(EMIN(J),E(J)-(MSIZE/2)*(ABS(EGRAD(J,1))+ABS(EGRAD(J,2))+ABS(EGRAD(J,3))))
                  END DO
                  CALL EIGEN(KY,KX,KZ,E,VR,VI)
                  CALL GRAD(KY,KX,KZ,VR,VI,EGRAD)
                  DO J=1,MSH
                      EMIN(J)=MIN(EMIN(J),E(J)-(MSIZE/2)*(ABS(EGRAD(J,1))+ABS(EGRAD(J,2))+ABS(EGRAD(J,3))))
                  END DO
                  CALL EIGEN(KZ,KY,KX,E,VR,VI)
                  CALL GRAD(KZ,KY,KX,VR,VI,EGRAD)
                  DO J=1,MSH
                      EMIN(J)=MIN(EMIN(J),E(J)-(MSIZE/2)*(ABS(EGRAD(J,1))+ABS(EGRAD(J,2))+ABS(EGRAD(J,3))))
                  END DO
               END DO
            END DO
         END DO
 
         IF (BANDSELECT(1)) WRITE(21,REC=I+6) EMIN(1)
         IF (BANDSELECT(2)) WRITE(22,REC=I+6) EMIN(2)
         IF (BANDSELECT(3)) WRITE(23,REC=I+6) EMIN(3)
         IF (MSH.GE.4) THEN
             IF (BANDSELECT(4)) WRITE(24,REC=I+6) EMIN(4)
         ENDIF
         IF (MSH.GE.7) THEN
             IF (BANDSELECT(5)) WRITE(25,REC=I+6) EMIN(5)
             IF (BANDSELECT(6)) WRITE(26,REC=I+6) EMIN(6)
             IF (BANDSELECT(7)) WRITE(27,REC=I+6) EMIN(7)
         ENDIF
 
         WRITE (*,*) 'COMPLETED.'
 
      ENDIF

!-----CALCULATE VALUES IN THE MESH--------------------------------------  
 
      WRITE (*,*) 'COMPUTING MESH...'
 
      I=I+10
 
      DO Z= 0,MESHNU-1
         DO Y= 0,MESHNU-1
            DO X= 0,MESHNU-1

               I=I+1
               KX= X*MSIZE-WAVMAX
               KY= Y*MSIZE-WAVMAX
               KZ= Z*MSIZE-WAVMAX
          
               CALL EIGEN(KX,KY,KZ,E,VR,VI)
          

               IF (VECALC) THEN
                   IF (BANDSELECT(1)) THEN
                       WRITE(31,REC=I) (DCMPLX(VR(J,1),VI(J,1)),J=1,N),(DCMPLX(VR(J,2),VI(J,2)),J=1,N)
                   ENDIF
                   IF (BANDSELECT(2)) THEN
                       WRITE(32,REC=I) (DCMPLX(VR(J,3),VI(J,3)),J=1,N),(DCMPLX(VR(J,4),VI(J,4)),J=1,N)
                   ENDIF
                   IF (BANDSELECT(3)) THEN
                       WRITE(33,REC=I) (DCMPLX(VR(J,5),VI(J,5)),J=1,N),(DCMPLX(VR(J,6),VI(J,6)),J=1,N)
                   ENDIF
                   IF (MSH.GE.4) THEN
                       IF (BANDSELECT(4)) THEN
                           WRITE(34,REC=I) (DCMPLX(VR(J,7),VI(J,7)),J=1,N),(DCMPLX(VR(J,8),VI(J,8)),J=1,N)
                       ENDIF
                   ENDIF
                   IF (MSH.GE.7) THEN
                       IF (BANDSELECT(5)) THEN
                           WRITE(35,REC=I) (DCMPLX(VR(J,9),VI(J,9)),J=1,N),(DCMPLX(VR(J,10),VI(J,10)),J=1,N)
                       ENDIF
                       IF (BANDSELECT(6)) THEN
                           WRITE(36,REC=I) (DCMPLX(VR(J,11),VI(J,11)),J=1,N),(DCMPLX(VR(J,12),VI(J,12)),J=1,N)
                       ENDIF
                       IF (BANDSELECT(7)) THEN
                           WRITE(37,REC=I) (DCMPLX(VR(J,13),VI(J,13)),J=1,N),(DCMPLX(VR(J,14),VI(J,14)),J=1,N)
                       ENDIF
                   ENDIF
               ENDIF
               IF (ENCALC) THEN
                   IF (BANDSELECT(1)) WRITE(21,REC=I) E(1)
                   IF (BANDSELECT(2)) WRITE(22,REC=I) E(2)
                   IF (BANDSELECT(3)) WRITE(23,REC=I) E(3)
                   IF (MSH.GE.4) THEN
                       IF (BANDSELECT(4)) WRITE(24,REC=I) E(4)
                   ENDIF
                   IF (MSH.GE.7) THEN
                       IF (BANDSELECT(5)) WRITE(25,REC=I) E(5)
                       IF (BANDSELECT(6)) WRITE(26,REC=I) E(6)
                       IF (BANDSELECT(7)) WRITE(27,REC=I) E(7)
                   ENDIF
               ENDIF
 
!              COMPUTING AND STORING GRADIENTS.
                   
               IF (GRCALC) THEN
                   CALL GRAD(KX,KY,KZ,VR,VI,EGRAD)
                   IF (BANDSELECT(1)) WRITE(41,REC=I) (EGRAD(1,J),J=1,3)
                   IF (BANDSELECT(2)) WRITE(42,REC=I) (EGRAD(2,J),J=1,3)
                   IF (BANDSELECT(3)) WRITE(43,REC=I) (EGRAD(3,J),J=1,3)
                   IF (MSH.GE.4) THEN
                       IF (BANDSELECT(4)) WRITE(44,REC=I) (EGRAD(4,J),J=1,3)
                   ENDIF
                   IF (MSH.GE.7) THEN
                       IF (BANDSELECT(5)) WRITE(45,REC=I) (EGRAD(5,J),J=1,3)
                       IF (BANDSELECT(6)) WRITE(46,REC=I) (EGRAD(6,J),J=1,3)
                       IF (BANDSELECT(7)) WRITE(47,REC=I) (EGRAD(7,J),J=1,3)
                   ENDIF
               ENDIF
 
!              COMPUTING AND STORING SECOND DERIVATIVES 
               IF (SDCALC) THEN
                   CALL SECNDD(KX,KY,KZ,VR,VI,E,SD)
                   IF (BANDSELECT(1)) WRITE(51,REC=I) (SD(J,1),J=1,6)
                   IF (BANDSELECT(2)) WRITE(52,REC=I) (SD(J,2),J=1,6)
                   IF (BANDSELECT(3)) WRITE(53,REC=I) (SD(J,3),J=1,6)
                   IF (MSH.GE.4) THEN
                       IF (BANDSELECT(4)) WRITE(54,REC=I) (SD(J,4),J=1,6)
                   ENDIF
                   IF (MSH.GE.7) THEN
                       IF (BANDSELECT(5)) WRITE(55,REC=I) (SD(J,5),J=1,6)
                       IF (BANDSELECT(6)) WRITE(56,REC=I) (SD(J,6),J=1,6)
                       IF (BANDSELECT(7)) WRITE(57,REC=I) (SD(J,7),J=1,6)
                   ENDIF
               ENDIF
            END DO
         END DO
      END DO
 
!.. END DO ........................................
           
      WRITE (*,*) 'COMPLETED.'
 
!-----WHEN A FINE MESH IS TO BE CALCULATED THE VARIABLES ARE CHANGED AND
!-----A JUMP BACK IS PERFORMED 
      IF (FINE) THEN
          WAVMAX=FBFRAC*2*PI/A0
          MSIZE=2*WAVMAX/(FMESHN-1)
          MESHNU=FMESHN
          FINE=.FALSE.

          IF (ENCALC) THEN
              IF (BANDSELECT(1)) THEN 
                  WRITE(21,REC=I+1) MESHNU
                  WRITE(21,REC=I+2) WAVMAX
                  WRITE(21,REC=I+4) FINE
                  WRITE(21,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(2)) THEN
                  WRITE(22,REC=I+1) MESHNU
                  WRITE(22,REC=I+2) WAVMAX
                  WRITE(22,REC=I+4) FINE
                  WRITE(22,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(3)) THEN
                  WRITE(23,REC=I+1) MESHNU
                  WRITE(23,REC=I+2) WAVMAX
                  WRITE(23,REC=I+4) FINE
                  WRITE(23,REC=I+5) 2*PI/A0
              ENDIF
              IF (MSH.GE.4) THEN
                  IF (BANDSELECT(4)) THEN
                      WRITE(24,REC=I+1) MESHNU
                      WRITE(24,REC=I+2) WAVMAX
                      WRITE(24,REC=I+4) FINE
                      WRITE(24,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
              IF (MSH.GE.7) THEN
                  IF (BANDSELECT(5)) THEN
                      WRITE(25,REC=I+1) MESHNU
                      WRITE(25,REC=I+2) WAVMAX
                      WRITE(25,REC=I+4) FINE
                      WRITE(25,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(6)) THEN
                      WRITE(26,REC=I+1) MESHNU
                      WRITE(26,REC=I+2) WAVMAX
                      WRITE(26,REC=I+4) FINE
                      WRITE(26,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(7)) THEN
                      WRITE(27,REC=I+1) MESHNU
                      WRITE(27,REC=I+2) WAVMAX
                      WRITE(27,REC=I+4) FINE
                      WRITE(27,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
          ENDIF
 
          IF (VECALC) THEN
              IF (BANDSELECT(1)) THEN
                  WRITE(31,REC=I+1) MESHNU
                  WRITE(31,REC=I+2) WAVMAX
                  WRITE(31,REC=I+4) FINE
                  WRITE(31,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(2)) THEN
                  WRITE(32,REC=I+1) MESHNU
                  WRITE(32,REC=I+2) WAVMAX
                  WRITE(32,REC=I+4) FINE
                  WRITE(32,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(3)) THEN
                  WRITE(33,REC=I+1) MESHNU
                  WRITE(33,REC=I+2) WAVMAX
                  WRITE(33,REC=I+4) FINE
                  WRITE(33,REC=I+5) 2*PI/A0
              ENDIF
              IF (MSH.GE.4) THEN
                  IF (BANDSELECT(4)) THEN
                      WRITE(34,REC=I+1) MESHNU
                      WRITE(34,REC=I+2) WAVMAX
                      WRITE(34,REC=I+4) FINE
                      WRITE(34,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
              IF (MSH.GE.7) THEN
                  IF (BANDSELECT(5)) THEN
                      WRITE(35,REC=I+1) MESHNU
                      WRITE(35,REC=I+2) WAVMAX
                      WRITE(35,REC=I+4) FINE
                      WRITE(35,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(6)) THEN
                      WRITE(36,REC=I+1) MESHNU
                      WRITE(36,REC=I+2) WAVMAX
                      WRITE(36,REC=I+4) FINE
                      WRITE(36,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(7)) THEN
                      WRITE(37,REC=I+1) MESHNU
                      WRITE(37,REC=I+2) WAVMAX
                      WRITE(37,REC=I+4) FINE
                      WRITE(37,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
          ENDIF
 
          IF (GRCALC) THEN
              IF (BANDSELECT(1)) THEN
                  WRITE(41,REC=I+1) MESHNU
                  WRITE(41,REC=I+2) WAVMAX
                  WRITE(41,REC=I+4) FINE
                  WRITE(41,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(2)) THEN
                  WRITE(42,REC=I+1) MESHNU
                  WRITE(42,REC=I+2) WAVMAX
                  WRITE(42,REC=I+4) FINE
                  WRITE(42,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(3)) THEN
                  WRITE(43,REC=I+1) MESHNU
                  WRITE(43,REC=I+2) WAVMAX
                  WRITE(43,REC=I+4) FINE
                  WRITE(43,REC=I+5) 2*PI/A0
              ENDIF
              IF (MSH.GE.4) THEN
                  IF (BANDSELECT(4)) THEN
                      WRITE(44,REC=I+1) MESHNU
                      WRITE(44,REC=I+2) WAVMAX
                      WRITE(44,REC=I+4) FINE
                      WRITE(44,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
              IF (MSH.GE.7) THEN
                  IF (BANDSELECT(5)) THEN
                      WRITE(45,REC=I+1) MESHNU
                      WRITE(45,REC=I+2) WAVMAX
                      WRITE(45,REC=I+4) FINE
                      WRITE(45,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(6)) THEN
                      WRITE(46,REC=I+1) MESHNU
                      WRITE(46,REC=I+2) WAVMAX
                      WRITE(46,REC=I+4) FINE
                      WRITE(46,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(7)) THEN
                      WRITE(47,REC=I+1) MESHNU
                      WRITE(47,REC=I+2) WAVMAX
                      WRITE(47,REC=I+4) FINE
                      WRITE(47,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
          ENDIF
 
          IF (SDCALC) THEN
              IF (BANDSELECT(1)) THEN
                  WRITE(51,REC=I+1) MESHNU
                  WRITE(51,REC=I+2) WAVMAX
                  WRITE(51,REC=I+4) FINE
                  WRITE(51,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(2)) THEN
                  WRITE(52,REC=I+1) MESHNU
                  WRITE(52,REC=I+2) WAVMAX
                  WRITE(52,REC=I+4) FINE
                  WRITE(52,REC=I+5) 2*PI/A0
              ENDIF
              IF (BANDSELECT(3)) THEN
                  WRITE(53,REC=I+1) MESHNU
                  WRITE(53,REC=I+2) WAVMAX
                  WRITE(53,REC=I+4) FINE
                  WRITE(53,REC=I+5) 2*PI/A0
              ENDIF
              IF (MSH.GE.4) THEN
                  IF (BANDSELECT(4)) THEN
                      WRITE(54,REC=I+1) MESHNU
                      WRITE(54,REC=I+2) WAVMAX
                      WRITE(54,REC=I+4) FINE
                      WRITE(54,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
              IF (MSH.GE.7) THEN
                  IF (BANDSELECT(5)) THEN
                      WRITE(55,REC=I+1) MESHNU
                      WRITE(55,REC=I+2) WAVMAX
                      WRITE(55,REC=I+4) FINE
                      WRITE(55,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(6)) THEN
                      WRITE(56,REC=I+1) MESHNU
                      WRITE(56,REC=I+2) WAVMAX
                      WRITE(56,REC=I+4) FINE
                      WRITE(56,REC=I+5) 2*PI/A0
                  ENDIF
                  IF (BANDSELECT(7)) THEN
                      WRITE(57,REC=I+1) MESHNU
                      WRITE(57,REC=I+2) WAVMAX
                      WRITE(57,REC=I+4) FINE
                      WRITE(57,REC=I+5) 2*PI/A0
                  ENDIF
              ENDIF
          ENDIF
 
          WRITE (*,*) 'FINE MESH:'
          GOTO 200
      ENDIF

      CLOSE(21)
      CLOSE(22)
      CLOSE(23)
      IF (MSH.GE.4) CLOSE(24)
      IF (MSH.GE.7) THEN
          CLOSE(25)
          CLOSE(26)
          CLOSE(27)
      ENDIF
      CLOSE(31)
      CLOSE(32)
      CLOSE(33)
      IF (MSH.GE.4) CLOSE(34)
      IF (MSH.GE.7) THEN
          CLOSE(35)
          CLOSE(36)
          CLOSE(37)
      ENDIF
      CLOSE(41)
      CLOSE(42)
      CLOSE(43)
      IF (MSH.GE.4) CLOSE(44)
      IF (MSH.GE.7) THEN
          CLOSE(45)
          CLOSE(46)
          CLOSE(47)
      ENDIF
      CLOSE(51)
      CLOSE(52)
      CLOSE(53)
      IF (MSH.GE.4) CLOSE(54)
      IF (MSH.GE.7) THEN
          CLOSE(55)
          CLOSE(56)
          CLOSE(57)
      ENDIF
 
9910  FORMAT (' ',A,3(T14,3(F10.7,' '),/))
9920  FORMAT (' ',A,I3)
9930  FORMAT (' ',A,F5.3)

      END

!                        SUGGESTIONS FOR IMPROVEMENT                      C
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!
! *  DURING PROGRAM DEVELOPMENT AND THE EARLY USE OF THE PROGRAM CALCULATIONS
!    WHERE OFTEN DONE ON TWO OR ALL BANDS AT A TIME. THEN IT WAS MOST 
!    EFFICIENT TO DO CALCULATIONS OF GRADIENTS AND SECOND DERIVATIVES 
!    FOR ALL BANDS TOO. LATER PRACTICE HAS SHOWN THAT THE BANDS USUALLY
!    ARE DESIRABLE TO COMPUTE SEPARATELY IN ORDER TO ADOPT THE OPTIMAL 
!    MESH FOR EACH BAND. THUS THE FIRST AND SECOND DERIVATIVES SUBROUTINES
!    SHOULD BE MODIFIED TO CALCULATE VALUES FOR ONE BAND ONLY.
! 
! *  THESE CONSIDERATIONS ALSO APPLY TO THE DETERMINATION OF ENERGY LIMITS
! 
! *  THE SECOND DERIVATIVES ROUTINE HAVE A LOT OF MATRIX OPERATIONS, BUT 
!    THE MATRICES ARE RATHER SPARSE. ONE SHOULD TRY TO TAKE ADVANTAGE OF
!    THIS
!
! *  FOR CALCULATIONS ON STRAINED SEMICONDUCTOR THE ALGORITHM WHICH 
!    DISTINGUISHES THE DIFFERENT EIGENVECTORS SHOULD BE IMPLEMENTED.
!    IT SHOULD ALSO TAKE CARE OF THE CASES WHERE THE LIGHT AND HEAVY 
!    HOLES ARE DEGENRATE WITH EACH OTHER. THE PROGRAM WILL WORK FOR 
!    MANY CASES OF STRAIN AS IT IS, BUT THE MODIFICATION WILL MAKE 
!    IT MORE ROBUST.
! 
! *  IF THE PROGRAM IS TO BE RUN ON A MACHINE WITH POOR I/O BUFFERING,
!    INTERNAL ARRAYS SHOULD BE USED AS BUFFERS.
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
