!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
!     THE GENERAL BAND LABELS USED BY THIS PROGRAM CORRESPOND TO REAL BANDS
!     AS FOLLOWS:
!     
!     BAND 1: SPLIT-OFF
!     BAND 2: LIGHT HOLE
!     BAND 3: HEAVY HOLE
!     BAND 4: 1ST CONDUCTION BANN
!     BAND 5: UPPER SPLIT-OFF LIKE CONDUCTION BAND
!     BAND 6: UPPER LIGHT HOLE LIKE CONDUCTION BAND
!     BAND 7: UPPER HEAVY HOLE LIKE CONDUCTION BAND
!
!     SEE APPENDIX D IN KARLSENS'S PROJECT THESIS FOR MORE INFORMATION.
!
!     THE FILE I/O IN THIS PROGRAM IS HARDCODED TO 64 BIT INTEGERS (8 BYTES).
!     OPEN STATEMENTS SPECIFY A RECORD LENGTH GIVEN BY THE TOTAL SIZE
!     (IN BYTES) OF ALL PARAMETERS WRITTEN TO FILE. IN THIS PROGRAM ALL OF
!     THESE ARE CALCULATED USING 1 INTEGER = 8 BYTES. THIS GIVES KPBAND.INI
!     A RECORD LENGTH OF 496. THEREFORE THIS PROGRAM MUST BE COMPILED WITH
!     THE COMPILER FLAG "-i8", TO ENSURE 64 BIT INTEGERS. IF THIS IS NOT DONE,
!     KPBAND2mkl.f WILL BE UNABLE TO READ THE .INI-FILE PRODUCED BY THIS PROGRAM.
!     
!     IN ORDER TO COMPILE THIS PROGRAM WITH IFORT, WRITE THE FOLLOWING:
!
!     ifort halvorinit.f90 KPINI.f90 -i8 -o KPINI.out
!
!     - Bjoernar Karlsen, December 2012
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 
! 
!                            KPINI
! 
!     This is a program for creating or changing the initialization 
!     file KPBAND.INI, which holds the user definable parameters of
!     for KPBAND. Band parameters and model sizes
!     must be changed in the source code of the module halvorinit.f90
! 
!CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
 

!     THIS IS A FUNCTION USED TO FIND THE END OF TEXT STRING 
      FUNCTION LAST(STR)
      INTEGER :: LAST
      CHARACTER(LEN=*) :: STR
 
      LAST=LEN(STR)
10    IF (STR(LAST:LAST).EQ.' '.AND.LAST.GT.1) THEN
          LAST=LAST-1
          GOTO 10
      ENDIF

      END
 
!     THIS SUBROUTINE CREATES THE INITIALIZATION FILE 
!     WITH SOME DEFAULT VALUES
 
      SUBROUTINE INIT
      USE Halvorinit

      INTEGER :: I

      OPEN(60,FILE='KPBAND.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='NEW',RECL=496)
 
      EXX=0.0
      EXY=0.0
      EYY=0.0
      EXZ=0.0
      EYZ=0.0
      EZZ=0.0
 
      ENCALC=.FALSE.
      GRCALC=.FALSE.
      VECALC=.FALSE.
      SDCALC=.FALSE.

      MESHNU=2
      BFRAC=1.0
 
      DO I=1,MSH
          BANDSELECT(I)=.FALSE.
          ENNAME(I)=' '
          GRNAME(I)=' '
          VENAME(I)=' '
          SDNAME(I)=' '
      END DO
 
      FINE=.FALSE.
      FBFRAC=1.0
      FMESHN=2

      WRITE (60,REC=1) EXX,EXY,EYY,EXZ,EYZ,EZZ,ENCALC,GRCALC,VECALC, &
                       SDCALC,ENNAME,GRNAME,VENAME,SDNAME,MESHNU,BFRAC, &
                       BANDSELECT,FINE,FBFRAC,FMESHN
 
      CLOSE(60)
 
      WRITE (*,*) ' '
      WRITE (*,*) 'File KPBAND.INI is created.'
      WRITE (*,*)

      END
 

 
      PROGRAM KPINI
      USE Halvorinit


!     "TECHNICAL" PARAMETERS 
      INTEGER :: X,Y,Z
      INTEGER, PARAMETER :: N = MS
      REAL(KIND=DBL) :: MSIZE,WAVMAX
      INTEGER :: J,ERROR,CH
 
!     MENU RELATED VARIABLES
      INTEGER :: L,LAST,I 
      INTEGER, PARAMETER :: ANTMEN=7
      CHARACTER(LEN=12) :: NAM
      CHARACTER :: SELECTOR
      CHARACTER(LEN=70) :: MENU(ANTMEN)
      DATA MENU/'Set strain tensor.', &
                'Describe mesh(es).', &
                'Set which bands to calculate.', &
                'Set what to calculate.', &
                'Define filenames.', &
                'Show all data.', &
                'Save and quit'/
     

!     ABORT PROGRAM IF MODEL SIZE MARAMETERS ARE INCONSITSTENT
      IF ((MS.EQ.6).OR.(MS.EQ.8).OR.(MS.EQ.14)) THEN
          IF ((MS/MSH).EQ.2) THEN
              IF ((MSD/MS).EQ.2) THEN
              ELSE
                  WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
                  STOP
              ENDIF
          ELSE
              WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
              STOP
          ENDIF
      ELSE
          WRITE(*,*) 'INCONSISTENT MODEL SIZE PARAMETERS'
          STOP
      ENDIF

!     INITIALIZING PARAMETERS WITH VALUES STORED ON FILE OR DEFAULT
!     VALUES
 
      OPEN(50,FILE='KPBAND.INI',ACCESS='DIRECT',FORM='UNFORMATTED',IOSTAT=ERROR,STATUS='OLD',RECL=496)
      CLOSE(50)
      IF (ERROR.NE.0) THEN
          CALL INIT
      ENDIF
 
      OPEN(50,FILE='KPBAND.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='OLD',RECL=496)
      READ (50,REC=1) EXX,EXY,EYY,EXZ,EYZ,EZZ,ENCALC,GRCALC,VECALC, &
                      SDCALC,ENNAME,GRNAME,VENAME,SDNAME,MESHNU,BFRAC, &
                      BANDSELECT,FINE,FBFRAC,FMESHN
      CLOSE(50)
 
100   WRITE (*,*) ' '
      WRITE (*,*) '**** KPINI MAIN MENU *****'
      WRITE (*,*) ' '
      WRITE (*,9900) (I,MENU(I),I=1,ANTMEN)
      WRITE (*,'(/,'' '',A)') 'ENTER SELECTION : '
110   READ (*,*) CH
      IF (CH.LT.1.OR.CH.GT.ANTMEN) THEN
          WRITE (*,*) 'ILLEGAL SELECTION! TRY ONCE MORE. '
          GOTO 110
      ENDIF
 
      WRITE (*,*) ' '
      GOTO (211,212,213,214,215,216,217) CH
 
211   WRITE (*,*) '**** SET STRAIN TENSOR ****'
      WRITE (*,*) ' '
      WRITE (*,9910) 'Strain: ',EXX,EXY,EXZ,EXY,EYY,EYZ,EXZ,EYZ,EZZ
      WRITE (*,*)'SPECIFY NEW STRAIN (EXX,EXY,EXZ,EYY,EYZ,EZZ): '
      READ  (*,*) EXX,EXY,EXZ,EYY,EYZ,EZZ
      GOTO 100
 
212   WRITE (*,*) '**** DESCRIBE MESH(ES) ****'
      WRITE (*,*) 'Coarse mesh:'
      WRITE (*,9920) '   1. Number of meshpoints : ',MESHNU
      WRITE (*,9930) '   2. Fraction of 2*PI/A0  : ',BFRAC
      WRITE (*,*) ' '
      WRITE (*,*) 'Fine mesh:'
      IF (FINE) THEN
          WRITE (*,*)    '   3. Fine mesh is generated.'
          WRITE (*,9920) '   4. Number of meshpoints : ',FMESHN
          WRITE (*,9930) '   5. Fraction of 2*PI/A0  : ',FBFRAC
          I=5
      ELSE 
          WRITE (*,*) '   3. No fine mesh is generated.'
          I=3
      ENDIF
      WRITE (*,'(/,'' '',A)') 'ENTER SELECTION (0=EXIT) : '
300   READ (*,*) CH
      WRITE (*,*) ' '
      IF (CH.EQ.0) GOTO 100
      IF (CH.EQ.3) THEN
          FINE=.NOT.FINE
          GOTO 212
      ENDIF
      IF (CH.LT.1.OR.CH.GT.I) THEN
          WRITE (*,*) 'ILLEGAL SELECTION! TRY ONCE MORE. '
          GOTO 300
      ENDIF
      GOTO (311,312,212,313,314) CH
311   WRITE (*,*) 'Enter number of meshpoints : '
      READ  (*,*) MESHNU
      GOTO 212
312   WRITE (*,*) 'Enter fraction : '
      READ  (*,*) BFRAC
      GOTO 212
313   WRITE (*,*) 'Enter number of fine meshpoints : '
      READ  (*,*) FMESHN
      GOTO 212
314   WRITE (*,*) 'Enter fraction : '
      READ  (*,*) FBFRAC
      GOTO 212
 
213   WRITE (*,*) '**** SET WHICH BANDS TO CALCULATE ****'
      WRITE (*,'(/)')
      IF (MSH.GE.7) THEN
          IF (BANDSELECT(7)) WRITE(*,9967) 7
          IF (BANDSELECT(6)) WRITE(*,9967) 6
          IF (BANDSELECT(5)) WRITE(*,9967) 5
      ENDIF
      IF (MSH.GE.4) THEN
          IF (BANDSELECT(4)) WRITE(*,9966) 4
      ENDIF
      IF (BANDSELECT(3)) WRITE(*,9965) 3
      IF (BANDSELECT(2)) WRITE(*,9965) 2
      IF (BANDSELECT(1)) WRITE(*,9965) 1
      WRITE(*,'(''+'',A)') 'Currently results are stored for bands : '
      DO I=1,MSH
          WRITE (*,9994) 'Should results for band',I,'be stored ? (Y/N)'
          READ  (*,9998) SELECTOR
          BANDSELECT(I) = SELECTOR.EQ.'y'.OR.SELECTOR.EQ.'Y' 
      END DO
      GOTO 100
 
214   WRITE (*,*) '**** SET WHAT TO CALCULATE ****'
      WRITE (*,*) ' '
      IF (SDCALC) WRITE (*,9963) 'SEC. DERIV.' 
      IF (ENCALC) WRITE (*,9962) 'ENERGIES'
      IF (GRCALC) WRITE (*,9961) 'GRADIENTS'
      IF (VECALC) WRITE (*,9960) 'EIGENVECTORS'
      WRITE (*,'(''+'',A)') 'Current setting : '
      WRITE (*,*) ' '
      WRITE (*,*) 'Should energies be calculated ? (Y/N) :'
      READ  (*,9998) SELECTOR
      ENCALC= SELECTOR.EQ.'y'.OR.SELECTOR.EQ.'Y' 
      WRITE (*,*) 'Should gradients be calculated ? (Y/N) :'
      READ  (*,9998) SELECTOR
      GRCALC= SELECTOR.EQ.'y'.OR.SELECTOR.EQ.'Y' 
      WRITE (*,*) 'Should second derivatives be calculated ? (Y/N) :'
      READ  (*,9998) SELECTOR
      SDCALC= SELECTOR.EQ.'y'.OR.SELECTOR.EQ.'Y' 
      WRITE (*,*) 'Should eigenvectors be calculated ? (Y/N) :'
      READ  (*,9998) SELECTOR
      VECALC= SELECTOR.EQ.'y'.OR.SELECTOR.EQ.'Y' 
      GOTO 100
 
215   WRITE (*,*) '**** DEFINE FILENAMES ****'
      WRITE (*,9940) 'Filenames','Energies    ','Gradients   ', &
                                 'Sec. deriv. ','Eigenvectors'
      WRITE (*,9950) '1. Band 1:',ENNAME(1),GRNAME(1),SDNAME(1), &
                     VENAME(1)
      WRITE (*,9950) '2. Band 2:',ENNAME(2),GRNAME(2),SDNAME(2), &
                     VENAME(2)
      WRITE (*,9950) '3. Band 3:',ENNAME(3),GRNAME(3),SDNAME(3), &
                     VENAME(3)
      IF(MSH.GE.4) THEN
          WRITE (*,9950) '4. Band 4:',ENNAME(4),GRNAME(4),SDNAME(4), &
                         VENAME(4)
      ENDIF
      IF(MSH.GE.7) THEN
          WRITE (*,9950) '5. Band 5:',ENNAME(5),GRNAME(5),SDNAME(5), &
                         VENAME(5)
          WRITE (*,9950) '6. Band 6:',ENNAME(6),GRNAME(6),SDNAME(6), &
                         VENAME(6)
          WRITE (*,9950) '7. Band 7:',ENNAME(7),GRNAME(7),SDNAME(7), &
                         VENAME(7)
      ENDIF
      WRITE (*,*) ' '
      WRITE (*,*) 'WHICH DO YOU WANT TO CHANGE (0=NONE) ?'
350   READ  (*,*) I 
      IF (I.EQ.0) GOTO 100
      IF (I.GT.MSH) THEN
          WRITE (*,*) 'ILLEGAL SELECTION! TRY ONCE MORE. '
          GOTO 350
      ENDIF
      WRITE (*,9995) 'New filenames for band',I
      L=LAST(ENNAME(I))
      WRITE (*,*) 'Enter name of energy file ['//ENNAME(I)(1:L)//'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') ENNAME(I)=NAM
      L=LAST(GRNAME(I))
      WRITE (*,*) 'Enter name of gradient file ['//GRNAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') GRNAME(I)=NAM
      L=LAST(SDNAME(I))
      WRITE (*,*) 'Enter name of sec. derivative file ['//SDNAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') SDNAME(I)=NAM
      L=LAST(VENAME(I))
      WRITE (*,*) 'Enter name of eigenvector file ['//VENAME(I)(1:L) &
                    //'] : '
      READ  (*,9997) NAM
      IF (NAM.NE.' ') VENAME(I)=NAM
      WRITE (*,*) ' '
      GOTO 215

216   WRITE (*,*) '**** SHOW ALL DATA ****'
      WRITE (*,*) ' ' 
      WRITE (*,9910) 'Strain: ',EXX,EXY,EXZ,EXY,EYY,EYZ,EXZ,EYZ,EZZ
 
      WRITE (*,*) 'Coarse mesh:'
      WRITE (*,9920) '   Number of meshpoints : ',MESHNU
      WRITE (*,9930) '   Fraction of 2*PI/A0  : ',BFRAC
      WRITE (*,*) ' '
      WRITE (*,*) 'Fine mesh:'
      IF (FINE) THEN
          WRITE (*,*)    '   Fine mesh is generated.'
          WRITE (*,9920) '   Number of meshpoints : ',FMESHN
          WRITE (*,9930) '   Fraction of 2*PI/A0  : ',FBFRAC
      ELSE 
          WRITE (*,*) '   No fine mesh is generated.'
      ENDIF
 
      WRITE (*,'(/)')
      IF (MSH.GE.7) THEN
          IF (BANDSELECT(7)) WRITE(*,9967) 7
          IF (BANDSELECT(6)) WRITE(*,9967) 6
          IF (BANDSELECT(5)) WRITE(*,9967) 5
      ENDIF
      IF (MSH.GE.4) THEN
          IF (BANDSELECT(4)) WRITE(*,9966) 4
      ENDIF
      IF (BANDSELECT(3)) WRITE(*,9965) 3
      IF (BANDSELECT(2)) WRITE(*,9965) 2
      IF (BANDSELECT(1)) WRITE(*,9965) 1
      WRITE(*,'(''+'',A)') 'Results are stored for bands : '
      WRITE (*,*) ' '

      IF (SDCALC) WRITE (*,9963) 'SEC. DERIV.'
      IF (ENCALC) WRITE (*,9962) 'ENERGIES'
      IF (GRCALC) WRITE (*,9961) 'GRADIENTS'
      IF (VECALC) WRITE (*,9960) 'EIGENVECTORS'
      WRITE (*,'(''+'',A)') 'The program calculates : '
      WRITE (*,*) ' '

      WRITE (*,9940) 'Filenames:','Energies    ','Gradients   ', &
                                 'Sec. deriv. ','Eigenvectors'
      WRITE (*,9950) '   Band 1:',ENNAME(1),GRNAME(1),SDNAME(1), &
                     VENAME(1)
      WRITE (*,9950) '   Band 2:',ENNAME(2),GRNAME(2),SDNAME(2), &
                     VENAME(2)
      WRITE (*,9950) '   Band 3:',ENNAME(3),GRNAME(3),SDNAME(3), &
                     VENAME(3)
      IF (MSH.GE.4) THEN
          WRITE (*,9950) '   Band 4:',ENNAME(4),GRNAME(4),SDNAME(4), &
                         VENAME(4)
      ENDIF
      IF (MSH.GE.7) THEN
          WRITE (*,9950) '   Band 5:',ENNAME(5),GRNAME(5),SDNAME(5), &
                         VENAME(5)
          WRITE (*,9950) '   Band 6:',ENNAME(6),GRNAME(6),SDNAME(6), &
                         VENAME(6)
          WRITE (*,9950) '   Band 7:',ENNAME(7),GRNAME(7),SDNAME(7), &
                         VENAME(7)
      ENDIF
      WRITE (*,*) ' '

      WRITE (*,*) 'PRESS ENTER TO RETURN...'
      READ  (*,9998) SELECTOR
 
      GOTO 100
 
217   WRITE (*,*) '**** SAVE AND QUIT ****'
      OPEN(60,FILE='KPBAND.INI',ACCESS='DIRECT',FORM='UNFORMATTED',STATUS='OLD',RECL=496)
      WRITE (60,REC=1) EXX,EXY,EYY,EXZ,EYZ,EZZ,ENCALC,GRCALC,VECALC, &
                       SDCALC,ENNAME,GRNAME,VENAME,SDNAME,MESHNU,BFRAC, &
                       BANDSELECT,FINE,FBFRAC,FMESHN
      CLOSE(60)
      WRITE (*,*) ' '
      WRITE (*,*) 'Data written to file KPBAND.INI.'

9900  FORMAT(' ',I1,'. ',A)
9910  FORMAT (' ',A,3(T14,3(F10.7,' '),/))
9920  FORMAT (' ',A,I3)
9930  FORMAT (' ',A,F5.3)
9940  FORMAT (' ',A,/,T15,A12,T30,A12,T45,A12,T60,A12)
9950  FORMAT (' ',A,T15,A12,T30,A12,T45,A12,T60,A12)
9960  FORMAT('+',T28,A12) 
9961  FORMAT('+',T41,A12)
9962  FORMAT('+',T54,A12)
9963  FORMAT('+',T67,A12) 
9965  FORMAT('+',T44,I1) 
9966  FORMAT('+',T46,I1) 
9967  FORMAT('+',T48,I1)  
9998  FORMAT(A1)
9997  FORMAT(A12)
9995  FORMAT(' ',A,I1)
9994  FORMAT(' ',A,' ',I1,' ',A) 

      END
